#import "_PNArc.h"

@class PNToken;

/**
 General purpose parent class at the level of PNNet, PNTransition, PNPlace,  PNArc, PNArcP2T, PNArcT2P, PNToken.
 
 PNToken is the parent of PNArcP2T and PNArcT2P. PNToken implements a guard expression based on its uuids and targetTokenEntityName properties, in -(BOOL)satisfactoryToken:.
 */
@interface PNArc : _PNArc
{
}
/**
	A set of NSStrings (uuids in string form) for use in the arc's guard expression; nil blocks, an empty set acts as wildcard.
 */
@property (readwrite, strong) NSSet *uuids;

/**
	The name of an entity (PNToken or subclass) for use in the arc's guard expression, nil acts as a wildcard. 
 */
@property (readwrite, strong) NSString *targetTokenEntityName;

/**
 This method implements the receiver's guard expression. The guard expression tests a given token for the token's entity name and uuid. The nil value for the uuids property acts to veto--every token fails. An empty set of uuids acts as a wildcard. Otherwise, testToken's uuid must be in the receiver's uuids set. Also, the receiver's targetTokenEntityName must be nil (wildcard), or match the class (entity name) of testToken. 
 
 @param testToken the token to test
 @returns YES, if testToken passes the receiver's guard expression; NO otherwise.
 */
-(BOOL)satisfactoryToken: (PNToken*)testToken;

@end
