//
//  PNDocSupportNet+CloudStorageUsing.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/31/12.
//
//

#import "PNDocSupportNet+CloudStorageUsing.h"
#import "PNDocSupportNetController+TokenVending.h"
#import "PNDocSupportNetController+AppPreppingAndCloudUsing.h"

#import "PNToken.h"
#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"

#import "PNCloudURLToken.h"

#import "PNURLTokenPlace.h"

#import "NSURL+NPAssisting.h"

#import "PNURLTokensTransition.h"

#import "PNCloudUpdate.h"

#import <UIKit/UIKit.h>


@implementation PNDocSupportNet (CloudStorageUsing)
@dynamic query;

-(NSMetadataQuery*)query
{
    if( nil == m_query ){
        m_query = [[NSMetadataQuery alloc] init];
                
        // Query like Warren's MultiDocument:
        [m_query setSearchScopes:
         [NSArray arrayWithObjects:
          NSMetadataQueryUbiquitousDataScope, NSMetadataQueryUbiquitousDocumentsScope, nil]];
        
        // We cannot look for the folder--must look for the
        // contained DocumentMetadata.plist.
        NSPredicate *p =
        [NSPredicate predicateWithFormat:@"%K like %@",
         NSMetadataItemFSNameKey, PNDocMDataDotPlistKey];
        
        [m_query setPredicate:p];

    }
    return m_query;
}

-(void)ignoreQuery
{
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    
    for( id qObs in m_queryObservers ){
        [dfc removeObserver: qObs];
    }
    [m_queryObservers removeAllObjects];
}
/*
-(void)gatherQueryUpdateItems: (NSArray*)updateItems
{
//    
//     This method can deadlock if we call it on the main thread.
//    

    // The query reports all files found, every time.
    
    for (NSMetadataItem *item in m_query.results){
        NSURL *url = [item valueForAttribute: NSMetadataItemURLKey];

 
//        Expect a URL of the form:
//         
//         file://<base>/<midPath>/<uuid>/<localized name>/DocumentMetadata.plist
//         
//         For example:
//         file://localhost/private/var/mobile/Library/Mobile%20Documents/TEAMCODE10~com~mycompany~myapp/Documents/example/group/EA67D46F-C054-46FE-9714-B01B2B9CB1E4/foobar/DocumentMetadata.plist
 

        
        // [0] Must be an NSURL:
        NSAssert( [url isKindOfClass: [NSURL class]],
                 @"Not NSURL: %@", [url description] );
        // This is a show-stopper:
        //   throw an exception for the hapless adopting developer:
        //   all the items in query.results must be NSURLs.
        
        // [1] The url must presently exist:
        
        
        BOOL __block urlIsDir = NO;
        BOOL __block urlExists = NO;
        
        NSFileManager *fileMgr = [self threadSafeFileManager];
        
        NSFileCoordinator *fileCoordinator =
        [[NSFileCoordinator alloc] initWithFilePresenter:nil];
        
        [fileCoordinator coordinateReadingItemAtURL:url
                                            options:NSFileCoordinatorReadingWithoutChanges
                                              error:nil
                                         byAccessor:^(NSURL *coordinatedURL) {
                                             
                                             urlExists = [fileMgr fileExistsAtPath: [url path]
                                                                       isDirectory: (BOOL*)&urlIsDir];
                                         }];
        
        
        // [2] Must NOT be hidden:
        NSNumber *hiddenNumber = nil;
        [url getResourceValue:&hiddenNumber
                       forKey:NSURLIsHiddenKey
                        error:nil];
        
        BOOL notHidden = hiddenNumber && ![hiddenNumber boolValue];
        
        // [3] The file name is DocumentMetadata.plist:
        BOOL gotDocumentMetadataDotPlist =
        [[url lastPathComponent] isEqualToString: @"DocumentMetadata.plist"];
        
        // [4] Extract the UUID string:
        //     We're expecting a URL in this form:
        //     /<cloudContainer>/<maybe group relative?>/<uuid>/<localized name>/DocumentMetadata.plist
        NSURL *findUuid = [url copy];
        findUuid = [findUuid URLByDeletingLastPathComponent]; // delete DocumentMetadata.plist
        findUuid = [findUuid URLByDeletingLastPathComponent]; // delete <localized name>
        NSString *uuidTest = [findUuid lastPathComponent]; // should be the <uuid> component
        
        BOOL gotUUID =
        [PNURLToken validUuid: uuidTest];
        //     If its UUID is valid, continue:
        
        if( urlExists && notHidden && gotUUID && gotDocumentMetadataDotPlist ){
            
            NSDictionary *tokenProperties = [PNURLToken propertiesFromFilePackageURL: url];
            
            enum PNUuidStatus status = [self statusOfUuid: uuidTest];
            
            NSManagedObjectContext *moc = self.managedObjectContext;
            switch( status ){
                case PNUuidStatusUnassignedLocalURL:
                {
                    // This case is unexpected, but allowable.
                    // If the associated local URL token exists; subsume it:
                    PNURLToken *localToken =
                    [self.unassignedLocalPlace tokenForUuid: uuidTest];
                    
                    [self.unassignedLocalPlace removeToken: localToken];
                    
                    // DO NOT delete localToken.filePackageURL
                    // e.g., by calling
                    //     [PNURLToken deleteURLsAndURLToken:error:]
                    // or
                    //     [localToken removeURLsWithError:] .
                    // Delete the token only:
                    [PNURLToken promote: localToken
                                  error: nil];
                    
                    // fall through, don't break:
                }
                case PNUuidStatusUnknown:
                {
                    // This is a common case:
                    // [1] existing local document
                    // [2] move it to iCloud:
                    //     [2a] close the document
                    //     [2b] promote its url token (deletes local url token)
                    //     [2c] establish the document
                    // [3] the cloud document's DocumentMetadata.plist appears in the next query.
                    
                    PNCloudURLToken *token = nil;
                    
                    token = [PNCloudURLToken insertCloudURLTokenWithProperties: tokenProperties
                                                        inManagedObjectContext: moc];
                    token.metadataItem = item;
                    
                    NSAssert( (nil != token), @" got a nil token. that's bad. can't work.");
                    
                    [self.unassignedCloudPlace addToken: token];
                    
                    break;
                }
                case PNUuidStatusUnassignedCloudURL:
                {
                    // If the uuid belongs to an existing available cloud token,
                    // do nothing.
                    
                    // This is a common case.
                    break;
                }
                case PNUuidStatusAssignedCloudURL:
                {
                    // If the uuid belongs to an existing assigned cloud token,
                    // do nothing.
                    
                    // This is a common case.
                    break;
                }
                    
                case PNUuidStatusAssignedLocalURL:
                {
                    // This case is unexpected.
                    // The query tells us this uuid is cloud, ground truth disagrees.
                    
                    NSAssert( (PNUuidStatusAssignedLocalURL!= status),
                             @"Status is assigned and local, but the query reports a cloud document.");
                    
                    break;
                }
                case PNUuidStatusInvalid: // 0 == PNUuidStatusInvalid
                {
                    NSAssert( (PNUuidStatusInvalid!= status),
                             @"Bogus status: PNUuidStatusInvalid");
                    break;
                }
                    
                default:
                {
                    NSAssert( NO, @"Unexpected value of status: %d", status);
                    
                    break;
                }
            } // end switch (status )
            
        } // end if (gotUUID)
    } // end for loop over items
}
 */
-(NSSet*)validUpdateItems: (NSArray*)metadataItems
{
    return nil;
}
-(NSSet*)itemsForExistingCloudTokensInMetadataItems: (NSSet*)items
{
    return nil;
}
-(NSSet*)itemsForExistingLocalTokensInMetadataItems: (NSSet*)items
{
    return nil;
}

-(void)handleQueryUpdate
{
    [self.query disableUpdates];{
        
        // [1] assure that each update item is of the form:
        //     <container>/<midpath>/<uuid>/<name>/DocumentMetadata.plist
        //     For example:
        // file://localhost/private/var/mobile/Library/Mobile%20Documents/TEAMCODE10~com~mycompany~myapp/Documents/example/group/EA67D46F-C054-46FE-9714-B01B2B9CB1E4/foobar/DocumentMetadata.plist

        NSSet *validUpdateItems = [self validUpdateItems: self.query.results];
        
        NSMutableSet *remainingItems = [validUpdateItems mutableCopy];
        
        NSSet *existing = [self itemsForExistingCloudTokensInMetadataItems: remainingItems];
        [remainingItems minusSet: existing];
        
        NSSet *promotions = [self itemsForExistingLocalTokensInMetadataItems: remainingItems];
        [remainingItems minusSet: promotions];
        
        NSSet *discoveredItems = remainingItems;
        
        [self.cloudUpate enableFor: self.]
        
        
        
    }[self.query enableUpdates];
    
}

-(void)observeQueryNotification: (NSString*)notificationName
{
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    id observer =
    [dfc addObserverForName: notificationName
                     object: self.query
                      queue: [NSOperationQueue mainQueue]
                 usingBlock:^(NSNotification __strong *notification)
     {
         NSArray *updateItems = self.query.results;
         
         [self gatherQueryUpdateItems: updateItems];

     }];
    [m_queryObservers addObject: observer];
    
}


-(void)cleanCloudContainer
{
    
    // Clean up +[PNCloudURLToken containerUrl].
    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    NSFileCoordinator* coordinator =
    [[NSFileCoordinator alloc] initWithFilePresenter:nil];
    [coordinator coordinateWritingItemAtURL:[PNCloudURLToken containerUrl]
                                    options:NSFileCoordinatorWritingForDeleting
                                      error:nil
                                 byAccessor:^(NSURL *coordinatedURL) {
                                     
                                     [fileMngr removeItemAtURL:coordinatedURL
                                                         error:nil];
                                     
                                 }];
    
    NSSet *unassignedCloud = [self.unassignedCloudPlace tokens];
    for( PNCloudURLToken *next in unassignedCloud ){
        [self.unassignedCloudPlace removeToken: next];
        [PNCloudURLToken deleteURLsAndURLToken: next
                                         error: nil];
    }
    NSSet *assignedCloud = [self.unassignedCloudPlace tokens];
    for( PNCloudURLToken *next in assignedCloud ){
        NSAssert( NO, @"There should not be any assigned cloud tokens.");
        [self.assignedCloudPlace removeToken: next];
        [PNCloudURLToken deleteURLsAndURLToken: next
                                         error: nil];
    }
    
}
- (void)startMonitoringUbiquitousDocumentsFolder
{
    /*
     See: http://stackoverflow.com/questions/9026242/nsmetadataquery-ignoring-custom-file-package-type
     
     Basically, NSMetadataQuery can't deal with file extensions.
     */
    
    [self stopMonitoringUbiquitousDocumentsFolder];{
        
        [self observeQueryNotification: NSMetadataQueryDidStartGatheringNotification];
        [self observeQueryNotification: NSMetadataQueryGatheringProgressNotification];
        [self observeQueryNotification: NSMetadataQueryDidUpdateNotification];
        [self observeQueryNotification: NSMetadataQueryDidFinishGatheringNotification];

    }[self.query startQuery];
    
}
- (void) stopMonitoringUbiquitousDocumentsFolder
{
    // called from -dealloc
    [self.query stopQuery];
    
    [self ignoreQuery];
    
    m_query = nil;
}

/*
-(void)resynchCloudResources
{
    BOOL x = [[PNDocSupportNetController sharedDocSupportNetController] usingCloudStorage];
    NSAssert( x, @"PNDocSupportNetController not using Cloud storage.");
    
    NSSet *assignedCloudURLTokens = [self.assignedCloudPlace tokens];
    NSAssert( (0==assignedCloudURLTokens.count),
             @"Can't resynch cloud resources now; there's at least one open cloud document.");
    
    [self stopMonitoringUbiquitousDocumentsFolder];{
        
        // [1] Delete any existing cloud url tokens
        //     Each has a file package and a log file.
        NSSet *unassignedCloud = [self.unassignedCloudPlace tokens];
        for( PNCloudURLToken *next in unassignedCloud ){
            [self.unassignedCloudPlace removeToken: next];
            [PNCloudURLToken deleteURLsAndURLToken: next
                                             error: nil];

        // [2] Clean out any other files or directories in the cloud documents container.
        NSFileManager *fileMngr =
        [PNDocSupportNetController threadSafeFileManager];
        
        NSFileCoordinator* coordinator =
        [[NSFileCoordinator alloc] initWithFilePresenter:nil];
        [coordinator coordinateWritingItemAtURL:[PNCloudURLToken containerUrl]
                                        options:NSFileCoordinatorWritingForDeleting
                                          error:nil
                                     byAccessor:^(NSURL *dirURL) {
                                                                                  
                                         NSArray *dirContents =
                                         [fileMngr contentsOfDirectoryAtURL: dirURL
                                                 includingPropertiesForKeys: nil
                                                                    options: nil
                                                                      error: nil];
                                     
                                         for( NSURL *next in dirContents ){
                                             [fileMngr removeItemAtURL:next
                                                                 error:nil];
                                         }
                                     }];
        
        }
        
    }[self startMonitoringUbiquitousDocumentsFolder];
    
    
}
*/
@end
