#import "_PNDocSupportNet.h"

// #import <PNDocSupport/PNUuidStatus.h>

@class PNURLTokenPlace, PNURLToken;

/**
 
 PNDocSupportNet is a Petri Net comprising the places and transitions we need to manage cloud-capable UIManagedDocuments. The places hold tokens; the tokens wrap resources. The transitions wrap operations on the resources.
 
 PNDocSupportNet owns four primary PNURLTokenPlaces: one for each value of PNUuidStatus other than PNUuidStatusInvalid and PNUuidStatusUnknown.
 
 PNDocSupportNet also owns transitions that wrap operations on the url tokens.
 
 PNDocSupportNet instantiates its places and transitions lazily, and refers to them by their permanentIDs. See NSManagedObjectContext (PNPermanentIDing).
 */

@class PNURLTokenPlace, PNURLTokensTransition, PNDocTokenTransition;
@class PNDeleteUUIDs;
@class PNSyncResources;
@class PNQueryUpdate, PNFlipStorage;
@class PNCreateDoc, PNOpenDoc, PNCloseDoc, PNRenameDoc;
@class PNCloseOpen, PNCloseCreate, PNCloseFlipOpen;

@interface PNDocSupportNet : _PNDocSupportNet
{
    NSFileManager *m_threadSafeFileManager;
    

    NSURL *m_permID_unassignedLocalPlace;
    NSURL *m_permID_unassignedCloudPlace;
    NSURL *m_permID_assignedLocalPlace;
    NSURL *m_permID_assignedCloudPlace;
    
    // Hard-wired URL Token Transitions:
    NSURL *m_permID_syncResources;
    NSURL *m_permID_deleteUUIDs;
    NSURL *m_permID_cloudUpdate;
    NSURL *m_permID_flipStorage;
    
    // Doc Token Transitions: Hard-wired in & out, Transient out:
    NSURL *m_permID_createDoc;
    NSURL *m_permID_openDoc;
    
    // Doc Token Transitions: Transient in, Hard-wired out:
    NSURL *m_permID_closeDoc;
    
    NSURL *m_permID_renameDoc;
    
    // Doc Token Transitions: Transient in, Transient out:
    NSURL *m_permID_closeCreateDoc;
    NSURL *m_permID_closeOpenDoc;
    //NSURL *m_permID_closeMigrateOpenDoc;
    NSURL *m_permID_closeFlipOpenDoc;
    
}
// Custom logic goes here.
@property (readonly, strong) NSFileManager *threadSafeFileManager;
@property (readonly, strong) NSMutableArray *notificationObservers;


// Places:
@property (readonly, strong) PNURLTokenPlace *unassignedLocalPlace;
@property (readonly, strong) PNURLTokenPlace *unassignedCloudPlace;
@property (readonly, strong) PNURLTokenPlace *assignedLocalPlace;
@property (readonly, strong) PNURLTokenPlace *assignedCloudPlace;

#pragma mark URL Token Transitions:
@property (readonly, strong) PNSyncResources *syncResources;
@property (readonly, strong) PNDeleteUUIDs *deleteUUIDs;
@property (readonly, strong) PNQueryUpdate *queryUpdate;
@property (readonly, strong) PNFlipStorage *flipStorage;

#pragma mark Doc Token Transitions:
@property (readonly, strong) PNCreateDoc *createDoc;
@property (readonly, strong) PNOpenDoc *openDoc;
@property (readonly, strong) PNCloseDoc *closeDoc;
@property (readonly, strong) PNRenameDoc *renameDoc;
@property (readonly, strong) PNCloseCreate *closeCreateDoc;
@property (readonly, strong) PNCloseOpen *closeOpenDoc;
@property (readonly, strong) PNCloseFlipOpen *closeFlipOpenDoc;

-(NSString*)quickLook;

+(NSFileManager*)threadSafeFileManager;
-(NSFileManager*)threadSafeFileManager;

-(enum PNUuidStatus)statusOfUuid: (NSString*)uuid;

-(PNURLToken*)urlTokenForUuid: (NSString*)uuid;

@end
