#import "PNDocSupportNet.h"

#import "NSManagedObject+PNPermanentIDing.h"

#import "PNURLToken.h"
#import "PNCloudURLToken.h"
#import "PNURLTokenPlace.h"

#import "PNSyncResources.h"
#import "PNDeleteUUIDs.h"
#import "PNQueryUpdate.h"
#import "PNFlipStorage.h"

#import "PNCreateDoc.h"
#import "PNOpenDoc.h"
#import "PNCloseDoc.h"
#import "PNRenameDoc.h"

#import "PNCloseCreate.h"
#import "PNCloseOpen.h"
#import "PNCloseFlipOpen.h"

#import "PNDocSupportNetController.h"

//@interface PNURLTokenPlace ()
//@property (readwrite, assign ) enum PNURLDeploymentCase ;
//@end
//@implementation PNURLTokenPlace ()
//@dynamic 
//@end
@interface PNDocSupportNet ()
// Doc Token Transitions: Transient in, Transient out:
@end

@implementation PNDocSupportNet
// Custom logic goes here.


@dynamic threadSafeFileManager;
@dynamic notificationObservers;

// Places:
@dynamic unassignedLocalPlace; 
@dynamic unassignedCloudPlace; 
@dynamic assignedLocalPlace; 
@dynamic assignedCloudPlace; 

// Transitions:
// Hard-wired URL Token Transitions:
@dynamic syncResources;
@dynamic deleteUUIDs;
@dynamic queryUpdate;
@dynamic flipStorage;

// Doc Token Transitions: Hard-wired in & out, Transient out:
@dynamic createDoc; 
@dynamic openDoc; 

// Doc Token Transitions: Transient in, Hard-wired in & out:
@dynamic closeDoc; // = m_closeDoc;

// Doc Token Transitions: Transient in, Transient out:
@dynamic closeCreateDoc; 
@dynamic closeOpenDoc;
@dynamic closeFlipOpenDoc;

-(NSString*)quickLookOLD
{
    NSMutableString *result = [NSMutableString stringWithString: @"PNDocSupportNet URL Token Places\n"];
    [result appendFormat: @"%@\n", [self.unassignedLocalPlace quickLook]];
    [result appendFormat: @"%@\n", [self.unassignedCloudPlace quickLook]];
    [result appendFormat: @"%@\n", [self.assignedLocalPlace   quickLook]];
    [result appendFormat: @"%@\n", [self.assignedCloudPlace   quickLook]];
    return result;
}
-(NSString*)quickLook
{
    // [1] name
    NSMutableString *result = [NSMutableString stringWithString: @"PNDocSupportNet:\n"];
        
    // [2] Places:
    [result appendString: @"places:\n"];
    for( PNPlace *nextPlace in self.places ){
        [result appendFormat: @"%@\n",
         [nextPlace quickLook]];
    }
    
    // [3] Transitions:
    [result appendString: @"transitions:\n"];
    for( PNTransition *nextTransition in self.transitions ){
        [result appendFormat: @"%@\n",
         [nextTransition quickLook]];
    }
    
    return result;
}

+(NSFileManager*)threadSafeFileManager
{
//    static NSFileManager *mm_threadSafeFileManager = nil;
//    
//    if( nil == mm_threadSafeFileManager ){
//        /*
//         Threading Considerations
//         The methods of the shared NSFileManager object can be called from multiple threads safely. However, if you use a delegate to receive notifications about the status of move, copy, remove, and link operations, you should create a unique instance of the file manager object, assign your delegate to that object, and use that file manager to initiate your operations.
//         */
//        mm_threadSafeFileManager = [[NSFileManager alloc] init];
//    }
//    return mm_threadSafeFileManager;
    
    return [PNDocSupportNetController threadSafeFileManager];
}
-(NSFileManager*)threadSafeFileManager
{
    return [PNDocSupportNetController threadSafeFileManager];    
}

-(enum PNUuidStatus)statusOfUuid: (NSString*)uuid
{
    // Return PNUuidStatusInvalid for nil:
    if( nil == uuid ) return PNUuidStatusInvalid;
    
    // Return PNUuidStatusInvalid if.uuid isn't a UUID.
    if( ![PNURLToken validUuid: uuid] ) return PNUuidStatusInvalid;
    
    // Otherwise build the result.
    // OR in a bit for every place that has the UUID:
    enum PNUuidStatus result = PNUuidStatusInvalid; // PNUuidStatusInvalid == 0x0.
    
    if( nil != [self.unassignedLocalPlace tokenForUuid: uuid] ){
        result = result | PNUuidStatusUnassignedLocalURL;
    }
    if( nil != [self.unassignedCloudPlace tokenForUuid: uuid] ){
        result = result | PNUuidStatusUnassignedCloudURL;
    }
    if( nil != [self.assignedLocalPlace tokenForUuid: uuid] ){
        result = result | PNUuidStatusAssignedLocalURL;
    }
    if( nil != [self.assignedCloudPlace tokenForUuid: uuid] ){
        result = result | PNUuidStatusAssignedCloudURL;
    }
   
    // But, if we found the UUID nowhere, set the result to PNUuidStatusUnknown.
    if( PNUuidStatusInvalid == result ) result = PNUuidStatusUnknown;
    
    // Sanity check:
    // The result should have exactly one true bit.
    //
    switch( result ){
        case PNUuidStatusUnknown:
        case PNUuidStatusUnassignedLocalURL:
        case PNUuidStatusUnassignedCloudURL:
        case PNUuidStatusAssignedLocalURL:
        case PNUuidStatusAssignedCloudURL:
        {
            break;
        }
        default:
        {
            NSAssert( NO, @"Sanity check on uuid FAILED.");
        }
    }
    return result;
}
//-(void)activate
//{
//    
//}
//-(void)deactivate
//{
//    
//}
-(PNURLToken*)urlTokenForUuid: (NSString*)uuid
{
    PNURLToken *result = nil;
    enum PNUuidStatus status = [self statusOfUuid: uuid];
    switch( status ){
        case PNUuidStatusAssignedLocalURL:
        {
            result = [self.assignedLocalPlace tokenForUuid: uuid];
            break;
        }
        case PNUuidStatusAssignedCloudURL:
        {
            result = [self.assignedCloudPlace tokenForUuid: uuid];
            break;
        }
       case PNUuidStatusUnassignedCloudURL:
        {
            result = [self.unassignedCloudPlace tokenForUuid: uuid];
            break;
        }
        case PNUuidStatusUnassignedLocalURL:
        {
            result = [self.unassignedLocalPlace tokenForUuid: uuid];
            break;
        }
        case PNUuidStatusInvalid:
        case PNUuidStatusUnknown:
        default:
        {
            result = nil; // redundant but clear
            break;
        }
    }
    return result;
}

#pragma mark Indirect Accessors for the four main URLtoken places: {local,cloud}x{unassigned,assigned}
-(PNURLTokenPlace*)unassignedLocalPlace
{
    PNURLTokenPlace *p = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_unassignedLocalPlace ){
        p = [PNURLTokenPlace insertInManagedObjectContext: moc];
        {
            p.name = @"unassignedLocalPlace";
            p.capacityValue = 0; // unlimited
            p.uuidStatus = PNUuidStatusUnassignedLocalURL;
            
            [self addPlace: p];
        }
        [moc processPendingChanges];
        m_permID_unassignedLocalPlace = [p permanentID];
    }
    p = (PNURLTokenPlace*)[moc managedObjectForPermanentID: m_permID_unassignedLocalPlace];
    return p;
}
-(PNURLTokenPlace*)unassignedCloudPlace
{
    PNURLTokenPlace *p = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_unassignedCloudPlace ){
        p = [PNURLTokenPlace insertInManagedObjectContext: self.managedObjectContext];
        {
            p.name = @"unassignedCloudPlace";
            p.capacityValue = 0; // unlimited
            p.uuidStatus = PNUuidStatusUnassignedCloudURL;
            
            [self addPlace: p];
        }
        [moc processPendingChanges];
        m_permID_unassignedCloudPlace = [p permanentID];
    }
    p = (PNURLTokenPlace*)[moc managedObjectForPermanentID: m_permID_unassignedCloudPlace];
    return p;
}
-(PNURLTokenPlace*)assignedLocalPlace
{
    PNURLTokenPlace *p = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_assignedLocalPlace ){
        p = [PNURLTokenPlace insertInManagedObjectContext: moc];
        {
            p.name = @"assignedLocalPlace";
            p.capacityValue = 0; // unlimited
            p.uuidStatus = PNUuidStatusAssignedLocalURL;
            
            [self addPlace: p];
        }
        [moc processPendingChanges];
        m_permID_assignedLocalPlace = [p permanentID];
    }
    p = (PNURLTokenPlace*)[moc managedObjectForPermanentID: m_permID_assignedLocalPlace];
    return p;
}
-(PNURLTokenPlace*)assignedCloudPlace
{
    PNURLTokenPlace *p = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_assignedCloudPlace ){
        p = [PNURLTokenPlace insertInManagedObjectContext: moc];
        {
            p.name = @"assignedCloudPlace";
            p.capacityValue = 0; // unlimited
            p.uuidStatus = PNUuidStatusAssignedCloudURL;
            
            [self addPlace: p];
        }
        [moc processPendingChanges];
        m_permID_assignedCloudPlace = [p permanentID];
    }
    p = (PNURLTokenPlace*)[moc managedObjectForPermanentID: m_permID_assignedCloudPlace];
    return p;
}

#pragma mark Indirect Accessors for the four Hard-wired URL Token Transitions: deleteUUIDs, (later: flip, toLocal, toCloud)

-(PNSyncResources*)syncResources
{
    PNSyncResources *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_syncResources ){
        t = [PNSyncResources insertInManagedObjectContext: moc];
        {
            t.name = @"syncResources";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_syncResources = [t permanentID];
    }
    t = (PNSyncResources*)[moc managedObjectForPermanentID: m_permID_syncResources];
    return t;
}

-(PNDeleteUUIDs*)deleteUUIDs
{
    PNDeleteUUIDs *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_deleteUUIDs ){
        t = [PNDeleteUUIDs insertInManagedObjectContext: moc];
        {
            t.name = @"deleteUUIDs";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_deleteUUIDs = [t permanentID];
    }
    t = (PNDeleteUUIDs*)[moc managedObjectForPermanentID: m_permID_deleteUUIDs];
    return t;
}

-(PNQueryUpdate*)queryUpdate
{
    PNQueryUpdate *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_cloudUpdate ){
        t = [PNQueryUpdate insertInManagedObjectContext: moc];
        {
            t.name = @"queryUpdate";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_cloudUpdate = [t permanentID];
    }
    t = (PNQueryUpdate*)[moc managedObjectForPermanentID: m_permID_cloudUpdate];
    return t;
}
-(PNFlipStorage*)flipStorage
{
    PNFlipStorage *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_flipStorage ){
        t = [PNFlipStorage insertInManagedObjectContext: moc];
        {
            t.name = @"flipStorage";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_flipStorage = [t permanentID];
    }
    t = (PNFlipStorage*)[moc managedObjectForPermanentID: m_permID_flipStorage];
    return t;
}

#pragma mark Indirect Accessors for the four Hard-wired URL Token Transitions: new, open, close
-(PNCreateDoc*)createDoc
{
    PNCreateDoc *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_createDoc ){
        t = [PNCreateDoc insertInManagedObjectContext: moc];
        {
            t.name = @"createDoc";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_createDoc = [t permanentID];
    }
    t = (PNCreateDoc*)[moc managedObjectForPermanentID: m_permID_createDoc];
    return t;
}

-(PNOpenDoc*)openDoc
{
    PNOpenDoc *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_openDoc ){
        t = [PNOpenDoc insertInManagedObjectContext: moc];
        {
            t.name = @"openDoc";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_openDoc = [t permanentID];
    }
    t = (PNOpenDoc*)[moc managedObjectForPermanentID: m_permID_openDoc];
    return t;
}


-(PNCloseDoc*)closeDoc
{
    PNCloseDoc *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_closeDoc ){
        t = [PNCloseDoc insertInManagedObjectContext: moc];
        {
            t.name = @"closeDoc";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_closeDoc = [t permanentID];
    }
    t = (PNCloseDoc*)[moc managedObjectForPermanentID: m_permID_closeDoc];
    return t;
}
-(PNRenameDoc*)renameDoc
{
    PNRenameDoc *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_renameDoc ){
        t = [PNRenameDoc insertInManagedObjectContext: moc];
        {
            t.name = @"renameDoc";
            [self addTransition: t];
            [t connectFixedArcs];
        }
        [moc processPendingChanges];
        m_permID_renameDoc = [t permanentID];
    }
    t = (PNRenameDoc*)[moc managedObjectForPermanentID: m_permID_renameDoc];
    return t;
}

#pragma mark Indirect Accessors for the four Pipelined Transitions: closeCreateDoc, closeOpenDoc, closeMigrateOpenDoc, closeFlipOpenDoc

-(PNCloseCreate*)closeCreateDoc
{
    PNCloseCreate *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_closeCreateDoc ){
        t = [PNCloseCreate insertInManagedObjectContext: moc];
        {
            t.name = @"closeCreateDoc";
            [self addTransition: t];
            [t buildPipeline];
        }
        [moc processPendingChanges];
        m_permID_closeCreateDoc = [t permanentID];
    }
    t = (PNCloseCreate*)[moc managedObjectForPermanentID: m_permID_closeCreateDoc];
    return t;
}
-(PNCloseOpen*)closeOpenDoc
{
    PNCloseOpen *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_closeOpenDoc ){
        t = [PNCloseOpen insertInManagedObjectContext: moc];
        {
            t.name = @"closeOpenDoc";
            [self addTransition: t];
            [t buildPipeline];
        }
        [moc processPendingChanges];
        m_permID_closeOpenDoc = [t permanentID];
    }
    t = (PNCloseOpen*)[moc managedObjectForPermanentID: m_permID_closeOpenDoc];
    return t;
}
-(PNCloseFlipOpen*)closeFlipOpenDoc
{
    PNCloseFlipOpen *t = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_closeFlipOpenDoc ){
        t = [PNCloseFlipOpen insertInManagedObjectContext: moc];
        {
            t.name = @"closeFlipOpen";
            [self addTransition: t];
            [t buildPipeline];
        }
        [moc processPendingChanges];
        m_permID_closeFlipOpenDoc = [t permanentID];
    }
    t = (PNCloseFlipOpen*)[moc managedObjectForPermanentID: m_permID_closeFlipOpenDoc];
    return t;
}

@end
