#import "_PNNet.h"

extern NSString* PNNetWillChangeMarkings;
extern NSString* PNNetDidChangeMarkings;

/**
 General purpose parent class at the level of PNNet, PNTransition, PNPlace,  PNArc, PNArcP2T, PNArcT2P, PNToken.
 A Petri Net comprises transitions and places connected by arcs.
 
 PNNet manages its firing sequence in a way peculiar to this static library. In this implementation, a transition first acquires its input tokens, then PNNet enqueues in its operation queue an NSInvocation to fire the transition. It's ... um ... informal; the Petri Net formalism suggests a more general firing sequence. 
 
 But, "in theory, theory and practice are the same; in practice, they're different."
 */
@interface PNNet : _PNNet
{
    // The firing sequence:
    NSOperationQueue *m_sequenceQueue;
    // This sequence is not "fair" in the formal sense.
    // It's a first-in/first-out queue.
    
}
// Custom logic goes here.

#pragma mark PNNet assures that transitions and places have distinct names as they are added.
-(PNPlace*)placeForName: (NSString*)placeName;
-(PNTransition*)transitionForName: (NSString*)transitionName;

-(void)addPlace: (PNPlace*)aPlace;
-(void)addTransition: (PNTransition*)aTransition;

-(void)addToFiringSequence: (PNTransition*)aTransition;
-(void)willFireTransition: (PNTransition*)aTransition;
-(void)didFireTransition: (PNTransition*)aTransition
            successfully: (BOOL)success;

@end

