//
//  PNDocSupportNetController+CoreDataManaging.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/6/12.
//
//

#import "PNDocSupportNetController+CoreDataManaging.h"


#import "NSManagedObject+PNPermanentIDing.h"

@implementation PNDocSupportNetController (CoreDataManaging)

@dynamic managedObjectContext; // = _managedObjectContext;
@dynamic managedObjectModel; // = _managedObjectModel;
@dynamic persistentStoreCoordinator; // = _persistentStoreCoordinator;


-(void)buildCoreDataStack
{
    // The call to -saveAnyChanges lazily instantiates _managedObjectContext,
    // which lazily instantiates _persistentStoreCoordinator,
    // which lazily instantiates _managedObjectModel
    [self.managedObjectContext saveAnyChanges];
}

#pragma mark - Core Data stack

// Returns the managed object context for the application.
// If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
- (NSManagedObjectContext *)managedObjectContext
{
    if (nil == _managedObjectContext) {
        
        NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
        if (coordinator != nil) {
            _managedObjectContext =
            [[NSManagedObjectContext alloc] initWithConcurrencyType: NSMainQueueConcurrencyType];
            
            [_managedObjectContext setPersistentStoreCoordinator:coordinator];
        }
        _managedObjectContext.retainsRegisteredObjects = YES;
    }
    return _managedObjectContext;
}

// Returns the managed object model for the application.
// If the model doesn't already exist, it is created from the application's model.
- (NSManagedObjectModel *)managedObjectModel
{
    if (nil == _managedObjectModel) {
        
        NSBundle *staticLibBundle = [NSBundle bundleForClass: [self class]];
        
        NSURL *modelURL = [staticLibBundle URLForResource:@"PNDocSupportModel"
                                            withExtension:@"momd"];
        _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    }
    return _managedObjectModel;
}

// Returns the persistent store coordinator for the application.
// If the coordinator doesn't already exist, it is created and the application's store added to it.
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (nil == _persistentStoreCoordinator) {
         
        /**
         The method -persistentStoreCoordinator builds a store of type NSSQLiteStoreType.
         Perhaps later we can change this to NSInMemoryStoreType.
         But, NSInMemoryStoreType precludes fetches, it seems.
         So, that optimization is premature as of 2012 Nov 06, initial implementation.
         */
        
        NSURL *storeURL = [_persistentStoreDirectory URLByAppendingPathComponent:@"PNDocSupportCoreData.sqlite"];
        
        NSError *error = nil;
        _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:self.managedObjectModel];
        if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType // NSInMemoryStoreType?
                                                       configuration:nil
                                                                 URL:storeURL
                                                             options:nil
                                                               error:&error]) {
            /*
             Replace this implementation with code to handle the error appropriately.
             
             abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
             
             Typical reasons for an error here include:
             * The persistent store is not accessible;
             * The schema for the persistent store is incompatible with current managed object model.
             Check the error message to determine what the actual problem was.
             
             
             If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
             
             If you encounter schema incompatibility errors during development, you can reduce their frequency by:
             * Simply deleting the existing store:
             [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
             
             * Performing automatic lightweight migration by passing the following dictionary as the options parameter:
             @{NSMigratePersistentStoresAutomaticallyOption:@YES, NSInferMappingModelAutomaticallyOption:@YES}
             
             Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
             
             */
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        }
    }
    return _persistentStoreCoordinator;
}
@end
