//
//  PNDocSupportNetController+GreatBbugReporting.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 4/9/13.
//
//

#import "PNDocSupportNetController+GreatBugReporting.h"
#import "PNDocSupportNetController+StorageControlling.h"

#import "PNURLToken.h"

@implementation PNDocSupportNetController (GreatBugReporting)

+(void)copyContainerToSandbox
{
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    
    enum PNStorageStatus status = dsnc.storageStatus;
    
    if( PNStorageStatusCloudAccountAvailable != status) return;
    
    NSString *path = [dsnc.cloudContainer path]; // the cloud container
    
    NSURL *appDocumentsDir = [PNURLToken containerUrl]; // the sandbox container
    // e.g.,
    // file://localhost/var/mobile/Applications/96AB74BA-3380-4CDD-BC6D-2EF7E2EF340B/Documents/

    NSURL *target = [appDocumentsDir URLByAppendingPathComponent: @"copyOfCloudContainer"];
    
    target = [target URLByAppendingPathComponent: [dsnc.cloudContainer lastPathComponent]];
    // e.g.,
    // file://localhost/var/mobile/Applications/96AB74BA-3380-4CDD-BC6D-2EF7E2EF340B/Documents/copyOfCloudContainer/TEAMCODE10~com~myCompany~TaggedLocationsDemo/
    
    
    NSString *targetPath = target.path;
    // e.g.,
    // /var/mobile/Applications/96AB74BA-3380-4CDD-BC6D-2EF7E2EF340B/Documents/copyOfCloudContainer/TEAMCODE10~com~myCompany~TaggedLocationsDemo

    NSFileManager *fm = [[NSFileManager alloc] init];
    NSError *error = nil;
    if ([fm createDirectoryAtPath:targetPath
      withIntermediateDirectories:YES
                       attributes:nil
                            error:&error]) {
        NSLog(@"Created container directory in sandbox: %@", targetPath);
    } else {
        if ([[error domain] isEqualToString:NSCocoaErrorDomain]) {
            if ([error code] == NSFileWriteFileExistsError) {
                //delete the existing directory
                error = nil;
                if ([fm removeItemAtPath:targetPath
                                   error:&error]) {
                    NSLog(@"Removed old sandbox copy of cloud container ");
                } else {
                    NSLog(@"Error trying to remove old sandbox copy of cloud container: %@", error);
                }
            }
        } else {
            NSLog(@"Error attempting to create sandbox copy od cloud container: %@", error);
            return;
        }
    }
    
    NSArray *subPaths = [fm subpathsAtPath:path];
    for (NSString *subPath in subPaths) {
        NSString *fullPath = [NSString stringWithFormat:@"%@/%@", path, subPath];
        NSString *fullSandboxPath = [NSString stringWithFormat:@"%@/%@", targetPath, subPath];
        BOOL isDirectory = NO;
        if ([fm fileExistsAtPath:fullPath
                     isDirectory:&isDirectory]) {
            if (isDirectory) {
                //create the directory
                BOOL createSuccess = [fm createDirectoryAtPath:fullSandboxPath
                                   withIntermediateDirectories:YES
                                                    attributes:nil
                                                         error:&error];
                if (createSuccess) {
                    //yay
                } else {
                    NSLog(@"Error creating directory in sandbox: %@", error);
                }
            } else {
                //simply copy the file over
                BOOL copySuccess = [fm copyItemAtPath:fullPath
                                               toPath:fullSandboxPath
                                                error:&error];
                if (copySuccess) {
                    //yay
                } else {
                    NSLog(@"Error copying item at path: %@\nTo path: %@\nError: %@",
                          fullPath, fullSandboxPath, error);
                }
            }
        } else {
            NSLog(@"Got subpath but there is no file at the full path: %@", fullPath);
        }
    }
    
}
@end
