//
//  PNDocSupportNetController+PipelineTransitionFiring.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 12/5/12.
//
//

#import "PNDocSupportNetController.h"

#import "PNTransitionDelegate.h"
#import "PNDocumentCreating.h"

@class PNDocToken, PNDocTokenPlace;

/**
 The PipelineTransitionFiring category of PNDocSupportNetController encapsulates methods that fire pipelnied transitions of PNDocSupportNet. See also the TransitionFiring category for simple transitions.
 */

@interface PNDocSupportNetController (PipelineTransitionFiring)

#pragma mark PNCloseCreate:
/**
	Closes a document, creates a new one.
 
 @param haplessDocToken holds an open document to close
 @param place the instance of PNDocTokenPlace that currently holds haplessDocToken and its open document.
 @param aLocalizedName the name for the newly created document
 @param documentClassName @"UIManagedDocument" or the name of a subclass
 @param midPathComponents an array of strings (perhaps nil or empty) that specifies iterim path components
 @param creationDelegate an object that implements PNDocumentCreating that may initialize the document's graph of objects.
 @param toPlace the instance of PNDocTokenPlace that will receive the newly created PNDocToken and its opened document.
 @param sender the PNTransitionDelegate. Receives either a message that the transition succeeded or failed.
 */

-(void)      closeDocToken: (PNDocToken*)haplessDocToken
                 fromPlace: (PNDocTokenPlace*)place
createDocWithLocalizedName: (NSString*)aLocalizedName
         documentClassName: (NSString*)documentClassName
         midPathComponents: (NSArray*)aMidPath
          creationDelegate: (id <PNDocumentCreating>)creationDelegate
                   toPlace: (PNDocTokenPlace*)toPlace
                    sender: (id <PNTransitionDelegate>)sender;


#pragma mark PNCloseOpen
/**
	Closes one document, opens another.
 @param haplessDocToken holds an open document to close
 @param place the instance of PNDocTokenPlace that currently holds haplessDocToken and its open document.
 @param uuid a UUID in string form that specifies an available URL token (local or cloud)
 @param toPlace the instance of PNDocTokenPlace that will receive the PNDocToken and its opened document.
 @param sender the PNTransitionDelegate. Receives either a message that the transition succeeded or failed.
 */
-(void)closeDocToken: (PNDocToken*)haplessDocToken
           fromPlace: (PNDocTokenPlace*)place
     openDocWithUUID: (NSString*)uuid
             toPlace: (PNDocTokenPlace*)toPlace
              sender: (id <PNTransitionDelegate>)sender;


#pragma mark PNCloseFlipOpen
/**
    Speculative and untested.
 */
-(void)flipDocToken: (PNDocToken*)haplessDocToken
              place: (PNDocTokenPlace*)place
            toCloud: (BOOL)flag
             sender: (id <PNTransitionDelegate>)sender;
@end
