//
//  PNDocSupportNetController+StorageControlling.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/19/13.
//
//

// See: iCloud Design Guide, 2012 September 19
// Section: Prepare Your App to Use iCloud
//
#import "PNDocSupportNetController.h"

#import "PNTransitionDelegate.h"

/**
 
 PNStorageStatus enumerates the values of the storageStatus property.
 The values indicate whether the app is using local or cloud storage, and why.
 */
typedef enum PNStorageStatus
{
    
	PNStorageStatusInvalid = 0,  /**< The storageStatus property should never return zero. */
    PNStorageStatusLocalStorageByDefault = 1, /**< The app has never had iCloud available. */
    PNStorageStatusLocalStorageForever, /**< The user has chosen to use local storage permanently. */
    PNStorageStatusCloudAccountIndeterminate, /**< An interim value */
    PNStorageStatusCloudAccountAvailable, /**< The user has chosen to use cloud storage permanently, and an iCloud account is available: nil != self.mostRecentUbiquityIdentityToken. */
    PNStorageStatusCloudAccountUnavailable, /**< The user has chosen to use cloud storage permanently, and an iCloud account is unavailable: nil == self.mostRecentUbiquityIdentityToken */
    
} PNStorageStatus;

extern NSString *PNStorageDidInitializeNotification;

extern NSString *PNCloudAccountSignedOutNotification;
extern NSString *PNCloudAccountSignedInNotification;
extern NSString *PNCloudAccountDidChangeNotification;


/**
	This category encapsulates methods that determine whether the app's documents use local or cloud storage. If the app is using cloud storage as available, this category observes changes in availability (changes in the applicable iCloud account). 
 */
@interface PNDocSupportNetController (StorageControlling)

@property (readwrite, strong) id mostRecentUbiquityIdentityToken;

@property (readonly, strong) NSMetadataQuery *query;
@property (readonly, strong) NSMutableArray *queryObservers;

@property (readonly, assign) enum PNStorageStatus storageStatus;

@property (readonly, strong) NSInvocation *pnQueryUpdateSucceededCallback;


/**
	Sets storageStatus to PNStorageStatusLocalStorageByDefault and launches PNSyncResources.
 Note that the adopting developer should implement a way for the app to store this information, e.g., in user defaults.
 */
-(void)useLocalStorageByDefault;
/**
 Sets storageStatus to PNStorageStatusLocalStorageForever and launches PNSyncResources.
 Note that the adopting developer should implement a way for the app to store this information, e.g., in user defaults.
 */
-(void)useLocalStorageForever;

/**
 Sets storageStatus to PNStorageStatusCloudAccountIndeterminate and launches an asynchrous block to determine the URL of the associated container.
 Note that the adopting developer should implement a way for the app to store this information, e.g., in user defaults.
 
 @param ubqContainerID is of the form: @"TEAMCODE10.com.mycompany.MyApp"
 
 See, for example,
 http://stackoverflow.com/questions/9050359/should-urlforubiquitycontaineridentifier-be-called-in-a-thread-outside-the-main
 */
-(void)useUbiquityContainerIdentifier: (NSString*)ubqContainerID;

/**
	Workaround circa 2013 mid-April for ..um.. as yet unimplemented support for renaming cloud-persistent documents. This workaround prevents crashes until we sort out more about Apple's iCloud support.
	@returns whether it is safe to try to rename a document; YES if we're using local storage.
 */
-(BOOL)canRenameDocuments;

/**
 Workaround circa 2013 mid-April for ..um.. trouble making documents on the iPad. Funny: code that almost always works on the phone fails uniformly on the iPad. This workaround prevents crashes until we sort out more about Apple's iCloud support.
 @returns whether it is safe to try to manke a new document; YES if we're using local storage or on a phone.
 */
-(BOOL)canCreateDocuments;

@end
