//
//  PNDocSupportNetController+AppPreppingAndCloudUsing.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/19/13.
//
//

#import "PNDocSupportNetController+StorageControlling.h"
#import "PNDocSupportNetController+GreatBugReporting.h"

#import "PNAlertViewHelper.h"

#import "PNDocSupportNetController+TokenVending.h"

#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"
#import "PNCloudURLToken.h"
#import "PNURLTokenPlace.h"

#import "PNDocToken.h"

#import "NSURL+NPAssisting.h"

#import "PNDocSupportNet.h"
#import "PNQueryUpdate.h"
#import "PNFlipStorage.h"
#import "PNCloseFlipOpen.h"

#import "PNSyncResources.h"

#import "PNDocTokenPlace.h"


static NSString *PNStorageStatusKey = @"PN Cloud/Local Storage Status";

const NSString *PNStorageDidInitializeNotification = @"PN Storage Did Initialize Notification";

const NSString *PNCloudAccountSignedOutNotification = @"PN Cloud Account Signed Out";
const NSString *PNCloudAccountSignedInNotification  = @"PN Cloud Account Signed In";
const NSString *PNCloudAccountDidChangeNotification = @"PN Cloud Account Signed Changed";

@interface PNDocSupportNetController ()
@property (readwrite, assign) BOOL thisDeviceHasLaunchedWithCloudAvailable;
@property (readwrite, strong) NSMetadataQuery *query;
@property (readwrite, assign) enum PNStorageStatus storageStatus;


@end

@implementation PNDocSupportNetController (StorageControlling)


@dynamic mostRecentUbiquityIdentityToken;
//@dynamic usingCloudStorage;

@dynamic query;

@dynamic queryObservers;

@dynamic storageStatus;

-(void)launchSyncResources
{
    SEL s = @selector(didSyncLocalResources);
    NSMethodSignature *mSig = [self methodSignatureForSelector: s];
    NSInvocation *callback = [NSInvocation invocationWithMethodSignature: mSig];
    callback.target = self;
    callback.selector = s;
    
    [self.docSupportNet.syncResources enableFor: self.appLevelTransitionDelegate
                                    targetUuids: [NSSet set] // the empty set acts as a wild card.
                                       callback: callback];

}
-(void)didSyncLocalResources
{
    m_didSyncLocalResources = YES;
    [self rendezvousLocalAndCloud];
}
-(void)rendezvousLocalAndCloud
{
    /**
     Posts PNStorageDidInitializeNotification if required.
     This method is called from two places:
     [1] -didSyncLocalResources, the callback of PNSyncResources in -launchSyncResources
     [2] -didGetUbiquityContainer: the first time through
         (when self.storageStatus == PNStorageStatusCloudAccountIndeterminate).
     
     */
    
    if( m_didSyncLocalResources && m_didGetInitialContainerURL && !m_didPostDidInitialize ){
        
        switch( self.storageStatus ){
            case PNStorageStatusLocalStorageByDefault:
            case PNStorageStatusLocalStorageForever:
            {
                break;
            }
            case PNStorageStatusCloudAccountIndeterminate:
            {
                if( nil == self.ubiquityContainerIdentifier ){
                    self.storageStatus = PNStorageStatusCloudAccountUnavailable;
                }else{
                    self.storageStatus = PNStorageStatusCloudAccountAvailable;
                    [PNDocSupportNetController copyContainerToSandbox];
                }
                break;
            }
            case PNStorageStatusCloudAccountAvailable:
            case PNStorageStatusCloudAccountUnavailable:
            {
                break;
            }
            default:
            {
                NSAssert( NO, @"Bogus storageStatus %d", self.storageStatus );
            }
        }
        
        [self postNotificationName: (NSString*)PNStorageDidInitializeNotification];
        m_didPostDidInitialize = YES;

    }

}
-(enum PNStorageStatus)storageStatus
{
    enum PNStorageStatus result =
    [self.userDefaults integerForKey:PNStorageStatusKey];
    // "If the specified key does not exist,
    // this method returns 0 (namely, TLDemoUserChoiceOfStorageCaseNeverAsked).",
    return result;
}
-(void)setStorageStatus:(enum PNStorageStatus)nextStorageStatus
{
    switch ( nextStorageStatus ) {
            
        case PNStorageStatusCloudAccountIndeterminate:
        case PNStorageStatusCloudAccountAvailable:
        case PNStorageStatusCloudAccountUnavailable:
        case PNStorageStatusLocalStorageByDefault:
        case PNStorageStatusLocalStorageForever:
        {
            [self.userDefaults setInteger: nextStorageStatus
                                   forKey: PNStorageStatusKey];
            break;
        }
        case PNStorageStatusInvalid:
        default:
        {
            NSAssert( NO, @"Bogus value of storage status = %d",
                     nextStorageStatus);
        }
    }
}
-(NSMutableArray*)queryObservers
{
    if( nil == m_queryObservers ){
        m_queryObservers = [[NSMutableArray alloc] initWithCapacity: 4];
    }
    return m_queryObservers;
}

-(void)observeUbiquityIdentifier
{
    if( nil == m_ubiquityIdentityObserver ) {
        
        NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
        
        m_ubiquityIdentityObserver =
        [dfc addObserverForName: NSUbiquityIdentityDidChangeNotification
                         object: self.threadSafeFileManager
                          queue: [NSOperationQueue mainQueue]
                     usingBlock: ^(NSNotification __strong *notification)
         {
             [self updateUbiquityIdentifierToken];
         }];

    }
}
-(void)ignoreUbiquityIdentifier
{
    if( nil != m_ubiquityIdentityObserver ){
        NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
        [dfc removeObserver: m_ubiquityIdentityObserver]; //"Must not be nil.:
    }
    m_ubiquityIdentityObserver = nil;
}

-(NSMetadataQuery*)query
{
    if( nil == m_query ){
        m_query = [[NSMetadataQuery alloc] init];
        
        // Query like Warren's MultiDocument:
        [m_query setSearchScopes:
         [NSArray arrayWithObjects:
          NSMetadataQueryUbiquitousDataScope, NSMetadataQueryUbiquitousDocumentsScope, nil]];
        
        // We cannot look for the folder--must look for the
        // contained DocumentMetadata.plist.
        NSPredicate *p =
        [NSPredicate predicateWithFormat:@"%K like %@",
         NSMetadataItemFSNameKey, PNDocMDataDotPlistKey];
        
        [m_query setPredicate:p];
    }
    return m_query;
}

-(void)observeCloud
{
    enum PNStorageStatus status = self.storageStatus;
    
    switch (status) {
        case PNStorageStatusCloudAccountAvailable:
        {
            [self observeUbiquityIdentifier];
            [self observeQuery];
            
             break;
        }
        case PNStorageStatusCloudAccountIndeterminate:
        case PNStorageStatusCloudAccountUnavailable:
        {
            [self observeUbiquityIdentifier];
            break;
        }
        case PNStorageStatusLocalStorageByDefault:
        case PNStorageStatusLocalStorageForever:
        case PNStorageStatusInvalid:
        default:
        {
            NSAssert( NO,
                     @"-observeCloud bogus storageStatus %d", status);

            break;
        }
    }
}
-(void)ignoreCloud
{
    [self ignoreUbiquityIdentifier];
    [self ignoreQuery];
}

//-(NSSet*)validUpdateItems: (NSArray*)metadataItems
//{
//    return nil;
//}
//-(NSSet*)itemsForExistingCloudTokensInMetadataItems: (NSSet*)items
//{
//    return nil;
//}
//-(NSSet*)itemsForExistingLocalTokensInMetadataItems: (NSSet*)items
//{
//    return nil;
//}

-(NSInvocation*)pnQueryUpdateSucceededCallback
{
    if( nil == m_cloudDidUpdateCallback ){
        
        SEL mySelector = @selector(pnQueryUpdateSucceeded);
        NSMethodSignature *mSig = [self methodSignatureForSelector: mySelector];
        m_cloudDidUpdateCallback = [NSInvocation invocationWithMethodSignature: mSig];
        m_cloudDidUpdateCallback.target = self;
        m_cloudDidUpdateCallback.selector = mySelector;
    }
    return m_cloudDidUpdateCallback;
}
-(void)observeQueryNotification: (NSString*)notificationName
{
    
    NSMetadataQuery __block *query = self.query; // lazily instantiated
    
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    id observer =
    [dfc addObserverForName: notificationName
                     object: query
                      queue: [NSOperationQueue mainQueue]
                 usingBlock:^(NSNotification __strong *notification)
     {
         
         // It's worthwhile to filter the query results
         // and launch the transform only if it's required.
         
         NSLog(@"notification:");
         NSLog(@" %@", [notification description]);
         NSLog(@" query.results.count = %d", query.results.count);
        
         NSSet *filteredResults =
         [PNQueryUpdate filteredQueryResults: query.results];
         // The filtered results are just those NSMetadataItems
         // that do not correspond to existing cloud url tokens.
         // (The raw results can contain empty dictionaries; those get filtered out, too.)
         
         if( 0 != filteredResults.count ){ // need at least one new cloud url token.
             
             [query disableUpdates];{
                 
                 NSInvocation *callback = self.pnQueryUpdateSucceededCallback;
                 
                 [self.docSupportNet.queryUpdate enableFor: self.appLevelTransitionDelegate
                                             metadataItems: filteredResults
                                                  callback: callback];
             }[query enableUpdates];
         }
         
     }];
    [m_queryObservers addObject: observer];
    
}


- (void)observeQuery
{
    /*
     See: http://stackoverflow.com/questions/9026242/nsmetadataquery-ignoring-custom-file-package-type
     
     Basically, NSMetadataQuery can't deal with file extensions.
     */
    
    [self ignoreQuery];{
        
        [self observeQueryNotification: NSMetadataQueryDidStartGatheringNotification];
        [self observeQueryNotification: NSMetadataQueryGatheringProgressNotification];
        [self observeQueryNotification: NSMetadataQueryDidUpdateNotification];
        [self observeQueryNotification: NSMetadataQueryDidFinishGatheringNotification];
        
    }[self.query startQuery];
    [self.query enableUpdates];
    
}
-(void)ignoreQuery
{
    // Use m_query instead of self.query here. That is, if m_query is nil, leave it that way.
    
    [m_query disableUpdates];
    [m_query stopQuery];
    
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    
    for( id qObs in m_queryObservers ){
        [dfc removeObserver: qObs];
    }
    [m_queryObservers removeAllObjects];
    
    m_query = nil;
}

-(BOOL)usingCloudStorage
{
//    BOOL step1 = self.thisDeviceHasLaunchedWithCloudAvailable;
//    BOOL step2 = self.userOnThisDeviceHasChosenCloudStorage;
//    BOOL step3 = (nil != self.mostRecentUbiquityIdentityToken);
//    
//    return step1 && step2 && step3;
    
    return (nil != self.mostRecentUbiquityIdentityToken);
}
static NSString *PNUbiquityIdentityTokenAsNSDataKey = @"PN Ubiquity Identity Token Archived as NSData";

+(NSString*)ubiquityIdentityTokenArchiveKey
{
    static NSString *mm_ubiquityIdentityTokenArchiveKey = nil;
    if( nil == mm_ubiquityIdentityTokenArchiveKey ){
        NSString* bundleID = [[NSBundle mainBundle] bundleIdentifier];
        
        mm_ubiquityIdentityTokenArchiveKey =
        [[NSString alloc] initWithFormat: @"%@.%@",
         bundleID,
         PNUbiquityIdentityTokenAsNSDataKey];
        
    }
    return mm_ubiquityIdentityTokenArchiveKey;
}
-(id)mostRecentUbiquityIdentityToken
{
    id token = nil;
    
    NSString *archiveKey = [[self class] ubiquityIdentityTokenArchiveKey];
    
    NSData *data = [self.userDefaults objectForKey: archiveKey];
    
    if( nil != data ){
        token = [NSKeyedUnarchiver unarchiveObjectWithData: data];
    }
    
    return token;
}
-(void)setMostRecentUbiquityIdentityToken:(id)nextUbiquityIdentityToken
{
    NSString *archiveKey = [[self class] ubiquityIdentityTokenArchiveKey];
    
    if (nextUbiquityIdentityToken) {
        
        NSData *newTokenData =
        [NSKeyedArchiver archivedDataWithRootObject: nextUbiquityIdentityToken];
        
        [self.userDefaults setObject: newTokenData
                              forKey: archiveKey];
    } else {
        [self.userDefaults removeObjectForKey: archiveKey];
    }
    
}

-(void)postNotificationName: (NSString*)notificationName
{
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];

    dispatch_queue_t mainQueue = dispatch_get_main_queue();
    dispatch_async(mainQueue, ^{
        [dfc postNotificationName: notificationName
                           object: self];
    });
}
-(void)updateUbiquityIdentifierToken
{
    /**
    
     See: iCloud Design Guide, 2012 Sept 19, page 20
     Handle Changes in iCloud Availability
     
     To handle changes in iCloud availability, implement a method to be invoked on receiving an NSUbiquityIdentityDidChangeNotification notification. Your method needs to perform the following work:
     
     
     [1] Call the ubiquityIdentityToken method and store its return value.
     Compare the new value to the previous value, to find out if the user signed out of their account or signed in to a different account.
     
     [2] If the previously-used account is now unavailable, save the current state locally as needed, empty your iCloud-related data caches, and refresh all iCloud-related user interface elements.
     
     [3] If you want to allow users to continue creating content with iCloud unavailable, store that content in your app’s sandbox container. When the account is again available, move the new content to iCloud. It’s usually best to do this without notifying the user or requiring any interaction from the user.
     
         See -storageStatus and -[PNCreateDoc createdURLToken].
     */


    // [1]
    id previousAccount = self.mostRecentUbiquityIdentityToken;
    NSFileManager *fMgr = self.threadSafeFileManager;
    id currentAccount = [fMgr ubiquityIdentityToken];
    // [currentAccount class] = __NSCFData
    // e.g.,
    //
    // (gdb) p (int)[currentAccount length]
    // $1 = 20
    // 
    // (gdb) po currentAccount
    // <42f00ba8 deadbeef 12345678 87654321 deadbeef> // digits changed

    self.mostRecentUbiquityIdentityToken = currentAccount;
    
    // [2]
    BOOL didChangeAccount = ![previousAccount isEqual: currentAccount];
    
    if( didChangeAccount ){
        [self ignoreCloud];{
            
            // Set self.storageStatus:
            if( nil == currentAccount ){
                self.storageStatus = PNStorageStatusCloudAccountUnavailable;
            }else{
                self.storageStatus = PNStorageStatusCloudAccountAvailable;
            }
            
            // Send the right notification:
            if( (nil != previousAccount) && (nil != currentAccount) ){
                [self postNotificationName: (NSString*)PNCloudAccountDidChangeNotification];
            }
            if( (nil == previousAccount) && (nil != currentAccount) ){
                [self postNotificationName: (NSString*)PNCloudAccountSignedInNotification];
            }
            if( (nil != previousAccount) && (nil == currentAccount) ){
                [self postNotificationName: (NSString*)PNCloudAccountSignedOutNotification];
            }

        }[self observeCloud];
     }
    
}
-(void)useLocalStorageByDefault
{
    self.storageStatus = PNStorageStatusLocalStorageByDefault;
    
    m_didGetInitialContainerURL = YES; // YES, as in we'll never need one.
    
    [self launchSyncResources];
}
-(void)useLocalStorageForever
{
    self.storageStatus = PNStorageStatusLocalStorageForever;
    
    m_didGetInitialContainerURL = YES; // YES, as in we'll never need one.
    
    [self launchSyncResources];
}


-(void)useUbiquityContainerIdentifier: (NSString*)uci
{
    NSAssert( (nil != uci),
             @"Bogus nil value for Ubiquity Container Identifier");
    
    m_ubiquityContainerIdentifier = [uci copy];
    
    self.storageStatus = PNStorageStatusCloudAccountIndeterminate;
    
    
    [self launchSyncResources];
    
    NSFileManager __block *fMgr = [self threadSafeFileManager];
    
    dispatch_queue_t q = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    dispatch_async(q, ^{
        NSURL __block *ubiquityContainer =
        [fMgr URLForUbiquityContainerIdentifier:self.ubiquityContainerIdentifier];
        
        dispatch_queue_t mainQueue = dispatch_get_main_queue();
        dispatch_async(mainQueue, ^{
            m_didGetInitialContainerURL = YES;
            [self didGetUbiquityContainer:ubiquityContainer];
        });
    });

    
}
-(void)didGetUbiquityContainer: (NSURL*)ubiquityContainer
{
    m_cloudContainer = [ubiquityContainer pnNormalizedURL];
    
    if( PNStorageStatusCloudAccountIndeterminate == self.storageStatus ){
        [self rendezvousLocalAndCloud];
    }
    
    // We haven't yet set storageStatus; it's Invalid.
    // So, we can't call -observeCloud.
    
    // //DISABLED! 2013 Feb 15
    [self observeUbiquityIdentifier];
 
    [self updateUbiquityIdentifierToken];
}

#pragma mark The metadata query update fires the PNQueryUpdate transition which, on successful completion, invokes its callback to this method.
-(void)pnQueryUpdateSucceeded
{
    // Probably don't need to call -updateUbiquityIdentifierToken
    //[self updateUbiquityIdentifierToken];

    if( nil != self.oneShotCloudStorageCallback ){
        NSInvocation *oneShotCallback = self.oneShotCloudStorageCallback;
        self.oneShotCloudStorageCallback = nil;
        
        [oneShotCallback invoke];
    }
}

#pragma mark Two methods for iCloud Damage Control, circa 2013 April 03
-(BOOL)canRenameDocuments
{   
    BOOL result = YES;
    
    // 2013 April 03
    // Rename for iCloud won't work until I get more clues from Apple.
    
    switch (self.storageStatus) {
        case PNStorageStatusCloudAccountAvailable:
        case PNStorageStatusCloudAccountIndeterminate:
        case PNStorageStatusInvalid:
        {
            result = NO;
            break;
        }
        case PNStorageStatusCloudAccountUnavailable:
        case PNStorageStatusLocalStorageByDefault:
        case PNStorageStatusLocalStorageForever:
        default:
        {
            result = YES;
            break;
        }
    } // switch()
    
    return result;
} 

-(BOOL)canCreateDocuments
{
    BOOL result = YES;
        
    // 2013 April 03
    // iPad can't create cloud docs until I get more clues from Apple.
    
    UIDevice *device = [UIDevice currentDevice];
    NSString *model = device.model;
    NSRange range = [model rangeOfString:@"iPhone"];
    BOOL isPhone = ( NSNotFound != range.location );
    
    switch (self.storageStatus) {
        case PNStorageStatusCloudAccountAvailable:
        case PNStorageStatusCloudAccountIndeterminate:
        case PNStorageStatusInvalid:
        {
            result = isPhone;
            break;
        }
        case PNStorageStatusCloudAccountUnavailable:
        case PNStorageStatusLocalStorageByDefault:
        case PNStorageStatusLocalStorageForever:
        default:
        {
            result = YES;
            break;
        }
    } // switch()

    return result;
}

@end
