//
//  PNDocSupportNetController+TokenVending.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/2/12.
//
//

#import "PNDocSupportNetController.h"

#import "PNURLToken.h"

@class PNURLTokenPlace, PNURLToken, PNDocToken;

/**
	The TokenVending category of PNDocSupportNetController encapsulates methods that provide URL tokens, typically as ordered sets. The adopting developer can use these methods to show the user what documents are currently open, and what URLs are available to open as documents.
    */
@interface PNDocSupportNetController (TokenVending)

/**
	Returns one of the four primary URL Token Places: {local, cloud} x {assigned, unassigned}
 @param status a value of PNUuidStatus
	@returns the PNPlace that holds tokens with the status, or nil if no place applies (e.g., PNUuidStatusUnknown).
 */
-(PNURLTokenPlace*)placeForUuidStatus: (enum PNUuidStatus)status;

/**
 Returns the tokens of the place returned by -placeForUuidStatus:
 @param status a value of PNUuidStatus
	@returns Returns the tokens of the place returned by -placeForUuidStatus:, ordered by date.
 */
-(NSOrderedSet*)orderedAvailableTokensWithUuidStatus: (enum PNUuidStatus)status;

/**
	Combines assigned and unassigned local tokens.
	@returns local URL tokens, available and assigned to open documents.
 */
-(NSOrderedSet*)orderedLocalURLTokens;
/**
 Combines assigned and unassigned local tokens.
 @returns cloud URL tokens, available and assigned to open documents.
 */
-(NSOrderedSet*)orderedCloudURLTokens;

/**
 Combines assigned local and cloud tokens.
 @returns local and cloud URL tokens assigned to open documents.
 */
-(NSOrderedSet*)orderedAssignedURLTokens;

/**
 Combines assigned local and cloud tokens.
 @returns local and cloud URL tokens not assigned to open documents.
 */
-(NSOrderedSet*)orderedUnassignedURLTokens;

@end
