//
//  PNDocSupportNetController+TokenVending.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/2/12.
//
//

#import "PNDocSupportNetController+TokenVending.h"

#import "PNDocSupportNet.h"
#import "PNURLTokenPlace.h"

#import "PNURLToken.h"

@implementation PNDocSupportNetController (TokenVending)

-(NSOrderedSet*)orderedSetFromArray: (NSArray*)array
{
    NSMutableArray *mArray = [array mutableCopy];
    [mArray sortUsingSelector: @selector(date)];
    NSOrderedSet *resultSet = [NSOrderedSet orderedSetWithArray: mArray];
    return resultSet;
}
-(PNURLTokenPlace*)placeForUuidStatus: (enum PNUuidStatus)status
{
    PNURLTokenPlace *p4d = nil;
    switch ( status ) {
        case PNUuidStatusUnassignedLocalURL:
        {
            p4d = self.docSupportNet.unassignedLocalPlace;
            break;
        }
        case PNUuidStatusUnassignedCloudURL:
        {
            p4d = self.docSupportNet.unassignedCloudPlace;
            break;
        }
            
        case PNUuidStatusAssignedLocalURL:
        {
            p4d = self.docSupportNet.assignedLocalPlace;
            break;
        }
        case PNUuidStatusAssignedCloudURL:
        {
            p4d = self.docSupportNet.assignedCloudPlace;
            break;
        }
        case PNUuidStatusUnknown:
        case PNUuidStatusInvalid:
        default:
        {
            p4d = nil; // redundant but clear
             break;
        }
           
    }
    return p4d;
}
-(NSOrderedSet*)orderedAvailableTokensWithUuidStatus: (enum PNUuidStatus)status
{
    PNURLTokenPlace *p4s = [self placeForUuidStatus: status];
    if( nil == p4s ) return [NSOrderedSet orderedSet];
    
    return [p4s orderedAvailableTokens];
}


-(NSOrderedSet*)orderedLocalURLTokens
{
    NSOrderedSet *assigned   = [self.docSupportNet.assignedLocalPlace   orderedAvailableTokens];
    NSOrderedSet *unassigned = [self.docSupportNet.unassignedLocalPlace orderedAvailableTokens];
    
    NSMutableArray *result = [[assigned array] mutableCopy];
    [result addObjectsFromArray: [unassigned array]];
    
    NSOrderedSet *resultSet = [self orderedSetFromArray: result];
    return resultSet;
}
-(NSOrderedSet*)orderedCloudURLTokens
{
    NSOrderedSet *assigned   = [self.docSupportNet.assignedCloudPlace   orderedAvailableTokens];
    NSOrderedSet *unassigned = [self.docSupportNet.unassignedCloudPlace orderedAvailableTokens];
    
    NSMutableArray *result = [[assigned array] mutableCopy];
    [result addObjectsFromArray: [unassigned array]];
    
    NSOrderedSet *resultSet = [self orderedSetFromArray: result];
    return resultSet;
}


-(NSOrderedSet*)orderedAssignedURLTokens
{
    NSOrderedSet *local = [self.docSupportNet.assignedLocalPlace orderedAvailableTokens];
    NSOrderedSet *cloud = [self.docSupportNet.assignedCloudPlace orderedAvailableTokens];
    
    NSMutableArray *result = [[local array] mutableCopy];
    [result addObjectsFromArray: [cloud array]];
    
    NSOrderedSet *resultSet = [self orderedSetFromArray: result];
    return resultSet;
}

-(NSOrderedSet*)orderedUnassignedURLTokens
{
    NSOrderedSet *local = [self.docSupportNet.unassignedLocalPlace orderedAvailableTokens];
    NSOrderedSet *cloud = [self.docSupportNet.unassignedCloudPlace orderedAvailableTokens];
    
    NSMutableArray *result = [[local array] mutableCopy];
    [result addObjectsFromArray: [cloud array]];
    
    NSOrderedSet *resultSet = [self orderedSetFromArray: result];
    return resultSet;
}

@end
