//
//  PNDocSupportNetController+TransitionFiring.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/2/12.
//
//

#import "PNDocSupportNetController.h"

#import "PNDocumentCreating.h"

@protocol PNTransitionDelegate;

@class PNDocToken, PNURLToken, PNDocTokenPlace,
PNURLTokensTransition, PNDocTokenTransition,
PNCreateDoc, PNOpenDoc, PNCloseDoc, PNDeleteUUIDs, PNFlipUbiquitiesUUIDs;

/**
	The TransitionFiring category of PNDocSupportNetController encapsulates methods that fire simple transitions of PNDocSupportNet. See the PipelineTransitionFiring category for more.
 */
@interface PNDocSupportNetController (TransitionFiring)

#pragma mark PNDeleteUUIDs
-(void)deleteURLsWithUuids: (NSSet*)uuids
                    sender: (id <PNTransitionDelegate>)sender;

#pragma mark PNCreateDoc
/**
	Creates and opens an instance of UIManagedDocument or subclass, with specified properties.
 @param aLocalizedName name of the document, e.g., @"Untitled"
 @param documentClassName @"UIManagedDocument" or the name of a subclass
 @param midPathComponents an array of strings (perhaps nil or empty) that specifies iterim path components
 @param creationDelegate an object that implements PNDocumentCreating that may initialize the document's graph of objects.
 @param toPlace the instance of PNDocTokenPlace that will receive the created PNDocToken and its document.
 @param sender the PNTransitionDelegate. Receives either a message that the transition succeeded or failed.
 */
-(void)createDocWithLocalizedName: (NSString*)aLocalizedName
                documentClassName: (NSString*)documentClassName
                midPathComponents: (NSArray*)aMidPath
                 creationDelegate: (id <PNDocumentCreating>)creationDelegate
                          toPlace: (PNDocTokenPlace*)toPlace
                           sender: (id <PNTransitionDelegate>)sender;

#pragma mark PNOpenDoc
/**
	Opens the document URL specified by the uuid.
 @param uuid a UUID in string form that specifies an available URL token (local or cloud)
 @param toPlace the instance of PNDocTokenPlace that will receive the PNDocToken and its opened document.
 @param sender the PNTransitionDelegate. Receives either a message that the transition succeeded or failed.
 */
-(void)openDocWithUUID: (NSString*)uuid
               toPlace: (PNDocTokenPlace*)toPlace
                sender: (id <PNTransitionDelegate>)sender;

#pragma mark PNCloseDoc
/**
 Speculative, stub.
 */

-(void)closeDocToken: (PNDocToken*)docToken
           fromPlace: (PNDocTokenPlace*)place
              sender: (id <PNTransitionDelegate>)sender;

#pragma mark PNRenameDoc
/**
	Renames the document in docToken (local only, circa 2013 mid-April). The new document URL has a new uuid.
 @param newName the new name for the document
 @param sourcePlace the instance of PNDocTokenPlace that currently holds docToken and its open document.
 @param toPlace the instance of PNDocTokenPlace that will receive the PNDocToken and its opened document.
 @param deleteOriginalFlag YES deletes the original file package; NO does not.
 @param sender the PNTransitionDelegate. Receives either a message that the transition succeeded or failed.
 */
-(void)renameDocToken: (PNDocToken*)docToken
              newName: (NSString*)newName
            fromPlace: (PNDocTokenPlace*)sourcePlace
              toPlace: (PNDocTokenPlace*)destPlace
       deleteOriginal: (BOOL)deleteOriginalFlag
               sender: (id <PNTransitionDelegate>)sender;

@end
