//
//  DocSupportNetController.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/31/12.
//
//

#import <Foundation/Foundation.h>

#import "CWLSynthesizeSingleton.h"

#import "PNTransitionDelegate.h"

@class PNDocSupportNet;
@class PNDocTokenPlace;

/**
 The PNDocSupportNet class provides a singleton instance.
 The singleton owns an instance of PNDocSupportNet, and configures it in -makeReady;
 
 */

@class NSManagedObjectContext, NSManagedObjectModel, NSPersistentStoreCoordinator;

extern NSString* PNDocSupportNetControllerCloudAvailabilityDidChange;

/**
 DocSupportNetController provides a singleton instance sharedDocSupportNetController. It owns a singleton instance of PNDocSupportNet. The adopting developer can rely on PNDocSupportNetController and its categories to cover most needs. PNDocSupportNet does the real work, but it's largely hidden from the adopting developer; the adopting developer can mostly use the PNDocSupportNetController API.  
 */
@interface PNDocSupportNetController : NSObject
{
    //PNDocSupportNet *m_docSupportNet;
    NSURL *m_permID_docSupportNet;
    
    NSFileManager *m_threadSafeFileManager;
    
    NSMutableArray *m_notificationObservers;

    // See: PNDocSupportNetController (CoreDataManaging)
    NSManagedObjectContext *_managedObjectContext;
    NSManagedObjectModel *_managedObjectModel;
    NSPersistentStoreCoordinator *_persistentStoreCoordinator;
    
    NSURL *_persistentStoreDirectory;
    
    // And if we ever use Cloud Storage, we need these:
    NSMetadataQuery *m_query; 
    NSMutableArray *m_queryObservers;
    id m_ubiquityIdentityObserver; // "An opaque object to act as the observer" of NSUbiquityIdentityDidChangeNotification
    
    NSString *m_ubiquityContainerIdentifier;
    NSURL *m_cloudContainer;
    
    // The StorageControlling category uses the following ivars
    NSInvocation *m_cloudDidUpdateCallback; 
    BOOL m_didSyncLocalResources;
    BOOL m_didGetInitialContainerURL;
    BOOL m_didPostDidInitialize;
}

@property (readonly, strong) PNDocSupportNet *docSupportNet;
@property (readonly, strong) NSMutableArray *notificationObservers;
@property (readonly, strong) NSURL *persistentStoreDirectory;
@property (readonly, strong) NSFileManager *threadSafeFileManager;

@property (readonly, strong) NSString *ubiquityContainerIdentifier;
@property (readonly, strong) NSURL *cloudContainer;

/**
	The transition delegate will handle any errors from PNSyncResources and PNQueryUpdate.
 */
@property (readwrite, weak) id <PNTransitionDelegate> appLevelTransitionDelegate;

@property (readonly, strong) NSUserDefaults *userDefaults;

@property (readwrite, strong) NSInvocation *oneShotCloudStorageCallback;

+(PNDocSupportNetController*)sharedDocSupportNetController;
CWL_DECLARE_SINGLETON_FOR_CLASS(PNDocSupportNetController);


/**
 
 [1] builds a core data stack for the PNDocSupportNet instance:
 [1a] core data model from PNDocSupportModel.xcdatamodeld, _managedObjectModel;
 [1b] persistent store coordinator, _persistentStoreCoordinator
 [1c] managed object context _managedObjectContext
 
 [2] instantiates the PNDocSupportNet instance, self.docSupportNet.
 
 [3] builds the object graph of the PNDocSupportNet instance:
 [3a] four main places: {local, cloud}x{unassigned, assigned}
 [3b] static arcs
 [3c] (transitions instantiated lazily)
 
 [4] sends -[PNDocSupportNet launchQuery]
 */
-(void)buildCoreDataStackInDirectory: (NSURL*)persistentStoreDirectory
          appLevelTransitionDelegate: (id <PNTransitionDelegate>)delegate;

+(NSFileManager*)threadSafeFileManager;

+(BOOL)guaranteeDirectoryAtURL: (NSURL*)url;
+(BOOL)guaranteeFileAtURL: (NSURL*)url;

@end
