#import "_PNPlace.h"

/**
 General purpose parent class at the level of PNNet, PNTransition, PNPlace,  PNArc, PNArcP2T, PNArcT2P, PNToken.
 
 PNPlace holds a set of PNTokens. Each token may be available. When a place adds a token, it may be available immediately, or it may become available later. (A token never becomes unavailable while held by a place, before a transition takes it.)
 
 The input arcs to a place are instances of PNArcT2P; the output arcs from a place are instances of PNArcP2T.
 */
@interface PNPlace : _PNPlace {}
// Custom logic goes here.

/**
 Available tokens.
 @returns the receiver's available tokens
 */
-(NSSet*)availableTokens;

/**
	Available tokens, sorted by date.
	@returns the receiver's available tokens sorted by date
 */
-(NSOrderedSet*)orderedAvailableTokens;

/**
	Removes any tokens.
 */
-(void)clear;


/**
	For debugging.
	@returns a string describing the tokens in the receiver.
 */
-(NSString*)quickLook;

-(void)addToken: (PNToken*)token;
-(void)removeToken: (PNToken*)token;
@end
