#import "PNPlace.h"

#import "PNToken.h"
#import "PNNet.h"

#import "PNArcP2T.h"
#import "PNArcT2P.h"

//#import <PNDocSupport/PNURLToken.h>
#import "PNURLToken.h"

@implementation PNPlace

// Custom logic goes here.


-(NSString*)quickLook
{
    NSMutableString *result = [NSMutableString stringWithFormat:@"%@\n", self.name];
    
    // [1] tokens:
    NSOrderedSet *orderedTokens = [self orderedAvailableTokens];
    if( 0 == orderedTokens.count ){
        [result appendFormat: @"\t <empty>\n"];
    }else{
        for( PNToken *t in orderedTokens ){
            [result appendFormat: @"\t %@\n",
             [t quickLook]];
        }
    }
    
    // [2] in arcs:
    [result appendString: @"inArcs:\n"];
    for( PNArcT2P *nextIn in self.inArcs ){
        [result appendFormat: @"%@\n",
         [nextIn route]];
    }
    
    // [3] out arcs:
    [result appendString: @"outArcs:\n"];
    for( PNArcP2T *nextOut in self.outArcs ){
        [result appendFormat: @"%@\n",
         [nextOut route]];
    }
    
    
    return result;
}

- (void)awakeFromInsert
{
    [super awakeFromInsert];
    
    // I think this method is called BEFORE we add the place to the net.
    
    NSUInteger pCount = self.net.places.count;
    
    if( nil == self.name ){
        NSString *tmpName = [NSString stringWithFormat: @"P_%d", 1+pCount];
        self.name = tmpName;
    }
}

-(NSSet*)availableTokens
{
    NSSet *all = self.tokens;
    NSMutableSet *available = [NSMutableSet setWithCapacity: all.count];
    
    for( PNToken *next in all ){
        if( next.isAvailableValue ){
            [available addObject: next];
        }
    }
    return available;
}
-(void)clear
{
    while( 0 != self.tokens.count ){
        PNToken *any = self.tokens.anyObject;
        if( nil != any){
            [self removeTokensObject: any];
            //[any.managedObjectContext deleteObject: any];
        }
    }
}
-(NSOrderedSet*)orderedAvailableTokens
{
    NSSet *available = [self availableTokens];
    
    NSMutableOrderedSet *result =
    [NSMutableOrderedSet orderedSetWithArray: [available allObjects]];
    
    [result sortUsingComparator:^(id obj1, id obj2) {
        
        PNURLToken *u1 = (PNURLToken *)obj1;
        PNURLToken *u2 = (PNURLToken *)obj2;
       
        NSTimeInterval t1 = u1.date.timeIntervalSinceReferenceDate;
        NSTimeInterval t2 = u2.date.timeIntervalSinceReferenceDate;
        
        if ( t1 > t2 ) {
            return (NSComparisonResult)NSOrderedDescending;
        }
        
        if ( t1 < t2 ) {
            return (NSComparisonResult)NSOrderedAscending;
        }
        return (NSComparisonResult)NSOrderedSame;
    }];
    return result;
}

-(void)addToken: (PNToken*)token
{
    NSAssert( ![self.tokens containsObject: token], @"contains token already");
    NSUInteger count = self.tokens.count;
    if( 0 != self.capacityValue ){
        NSAssert( (self.capacityValue > count), @"Overfull" );
    }
    // can't be [super addTokensObject: token];
    // [PNURLTokenPlace addTokensObject:]: unrecognized selector sent to instance 0x78e72c0
    [self addTokensObject: token];
    
    // Let Core Data manage the inverse relationship in its own way, in its own time.
    // This assert seems to be a mistake.
    // It's OK on the simulator, but not on the iPhone.
    //NSAssert( (self == token.place), @"Busted inverse.");
}
-(void)removeToken: (PNToken*)token
{
    NSLog(@" token = %@", [token description]);
    
    NSAssert( [self.tokens containsObject: token], @"token missing");
    
    if( NO == token.isAvailableValue){
        NSAssert( token.isAvailableValue, @"token not available");
    }
   
    // can't be [super removeTokensObject: token];
    // [PNURLTokenPlace removeTokensObject:]: unrecognized selector sent to instance 0x78e72c0
    [self removeTokensObject: token];

    // Let Core Data manage the inverse relationship in its own way, in its own time.
    // This assert seems to be a mistake.
    // It's OK on the simulator, but not on the iPhone.
    //NSAssert( (nil == token.place), @"Busted inverse."); 
}
@end
