//
//  PNDocumentCreating.h
//  TaggedLocationsDemo
//
//  Created by Don Briggs on 7/3/12.
//  Copyright (c) 2012 Directed Graph Software LLC (AB). All rights reserved.
//

#import <Foundation/Foundation.h>

@class PNURLToken;
@class PNDocToken;

/**
	Set of methods to be implemented to act as a document creation delegate. A creation delegate specifies the class (UIMangedDocumetn or subclass) of a new document to be created, and has the opportunity to intialize the new document's graph of objects.
 */
@protocol PNDocumentCreating //<NSObject>

#pragma mark Background Methods: 

/**
	Determines the class (UIManagedDocument or subclass) of the document to be created with the url token. The static library calls this delegate method to determine which class to use when it instantiates a document:
 
  Class SomeClass = NSClassFromString( someClassName );
  UIMangedDocument *result = [[SomeClass alloc] initWithURL: url];
 
	@returns a class name (@"UIManagedDocument" or the name of a subclass)
 */
-(NSString*)documentClassNameForUrlToken: (PNURLToken*)urlToken;

/**
	The delegate can build the initial graph of managed objects in a newly instantiated document. The static library calls this delegate method after it creates a new document from scratch.
 */
-(void)initializeDocumentObjectGraphInDocToken: (PNDocToken*)newDocToken;

@end
