//
//  PNTransitionDelegate
//  TaggedLocationsDemo
//
//  Created by Don Briggs on 7/10/12.
//  Copyright (c) 2012 Directed Graph Software LLC (AB). All rights reserved.
//

#import <Foundation/Foundation.h>

@class PNTransition;

/**
 Set of methods to be implemented to act as the transition delegate of a PNTransition. A transition delegate receives messages to signal success or failure of its active transition.
 */
@protocol PNTransitionDelegate <NSObject>
-(PNTransition*)activeTransition;


/**
	Called in the -[PNTransition enableFor:] method
 */
-(void)setActiveTransition: (PNTransition*)transition;


/**
	Called from -[PNTransition didTransform] on success.
 */
-(void)didFireTransition: (PNTransition*) transition;

/**
	Speculative and untested. Return NO for now.
	@returns Delegates should implement to return NO for now.
 */
-(BOOL)shouldAttemptToRollbackTransition: (PNTransition*) transition; 

/**
 Called from -[PNTransition didTransform] on failure.
 */
-(void) didFailTransition: (PNTransition*) transition // delegate must manage self.activeTransition
   rolledBackSuccessfully: (BOOL)success;
@end
