//
//  PNCloudURLToken+iCloudTransferring.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/31/12.
//
//

#import "PNCloudURLToken+iCloudTransferring.h"

#import <CoreData/CoreData.h>
#import "PNDocSupportNet.h"
#import "PNDocSupportNetController.h"

/*
 FOUNDATION_EXPORT NSString * const NSURLIsUbiquitousItemKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemHasUnresolvedConflictsKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemIsDownloadedKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemIsDownloadingKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemIsUploadedKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemIsUploadingKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemPercentDownloadedKey;
 FOUNDATION_EXPORT NSString * const NSMetadataUbiquitousItemPercentUploadedKey;
 
 */

@interface NSMetadataItem (PNInspecting)
-(NSNumber*)numberForUbiquitousItemKey: (NSString*)key;
-(BOOL)boolForUbiquitousItemKey: (NSString*)key;
-(float)floatForUbiquitousItemKey: (NSString*)key;
@end

@implementation NSMetadataItem (PNInspecting)

-(NSNumber*)numberForUbiquitousItemKey: (NSString*)key
{
    NSURL *url = [self valueForKey: NSMetadataItemURLKey];
    NSError *error = nil;
    id x = nil;
    BOOL y = [url getResourceValue: &x
                            forKey: key
                             error: &error];
    
    if( y && (nil == error) && [x isKindOfClass:[NSNumber class]]  ){
        return x;
    }else{
        // We may want to alert the user if the return value is nil.
        return nil;
    }
}
-(BOOL)boolForUbiquitousItemKey: (NSString*)key
{
    NSNumber *x = [self numberForUbiquitousItemKey: key];
    if( x ){
        return [x boolValue];
    }else{
        return NO; // Hmm... We need NAB (not a BOOL) like NAN (not a number).
    }
}
-(float)floatForUbiquitousItemKey: (NSString*)key
{
    NSNumber *x = [self numberForUbiquitousItemKey: key];
    if( x ){
        return [x floatValue];
    }else{
        return (float)NAN;
    }
}

@end

@implementation PNCloudURLToken (iCloudTransferring)

-(BOOL)isUbiquitous
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataItemIsUbiquitousKey];
}
-(BOOL)hasUnresolvedConflicts
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataUbiquitousItemHasUnresolvedConflictsKey];
}

-(BOOL)isDownloaded
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataUbiquitousItemIsDownloadedKey];
}
-(BOOL)isDownloading
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataUbiquitousItemIsDownloadingKey];
}
-(BOOL)isUploaded
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataUbiquitousItemIsUploadedKey];
}
-(BOOL)isUploading
{
    return [self.metadataItem boolForUbiquitousItemKey: NSMetadataUbiquitousItemIsUploadingKey];
}

-(float)percentageDownloaded
{
    return [self.metadataItem floatForUbiquitousItemKey: NSMetadataUbiquitousItemPercentDownloadedKey];
}
-(float)percentageUploaded
{
    return [self.metadataItem floatForUbiquitousItemKey: NSMetadataUbiquitousItemPercentUploadedKey];
}

-(BOOL)evict
{
    NSURL *haplessURL = [self cloudFilePackageURL];
    /*
     Discussion
     This method does not remove the item from the cloud.
     It removes only the local version.
     You can use this method to force iCloud to download a new version of the file or directory from the server.
     
     To delete a file permanently from the user’s iCloud storage, use the regular NSFileManager routines for deleting files and directories.
     Remember that deleting items from iCloud cannot be undone. Once deleted, the item is gone forever.
     */
    // See Erica Sadun's CloudHelper.m
    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    BOOL targetExists = [fileMngr fileExistsAtPath:haplessURL.path];
    if (!targetExists) return NO;
    
    NSError *error = nil;
    BOOL success = [fileMngr evictUbiquitousItemAtURL:haplessURL
                                                error:&error];
    
    if (!success){
        NSLog(@"Error evicting current copy of %@.: %@",
              haplessURL.path,
              error.localizedFailureReason);
    }
    return success;
}

-(BOOL)forceDownload
{
    NSURL *targetURL = [self cloudFilePackageURL];
    
    BOOL success = [self evict];
    if (!success) return NO;
    
    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    NSError *error = nil;
    success = [fileMngr startDownloadingUbiquitousItemAtURL:targetURL
                                                      error:&error];
    if (!success){
        NSLog(@"Error starting download of %@: %@",
              targetURL.path,
              error.localizedFailureReason);
    }
    
    return success;
}

@end

