#import "PNCloudURLToken.h"

#import "PNDocSupportNet.h"

#import "PNDocSupportNetController.h"

#import "NSManagedObject+PNPermanentIDing.h"

#import "NSURL+NPAssisting.h"

#import "PNDocToken.h"

static NSString *PN_ChangeLogsKey = @"ChangeLogs";
static NSString *PN_DocumentsKey = @"Documents";

@interface PNCloudURLToken ()
//@property (readwrite, strong) NSURL *containerURL;
//@property (readwrite, strong) NSArray *midPathComponents;
//@property (readwrite, strong) NSString *uuid;
//@property (readwrite, strong) NSString *localizedName;

@end
@implementation PNCloudURLToken

@synthesize uuid;
@synthesize localizedName;
@dynamic persistentStoreOptions;
@dynamic cloudFilePackageURL;

// Custom logic goes here.

//@synthesize containerURL = m_containerURL;
//@synthesize midPathComponents = m_midPathComponents;

@synthesize metadataItem = m_metadataItem;
@dynamic metadata;

//@dynamic changeLogsURL;

+(NSURL*)containerUrl
{
    /*
     From Apple Docs:
     http://developer.apple.com/library/ios/#DOCUMENTATION/iPhone/Conceptual/iPhoneOSProgrammingGuide/iCloud/iCloud.html
     
     When storing documents in iCloud, place them in the Documents subdirectory whenever possible. Documents inside a Documents directory can be deleted individually by the user to free up space. However, everything outside that directory is treated as data and must be deleted all at once.
     */
    
    // Note:
    // +[NPCloudURLToken containerURL] returns a URL
    // with @"Documents" as the last path component.
    // Therefore, we use NSMetadataQueryUbiquitousDocumentsScope
    // in -[PNDocSupportNet query].
    
    static NSURL *mm_cloudContainerUrl = nil;
    if( nil == mm_cloudContainerUrl ){
                
        PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
        NSURL *result = [dsnc cloudContainer];
        
        // Follow Warren's MultiDocument --
 
        if( nil != result ){
            
            result = [result URLByAppendingPathComponent: PN_DocumentsKey
                                             isDirectory:YES];
            
            NSAssert( [PNDocSupportNetController guaranteeDirectoryAtURL: result],
                     @"Programming error: +containerUrl");
        }
        mm_cloudContainerUrl = [[result pnNormalizedURL] copy];
        
    }
    return mm_cloudContainerUrl;
}
-(NSURL*)cloudFilePackageURL
{
    if( nil == m_cloudFilePackageURL ){
        
        // Uses the PNURLToken containerUrl explicitly, always local.
        // Compare with -[PNURLToken filePackageURL] and -[PNURLToken localFilePackageURL].
        
        m_cloudFilePackageURL = [[self filePackageURLWithContainer:[PNCloudURLToken containerUrl]] copy];
        
        // When we call
        //    [[UIDocument alloc] initWithFileURL: foo];
        // the first time on "foo", foo is a file URL.
        // Thereafter, it's a directory.
        
        NSAssert( [PNDocSupportNetController guaranteeDirectoryAtURL: [m_cloudFilePackageURL URLByDeletingLastPathComponent]],
                 @"Programming error: -filePackageURL");
        
        NSLog(@"-[%@ cloudFilePackageURL] = %@",
              NSStringFromClass([self class]),
              m_cloudFilePackageURL.absoluteString );
        
        //        if( 2 < ([m_localDocumentURL.absoluteString componentsSeparatedByString:@"//"].count) ){
        //            NSLog( @"Snagged it...");
        //        }
    }
    return m_cloudFilePackageURL;
}

+(NSURL*)changeLogsContainerUrl
{
    /*
     From Apple Docs:
     http://developer.apple.com/library/ios/#DOCUMENTATION/iPhone/Conceptual/iPhoneOSProgrammingGuide/iCloud/iCloud.html
     
     When storing documents in iCloud, place them in the Documents subdirectory whenever possible. Documents inside a Documents directory can be deleted individually by the user to free up space. However, everything outside that directory is treated as data and must be deleted all at once.
     */
    
    
    static NSURL *mm_changeLogsContainerUrl = nil;
    if( nil == mm_changeLogsContainerUrl ){
        
        PNDocSupportNetController *dsnc =
        [PNDocSupportNetController sharedDocSupportNetController];
        
        NSURL *result = [dsnc cloudContainer];
        
        if( nil != result ){
            
            
            result = [result URLByAppendingPathComponent: PN_ChangeLogsKey
                                             isDirectory: YES];
            
// Maybe we can't guarantee the existence of this directory at this time.
//            NSAssert( [PNDocSupportNetController guaranteeDirectoryAtURL: result],
//                     @"Programming error: +changeLogsContainerUrl");

        }
        mm_changeLogsContainerUrl = [[result pnNormalizedURL] copy];
        
    }
    return mm_changeLogsContainerUrl;
}

-(NSURL*)changeLogsURL
{
    if( nil == m_changeLogsURL ){
        
        /**
         From "iCloud Design Guide," 2012-09-19:
         
         When you create a managed document, you need to specify a unique location for its change log files so that later, if you need to delete the document, you can find those files (to delete them as well).
         To set the location for a managed document’s change log files, specify a unique name (using the NSPersistentStoreUbiquitousContentNameKey key) and a unique URL (using the NSPersistentStoreUbiquitousContentURLKey key) in the managed document’s persistentStoreOptions property.
         For the URL, choose a location within the ubiquity container that you want to use, but outside of the Documents subdirectory. The document saves the specified name and URL in its DocumentMetadata.plist property list file, which is present inside the document’s file package.
         */
        
        NSDictionary *pStoreOptions = [self persistentStoreOptions];
        NSURL *uniqueURL     = [pStoreOptions objectForKey: NSPersistentStoreUbiquitousContentURLKey];
        NSString *uniqueName = [pStoreOptions objectForKey: NSPersistentStoreUbiquitousContentNameKey];
        
        m_changeLogsURL = [uniqueURL URLByAppendingPathComponent: uniqueName];
        
        NSLog(@"-[%@ m_changeLogsURL] = %@",
              NSStringFromClass([self class]),
              m_changeLogsURL.absoluteString );
        
    }
    return m_changeLogsURL;
}

-(NSString*)ubiquitousContentName
{
    // No callers to this method?
    return self.uuid;
}
-(NSDictionary*)persistentStoreOptions
{
    /* See: https://developer.apple.com/library/ios/#releasenotes/DataManagement/RN-iCloudCoreData/_index.html#//apple_ref/doc/uid/TP40011298
     
     [1] If you use a SQLite store, you need to provide a unique identifier for the store. You set the identifier using the NSPersistentStoreUbiquitousContentNameKey in the setPersistentStoreOptions: dictionary for the managed document.
     
     [2] You should also provide a ubiquitous content URL using the NSPersistentStoreUbiquitousContentURLKey.
     (The default URL is based on the bundle identifier. If you provide a custom URL for the transaction logs, you can share documents across applications with different identifiers. Typically you specify a directory called TransactionLogs in the ubiquity container.)
     So that you can retrieve it later if necessary (for example, when deleting a document), UIManagedDocument saves the store’s ubiquitous content name in a property list file named DocumentMetadata.plist in the document package.
     
     Note:
     Transaction logs for a document are stored in a directory:
     
     <Value of NSPersistentStoreUbiquitousContentURLKey>/<Value of NSPersistentStoreUbiquitousContentNameKey>.
     
     */
    
    /*
     From iCloud Design Guide, 2012-Sept-19
     
     When you create a managed document, you need to specify a unique location for its change log files so that later, if you need to delete the document, you can find those files (to delete them as well).
     To set the location for a managed document’s change log files, specify a unique name (using the NSPersistentStoreUbiquitousContentNameKey key) and a unique URL (using the NSPersistentStoreUbiquitousContentURLKey key) in the managed document’s persistentStoreOptions property.
     For the URL, choose a location within the ubiquity container that you want to use, but outside of the Documents subdirectory. The document saves the specified name and URL in its DocumentMetadata.plist property list file, which is present inside the document’s file package.
     */
    
    // In this static library,
    // the method -[PNDocToken nqd_allocInitDocument] calls this method.
    
    if( nil == m_persistentStoreOptions){
        
        // contains:
        // { *NameKey = uuid; *URLKey = file:<cloud base>/LogFiles/; }
        // where * = NSPersistentStoreUbiquitousContent.

        NSURL *changeLogsContainerUrl = [PNCloudURLToken changeLogsContainerUrl];
        
//        NSMutableArray *relativePathComponentsToUuidDir = [[self relativePathComponents] mutableCopy];
//        [relativePathComponentsToUuidDir removeLastObject]; // remove localizedName component, leave UUID as last.
        
//        NSURL *ubiquitousContentURL =
//        [changeLogsContainerUrl URLByAppendingPathComponents: relativePathComponentsToUuidDir
//                                                 isDirectory: YES];
        
        m_persistentStoreOptions =
        [[NSDictionary alloc] initWithObjectsAndKeys:
         
         [NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption,
         [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption,
         self.uuid,                     NSPersistentStoreUbiquitousContentNameKey,
         changeLogsContainerUrl,        NSPersistentStoreUbiquitousContentURLKey,
         
         nil];

        /*
         Example result:
         (gdb) po m_persistentStoreOptions
         {
         NSInferMappingModelAutomaticallyOption = 1;
         NSMigratePersistentStoresAutomaticallyOption = 1;
         NSPersistentStoreUbiquitousContentNameKey = "4BAB95BD-EDB9-4A53-808A-3A9E7F2D0BB3";
         NSPersistentStoreUbiquitousContentURLKey = "file://localhost/var/mobile/Library/Mobile%20Documents/~com~myCompany~TaggedLocationsDemo/ChangeLogs/";
         }

         */
    }
     
    return m_persistentStoreOptions;
}
-(BOOL)removeURLsWithError:(NSError**)error
{
    /**
     From "iCloud Design Guide," 2012-09-19:
     
     To delete a managed document, you must first close the document. 
     Then you must delete two directories, using coordinated write operations:
     [1] The directory for the Core Data change logs for the managed document 
     [2] The managed document’s file package
     */
    
    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    [fileMngr setUbiquitous: NO
                  itemAtURL: self.cloudFilePackageURL
             destinationURL: self.localFilePackageURL
                      error: error];

    /*
     See:
     FEB 5, 2012
     Getting started with iCloud Storage
     http://samvermette.com/312
     
     "... you’ll need to remove the old version first by calling setUbiquitous:NO using the iCloud URL as sourceURL and your local sandbox as destinationURL)."
     */
    if( nil != error ) return NO;

    NSFileCoordinator* coordinator =
    [[NSFileCoordinator alloc] initWithFilePresenter:nil];
    
    [coordinator coordinateWritingItemAtURL:self.cloudFilePackageURL
                                    options:NSFileCoordinatorWritingForDeleting
                                      error:nil
                                 byAccessor:^(NSURL *coordinatedURL) {
                                     
                                     [fileMngr removeItemAtURL:coordinatedURL
                                                         error:error];
                                     
                                 }];
    
    if( nil != error ) return NO;

    [coordinator coordinateWritingItemAtURL:self.changeLogsURL
                                    options:NSFileCoordinatorWritingForDeleting
                                      error:nil
                                 byAccessor:^(NSURL *coordinatedURL) {
                                     
                                     [fileMngr removeItemAtURL:coordinatedURL
                                                         error:error];
                                     
                                 }];
    
    return (nil == error);
}

-(NSDictionary*)metadata
{
    // No callers to this method?
    NSURL *metadataURL = [self.metadataItem valueForAttribute: NSMetadataItemURLKey];
    NSDictionary *d = [NSDictionary dictionaryWithContentsOfURL: metadataURL];
    return d;
}
-(BOOL)setUbiquitous: (BOOL)flag
               error: (NSError**)error
{
    NSLog(@"-[PNCloudURLToken setUbiquitous:error:] Begin");
    
    NSLog(@"[self documentState] = %@",
          self.docToken.documentStateAsString);
    
    
//    NSURL *cloudUuidDir = [self.cloudFilePackageURL URLByDeletingLastPathComponent];
//    BOOL uuidDirExists = [PNDocSupportNetController guaranteeDirectoryAtURL: cloudUuidDir];
    
    NSFileManager *fMgr =
    [PNDocSupportNetController threadSafeFileManager];
   
    BOOL isCloudDir = NO;
    BOOL cloudFilePackageURLExists =
    [fMgr fileExistsAtPath: self.cloudFilePackageURL.path
               isDirectory: &isCloudDir];
    
    BOOL probablyUbiquitous = ( cloudFilePackageURLExists && isCloudDir);
    
    BOOL success = YES;
    if( !probablyUbiquitous ){
        [fMgr setUbiquitous: YES
                  itemAtURL: self.localFilePackageURL
             destinationURL: self.cloudFilePackageURL
                      error: error];
    }
    
    NSLog(@"[self documentState] = %@",
          self.docToken.documentStateAsString);

    NSLog(@"-[PNCloudURLToken setUbiquitous:error:] End %@",
          success ? @"YES" : @"NO");
    return success;
}

@end
