//
//  PNDocToken+CallbackSupporting.h
//  TaggedLocationsDemo
//
//  Created by Don Briggs on 6/22/12.
//  Copyright (c) 2012 Directed Graph Software LLC (AB). All rights reserved.
//

#import "PNDocToken.h"

#import "PNURLTokensTransition.h"


/**
	PNDocSupport static library's errors:
 
 UIDocument has a set of file operations with a "^(BOOL success){...};"
 as a completion handler argument.
 
 For the sake of completeness and consistency,
 we assign an error code to each failed operation.
 
 */
typedef enum PNDocTokenFileOpsError
{
	PNURLTokenFileOpsErrorInvalid = 0, /**< Shouldn't appear; a nil object might return this value. */
    
PNDocTokenFileOpsErrorInvalid = 0, /**< Shouldn't appear either; a nil object might return this value. */
    
	PNURLTokenFileOpsErrorSetUbiquitous = 1000, /**< library-specific errors start at 1000. */

    PNDocTokenFileOpsErrorOpen, /**< Error opening a file package URL */
    
	PNDocTokenFileOpsErrorSaveForCreating, /**< Error on save-document-for-creating */
	PNDocTokenFileOpsErrorSaveForOverwriting, /**< Error on save-document-for-overwriting */
	PNDocTokenFileOpsErrorRevert, /**< Error on revert document */
	PNDocTokenFileOpsErrorClose, /**< Error on close document */
	PNDocTokenFileOpsErrorCopy, /**< Error on copy document */
	PNDocTokenFileOpsErrorSetUbiquitous /**< Error on set document file package URL ubiquitous */
} PNDocTokenFileOpsError;



/**
	See also UIManagedDocument (PNDocTokenCallbackSupporting). This category should be shared more guardedly with UIManagedDocument+PNDocTokenCallbackSupporting and not made so generally accessible.

 
 A doc token can perform file operations singly or aggregated.
 Such file operations, whether single or agggregate,
 result in:
 [1] on success, a call to
 [self.marker docTokenDidBecomeAvailable: self];
 [2] on failure at any step, a call to:
 [self.marker docToken: self didFailWithError: m_error];
 
 The method
 -becomeAvailable
 is enqueued as the last operation of the queue.
 
 */
@interface PNDocToken (CallbackSupporting)


-(void)didOpenSuccessfully: (BOOL)success;

-(void)didSaveSuccessfully: (BOOL)success
          forSaveOperation: (UIDocumentSaveOperation)saveOperation;
-(void)didRevertSuccessfully: (BOOL)success 
             toContentsOfURL: (NSURL *)url;
-(void)didCloseSuccessfully: (BOOL)success;

-(NSString*)stringForSaveOperation: (UIDocumentSaveOperation)saveOperation;

-(void)handleSuccess: (BOOL)success
           operation: (SEL)cmd
           errorCode: (enum PNDocTokenFileOpsError)errorCode;

-(void)handleSuccess: (BOOL)success
           operation: (SEL)cmd
               error: (NSError*)error;
@end

