//
//  PNDocToken+CallbackSupporting.m
//  TaggedLocationsDemo
//
//  Created by Don Briggs on 6/22/12.
//  Copyright (c) 2012 Directed Graph Software LLC (AB). All rights reserved.
//

#import "PNDocToken+CallbackSupporting.h"
#import "PNDocToken+FileOperations.h"
#import "PNDocTokenMarking.h"

#import "NSURL+NPAssisting.h"

#import "PNURLTokensTransition.h"
#import "PNDocTokenTransition.h"

#import "PNDocTokenPlace.h"

#import "PNURLToken.h"

#import "PNCloudURLToken.h"

#import "PNDocSupportNetController.h"


@implementation PNDocToken (CallbackSupporting)
#pragma mark Simple Operations


-(NSDictionary*)statusOfFailedOperation: (SEL)cmd
{
    
    NSString *selString = NSStringFromSelector( cmd );
    
        [self.queueLog appendFormat:@"%@: Failed operation: -%@", 
         [[NSDate date] description],
         selString ];
        
        NSArray *stalledOps = [self.operationQueue operations]; 
       [self.queueLog appendFormat:@" remaining operations: %d ", stalledOps.count ];
        for( NSInvocationOperation *op in stalledOps ){
            [op cancel];
            NSInvocation *nVoc = op.invocation;
            SEL theSelector = [nVoc selector];
            NSString *ss = NSStringFromSelector( theSelector );
            [self.queueLog appendFormat:@"%@ ", ss];
        }
    
    NSMutableDictionary *damageControl = 
    [NSMutableDictionary dictionaryWithObjectsAndKeys:
        selString, PNDocTokenFailedOperationKey,
        [self quickLook], PNDocTokenFailedOperatorKey,
        [self.queueLog copy], PNDocTokenFailedOperationLogKey,
     nil];
    
    NSLog(@"-[PNDocToken statusOfFailedOperation: %@", selString);
    NSLog(@"%@", [damageControl description] );

    return damageControl;
}
-(void)logSuccessfulOperation: (SEL)cmd
{
    [self.queueLog appendFormat:@" performed -%@ at %@ \n", 
     NSStringFromSelector( cmd ), 
     [[NSDate date] description]];
}

-(void)handleSuccess: (BOOL)success
           operation: (SEL)cmd
           errorCode: (enum PNDocTokenFileOpsError)errorCode
{
    //@synchronized(self){
    
    if(success){
        [self logSuccessfulOperation: cmd];
        
        // If we've succeeded, proceed with remaining operations.
        [self.operationQueue setSuspended: NO];
        
    }else{
        // Otherwise,
        // DON'T proceed with remaining operations.
        [self.operationQueue setSuspended: YES];
        
        // Build a report of what just happened:
        NSDictionary *report = [self statusOfFailedOperation: cmd];
        
        // Build an error:
        NSError *error =
        [NSError errorWithDomain:@"Static Library"
                            code:errorCode
                        userInfo:report];
        
        [self handleSuccess: success
                  operation:cmd
                      error: error];
    }
    
    //} //@synchronized(self)
}

-(void)handleSuccess: (BOOL)success
           operation: (SEL)cmd
               error: (NSError*)error
{
    //@synchronized(self){
    
    if(success){
        [self logSuccessfulOperation: cmd];
        
        // If we've succeeded, proceed with remaining operations.
        [self.operationQueue setSuspended: NO];
        
    }else{
        // Otherwise,
        // DON'T proceed with remaining operations.
        [self.operationQueue setSuspended: YES];
        
        // Build a report of what just happened:
        NSDictionary *report = [self statusOfFailedOperation: cmd];
        
        NSMutableDictionary *info = [error.userInfo mutableCopy];
        if( nil == info ){
            info = [NSMutableDictionary dictionaryWithCapacity: 1];
        }
        [info setObject: report
                 forKey: @"PNDocTokenFileOpsErrorCopy"];
        
        m_error = [NSError errorWithDomain: error.domain
                                      code: error.code
                                  userInfo: info];
        
        
        
        // Send it to the marker:
        dispatch_async(dispatch_get_main_queue(),
                       ^{
                           PNDocTokenPlace *p = (PNDocTokenPlace*)self.place;
                           [p.marker docToken: self
                             didFailWithError: m_error];
                       }
                       );
    }
    
    //}// @synchronized(self)
}

-(void)didOpenSuccessfully: (BOOL)success
{
    NSLog(@"-[PNDocToken didOpenSuccessfully:%@]",
          (success ? @"YES":@"NO"));
    
    NSAssert( UIDocumentStateNormal == self.document.documentState,
             @"%@ found bogus document state = %@",
             NSStringFromSelector( _cmd ),
             self.documentStateAsString);

    [self logRegisteredObjects];
    NSLog( @"self.localDocumentURL = %@", self.urlToken.localFilePackageURL);
    
    if( success ){
        //[self registerUuid];
    }
    [self handleSuccess: success
              operation: _cmd
              errorCode: (enum PNDocTokenFileOpsError) PNDocTokenFileOpsErrorOpen];
    
}
-(NSString*)stringForSaveOperation: (UIDocumentSaveOperation)saveOperation
{
    NSString *result = nil;
    switch (saveOperation) {
        case UIDocumentSaveForCreating:
        {
            result = @"Creating";
            break;  
        }
        case UIDocumentSaveForOverwriting:
        {
            result = @"Overwriting";
            break;
        }
        default:
        {
            result =
            [NSString stringWithFormat: @"Unknown save operation: %d", saveOperation];
            break;
        }
    }
    return result;
}
-(void)didSaveSuccessfully: (BOOL)success
          forSaveOperation: (UIDocumentSaveOperation)saveOperation
{
    NSLog(@"-[PNDocToken didSaveSuccessfully:%@ forSaveOperation: %@]", 
          (success ? @"YES":@"NO"),
          [self stringForSaveOperation: saveOperation]);
    
    enum UIDocumentState state = self.document.documentState;

    BOOL showStopper =
    (state & UIDocumentStateClosed) ||
    (state & UIDocumentStateInConflict) || //??
    (state & UIDocumentStateSavingError);

    NSAssert( !showStopper,
             @"%@ found bogus document state = %@",
             NSStringFromSelector( _cmd ),
             self.documentStateAsString);
    
    NSLog( @"self.localDocumentURL = %@", self.urlToken.localFilePackageURL);
    
    enum PNDocTokenFileOpsError __block code = PNDocTokenFileOpsErrorInvalid;
    switch (saveOperation) {
        case UIDocumentSaveForCreating:
        {
            code = PNDocTokenFileOpsErrorSaveForCreating;
            break;
        }
        case UIDocumentSaveForOverwriting:
        {
            code = PNDocTokenFileOpsErrorSaveForOverwriting;
            break;
        }
        default:
        {
            code = PNDocTokenFileOpsErrorInvalid;
            break;
        }
    }
    [self handleSuccess: success
              operation: _cmd
              errorCode: code];
    
    
}

-(void)didRevertSuccessfully: (BOOL)success
             toContentsOfURL: (NSURL *)url
{
    NSLog(@"-[PNDocToken didRevertSuccessfully:%@ toContentsOfURL:]",
          (success ? @"YES":@"NO"));
    [self handleSuccess: success
              operation: _cmd
              errorCode: PNDocTokenFileOpsErrorRevert];
}

-(void)didCloseSuccessfully: (BOOL)success
{
    NSLog(@"-[PNDocToken didCloseSuccessfully:%@]", 
          (success ? @"YES":@"NO"));
    //[self logRegisteredObjects];
    NSAssert( UIDocumentStateClosed == self.document.documentState,
             @"%@ found bogus document state = %@",
             NSStringFromSelector( _cmd ),
             self.documentStateAsString);

    
    if( success ){
        //[self unregisterUuid];
    }

    [self handleSuccess: success
              operation: _cmd
              errorCode: PNDocTokenFileOpsErrorClose];
    
}

@end
