//
//  PNDocToken+Managing.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 3/30/13.
//
//

#import "PNDocToken+Managing.h"

#import "PNCloudURLToken.h"

#import "PNCloudURLToken+iCloudTransferring.h"

#import "NSURL+NPAssisting.h"

#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"

#import "PNDocTokenPlace.h"

#import "PNDocTokenMarking.h"

#import "NSManagedObject+PNPermanentIDing.h"

#import "PNDocSupportNetController.h"
#import "PNDocTokenTransition.h"


#import "UIManagedDocument+PNDocTokenCallbackSupporting.h"
#import "PNDocToken+CallbackSupporting.h"

@implementation PNDocToken (Managing)

-(void)redact2 // meaning: "bring back", "take back"
{
    NSLog(@"-[PNDocToken redact]");
    self.onceAndFuturePlace = (PNDocTokenPlace*)self.place;
    NSLog(@" self.place.marker = %@", self.onceAndFuturePlace.marker);
    
    // First operation in the queue
    [self.place removeToken: self];
}
-(void)restore2
{
    NSLog(@"-[PNDocToken restore]");
    
    // Next to last operation in the queue (before -becomeAvailable).
    [self.onceAndFuturePlace addToken: self];
    NSLog(@" self.place.marker = %@", self.onceAndFuturePlace.marker);
}
-(void)becomeAvailable2
{
    NSLog(@"-[PNDocToken becomeAvailable2]");
    
   //NSAssert( !self.isAvailableValue, @"must be unavailable");
    
    // Don't call self.setIsAvailableValue:
    // Use the related method to avoid an infinite loop.
    self.isAvailableValue = YES;
    

    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       PNDocTokenPlace *p = (PNDocTokenPlace*)(self.onceAndFuturePlace);
                       NSObject *m = (NSObject *)(p.marker);
                       
                       NSLog(@"self.place.marker = %@",
                             [m description]);
                       //[((PNDocTokenPlace*)(self.place)).marker docTokenDidBecomeAvailable: self];
                       id <PNDocTokenMarking> mPrime = (id <PNDocTokenMarking>)m;
                       [mPrime docTokenDidBecomeAvailable: self];
                   }
                   );
}

-(void)failGracefully
{
    NSLog(@"-[PNDocToken failGracefully] ");
    

    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       PNDocTokenPlace *p = (PNDocTokenPlace*)self.place;
                       [p.marker docToken: self
                         didFailWithError: m_error];
                   }
                   );
}
-(void)concludeSuccessfully
{
    [self restore2];
    [self becomeAvailable2];
}
-(void)allocInitDocumentWithLocalURL
{
    NSLog(@"-[PNDocToken allocInitDocumentWithLocalURL] (%@) ",
          self.urlToken.localizedName);
    
    NSString *documentClassName = self.urlToken.documentClassName;
    if( nil == documentClassName && (nil != self.creationDelegate)){
        documentClassName = [self.creationDelegate documentClassNameForUrlToken: self.urlToken];
        self.urlToken.documentClassName = documentClassName;
    }
    Class docFactory = NSClassFromString(self.urlToken.documentClassName);
    
    NSAssert( [docFactory isSubclassOfClass: [UIManagedDocument class]],
             @"%@ is not a subclass of UIManagedDocument.",
             documentClassName);
    
    m_document = nil;
    m_document = [[docFactory alloc] initWithFileURL: self.urlToken.localFilePackageURL];
    
    m_document.persistentStoreOptions = self.urlToken.persistentStoreOptions;
    
    m_document.managedObjectContext.mergePolicy = NSRollbackMergePolicy;

}
-(void)configureNewDocument
{
    
    NSLog(@"-[PNDocToken configureNewDocument] (%@) ",
          self.urlToken.localizedName);
    
    if( NO == m_documentObjectGraphInitialized ){
        [self.creationDelegate initializeDocumentObjectGraphInDocToken: self];
        m_documentObjectGraphInitialized = YES;
        
        NSManagedObjectContext *moc = self.document.managedObjectContext;
        [moc processPendingChanges];
    }
}
-(void)createAndSaveDocumentFolded
{
    NSLog(@"-[PNDocToken createAndSaveDocumentFolded] (%@) BEGIN",
          self.urlToken.localizedName);

    [self redact2];
    
    [self allocInitDocumentWithLocalURL];
    
    
    // The debugger gets confused in the blocks, and
    // reports these r-values as nil.
    UIManagedDocument __block *blockDoc = m_document;
    PNURLToken __block *blockUrlToken = self.urlToken;
    PNDocToken __block *this = self;
    
    [blockDoc saveToURL: blockUrlToken.localFilePackageURL
       forSaveOperation: UIDocumentSaveForCreating
      completionHandler:^(BOOL success) {
          if( success ){ 
              
              [this configureNewDocument];
              
              if ([blockUrlToken isMemberOfClass:[ PNCloudURLToken class]]) {
                  
                  /**
                   Erica Sadun indicates (by omission) that it's unnecessary to setUbiquitous:...
                   I find otherwise.
                   If I skip setUbiquitous:...,
                   [1] the application's cloud storage "looks wrong" in
                   the desktop Settings.app; and
                   [2] other devices don't discover supposedly cloud-resident documents.
                   */
                  
                  dispatch_queue_t queue =
                  dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
                  
                  dispatch_async(queue, ^{
                      
                      
                      PNCloudURLToken *cloudToken = (PNCloudURLToken *)blockUrlToken;
                      NSError *error = nil;
                      
                      BOOL success =
                      [cloudToken setUbiquitous: YES
                                          error:&error];
                      
                      // document state is [EditingDisabled].
                      if(!success){
                          m_error = error;
                          [this failGracefully];
                      }
                      
                  });
              }

               PNDocToken __block *this2 = this;
              UIManagedDocument __block *blockDoc2 = [this2 document];
              
              [blockDoc2 saveToURL: blockUrlToken.localFilePackageURL
                  forSaveOperation: UIDocumentSaveForOverwriting
                 completionHandler:^(BOOL success) {
                     
                     if( success ){
                         NSLog(@"Saved for overwriting");
                         
                         [this2 concludeSuccessfully];
                         
                     }else{
                         NSLog(@"Failed to Save for overwriting");
                         [this2 failGracefully];
                     }
                 }];
              
          }else{
              [this failGracefully];
          }
      }];
    
    NSLog(@"-[PNDocToken createAndSaveDocumentFolded] (%@) END",
          self.urlToken.localizedName);

}
-(void)establishDocumentFolded
{
    NSLog( @"-establishDocumentFolded BEGIN ");

    [self redact2];
    
    [self allocInitDocumentWithLocalURL];
    
    NSFileManager *fMgr = [PNDocSupportNetController threadSafeFileManager];
 
    if ([fMgr fileExistsAtPath: self.urlToken.localFilePackageURL.path]){
        
        NSLog(@"Attempting to open existing file");
        
        [self.document openWithCompletionHandler:^(BOOL success){
            if (!success) {
                NSLog(@"In -establishDocumentFolded, Error opening file");
                return;
                [self failGracefully];
            }else{
                NSLog(@"File opened");
                [self concludeSuccessfully];
            }
            
        }]; //openWithCompletionHandler
        
    }else{
        NSLog(@"Creating file.");
        // 1. save it  (creating), 2. close it, 3. read it back in.
        
        [self.document saveToURL: self.urlToken.localFilePackageURL
           forSaveOperation:UIDocumentSaveForCreating
          completionHandler:^(BOOL success){
              if (!success) {
                  NSLog(@"In -establishDocumentFolded, Error creating file");
                  [self failGracefully];
              }else{
                  NSLog(@"File created");
                  
                  if( [self.urlToken isMemberOfClass:[ PNCloudURLToken class]] ){
                      
                      PNCloudURLToken *cloudToken = (PNCloudURLToken*)self.urlToken;
                      
                      [self.document closeWithCompletionHandler:^(BOOL success){
                          NSLog(@"Closed new file: %@", success ? @"Success" : @"Failure");
                          
                          if (!success) {
                              NSLog(@"In -establishDocumentFolded, Error closing file after creating.");
                              [self failGracefully];
                          }else{
            
                              NSError *error = nil;
                              
                              BOOL success = [cloudToken setUbiquitous: YES
                                                                 error: &error];
                              if (!success) {
                                  m_error = error;
                                  [self failGracefully];
                              }
                              
                              // After we close the doc, we can no longer use that instance.
                              // We must instantiate a new one and set its store options again:
                              
                              m_document = nil;
                              [self allocInitDocumentWithLocalURL];
                              
                              [self.document openWithCompletionHandler:^(BOOL success){
                                  
                                  if (!success) {
                                      NSLog(@"In -establishDocumentFolded, Error opening file after creating and closing.");
                                      [self failGracefully];
                                  }else{
                                      NSLog(@"Opened created file for reading.");
                                      [self concludeSuccessfully];
                                  }
                                  
                              }]; //open handler
                          } //if( close->success )
                      }]; // closeWithCompletionHandler:
                      
                      
                  } // if( [self.urlToken isMemberOfClass:[ PNCloudURLToken class]] )
                  
              }
          }]; //saveToURL:saveCreating:completionHandler:
    }
    NSLog( @"-establishDocumentFolded END ");
    
}

@end
