#import "_PNDocToken.h"

#import "PNDocumentCreating.h"
#import "PNDocTokenMarking.h"

#import <UIKit/UIKit.h>

@class UIManagedDocument;

@class PNDocTokenPlace;

/**
	PNDocToken is the workhorse of the static library. It wraps a document and the document's url token. It has its own operations queue. It manages operations on its document and the callbacks.

 See the UIManagedDocument (PNDocTokenCallbackSupporting) and the PNDocToken (CallbackSupporting) categories.
 
 */

@interface PNDocToken : _PNDocToken
{
    PNURLToken *m_urlToken;
    UIBackgroundTaskIdentifier m_bkgTaskID;
    
    NSOperationQueue  *m_operationQueue;
    
    NSMutableString *m_queueLog;
    
    UIManagedDocument *m_document;
    //id <PNDocumentCreating> m_delegate;
        
    BOOL m_readyToRollback;
    
    BOOL m_documentObjectGraphInitialized;
    
    // During document initialization,
    // we only need to send the -initializeDocumentObjectGraphInDocToken:
    // message if we've created a new <uuid>.
    // Otherwise, we're opening a document from
    // > an existing (discovered) local file; or
    // > a metadata query result.
    
    NSError *m_error;
    
}
// Custom logic goes here.

@property (readonly, strong) UIManagedDocument *document;

//@property (readwrite, assign) BOOL logEnabled;
@property (readonly, strong) NSString *log; // useful for error reporting and debugging.

//@property (readonly, assign) BOOL isAvailable;
@property (readonly, assign) BOOL documentReadable;
@property (readonly, assign) BOOL documentWriteable;
//@property (readonly, strong) NSURL *filePackageURL;

// The delegate:
@property (readwrite, strong) id <PNDocumentCreating> creationDelegate; // typically a UIViewController

// Property for closing and finalizing the document:
@property (readonly, assign) UIBackgroundTaskIdentifier bkgTaskID;

@property (readonly, strong) NSOperationQueue  *operationQueue;
@property (readonly, strong) NSMutableString  *queueLog;

@property (readonly,  strong) NSError *error;

// Used only in FileOperations:
@property (readwrite, strong) PNDocTokenPlace *onceAndFuturePlace;
@property (readwrite, strong) NSURL *savedPlacePermanentID;


//-(NSURL*)persistentStoreDbURL;

-(void)logRegisteredObjects; // a diagnostic, for the adopting developer

-(NSString *) documentStateAsString;
@end


