//
//  PNURLToken+LifeCycling.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 12/14/12.
//
//
#import "PNURLToken.h"

#import "PNURLToken+LifeCycling.h"
#import "PNCloudURLToken.h"

#import "NSManagedObject+PNPermanentIDing.h"
#import "PNDocSupportNetController.h"
#import "PNDocSupportNetController+StorageControlling.h"

#import "PNDocSupportNet.h"

#import "PNPlace.h"

#import "PNDocSupportNetController.h"

const NSString *PNDocMDataDotPlistKey = @"DocumentMetadata.plist";

@interface PNURLToken ()

//@property (readwrite, strong) NSURL *containerURL;
//@property (readwrite, strong) NSString *percentEscapedName;

@end


@implementation PNURLToken (LifeCycling)


-(void)assertProperties: (NSDictionary*)properties
{
    NSString *aUuid = [properties objectForKey: PNUuidStringKey];
    if( nil == aUuid ){
        aUuid = [PNURLToken nextUuid];
    }
    self.uuid = aUuid;
    
    
    NSString *test = [properties objectForKey: PNDocumentClassNameKey];
    if( nil != test ){
        self.documentClassName = test;
    }
    // If nil == self.documentClassName,
    
    self.midPathComponents = [properties objectForKey: PNMidPathComponentsKey];
    
    NSString *locName = [properties objectForKey: PNLocalizedNameKey];
    
    self.localizedName = locName;
    
}


+(PNURLToken*)insertLocalURLTokenWithProperties: (NSDictionary*)properties
                         inManagedObjectContext: (NSManagedObjectContext*) moc
{
    // Defensive programming:
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    PNURLToken *result = [PNURLToken insertInManagedObjectContext: moc];
    
    [result assertProperties: properties];

    if( nil != result ){
        NSURL *uuidDir = [[result localFilePackageURL] URLByDeletingLastPathComponent];
        [PNDocSupportNetController guaranteeDirectoryAtURL: uuidDir];
    }
    [moc saveAnyChanges];

    return result;
}

+(PNCloudURLToken*)insertCloudURLTokenWithProperties: (NSDictionary*)properties
                              inManagedObjectContext: (NSManagedObjectContext*)moc
{
    // Defensive programming:
    NSAssert( (nil != moc), @"Failed to get moc.");

    PNCloudURLToken *result = [PNCloudURLToken insertInManagedObjectContext: moc];
    
    [result assertProperties: properties];
    
    if( nil != result ){
        NSURL *uuidDir = [[result localFilePackageURL] URLByDeletingLastPathComponent];
        [PNDocSupportNetController guaranteeDirectoryAtURL: uuidDir];
        
        uuidDir = [[result cloudFilePackageURL] URLByDeletingLastPathComponent];
        [PNDocSupportNetController guaranteeDirectoryAtURL: uuidDir];
    }
    [moc saveAnyChanges];

    return result;
}
+(PNCloudURLToken*)insertCloudURLTokenWithMetadataItem: (NSMetadataItem*)item
                                inManagedObjectContext: (NSManagedObjectContext*)moc
{
    // Defensive programming:
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    
    NSURL *itemURL = [item valueForAttribute: NSMetadataItemURLKey];
    NSString *testTail = [itemURL lastPathComponent];
    NSAssert( [testTail isEqualToString: (NSString*)PNDocMDataDotPlistKey],
             @"Bogus NSMetadataItem not a document metadata item.");
    

    NSURL *docURL = [itemURL URLByDeletingLastPathComponent];
    [PNDocSupportNetController guaranteeDirectoryAtURL: docURL];
    
    NSDictionary *properties = [self propertiesFromFilePackageURL: docURL];
    
    PNCloudURLToken *result = [PNCloudURLToken insertInManagedObjectContext: moc];
    [result assertProperties: properties];
    result.metadataItem = item;
    
    [moc saveAnyChanges];
    
    return result;
}
+(PNURLToken*)urlTokenForURL: (NSURL*)url
{
    // Shall we build a local or a cloud token?
    // Called only from
    //   -[PNSyncResources visitLocalFilePackageURL:localError:]
    //
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    PNDocSupportNet *dsn = [dsnc docSupportNet];

    NSManagedObjectContext *moc = dsn.managedObjectContext;
    
    PNURLToken *newToken = nil;
    enum PNStorageStatus storageStatus = dsnc.storageStatus;
    NSFileManager *fMngr = [dsnc threadSafeFileManager];
    
    BOOL cloudy =
    (PNStorageStatusCloudAccountIndeterminate == storageStatus) ||
    (PNStorageStatusCloudAccountUnavailable   == storageStatus) ||
    (PNStorageStatusCloudAccountAvailable     == storageStatus);
    
    BOOL isUbiquitous = [fMngr isUbiquitousItemAtURL: url];
    
    BOOL buildCloudToken = cloudy && isUbiquitous;
    
    NSDictionary *urlProperties =
    [PNURLToken propertiesFromFilePackageURL: url];

    if( buildCloudToken ){
        newToken =
        [PNURLToken insertCloudURLTokenWithProperties: urlProperties
                               inManagedObjectContext: moc];
        
    }else{
        newToken =
        [PNURLToken insertLocalURLTokenWithProperties: urlProperties
                               inManagedObjectContext: moc];
    }
    return newToken;
}

-(PNURLToken*)urlTokenWithLocalizedName: (NSString*)newLocalizedName
{
    NSDictionary *original = [[self class] propertiesFromFilePackageURL: self.localFilePackageURL];
    
    NSMutableDictionary *extraCrispy = [original mutableCopy];
    
    [extraCrispy setObject: [PNURLToken nextUuid]
                    forKey: PNUuidStringKey];
    [extraCrispy setObject: newLocalizedName
                    forKey: PNLocalizedNameKey];
    [extraCrispy setObject: self.documentClassName
                    forKey: PNDocumentClassNameKey];
  
    return [PNURLToken insertLocalURLTokenWithProperties: extraCrispy
                             inManagedObjectContext: self.managedObjectContext];
    
}
+(void)deleteURLToken: (PNURLToken*)haplessToken
{
    // Defensive programming:
    NSAssert( (nil != haplessToken), @"nil token.");
    NSAssert( ([haplessToken isMemberOfClass:[PNURLToken class]]),
             @"Bogus token of class %@",
             NSStringFromClass([haplessToken class]) );
    
    NSManagedObjectContext *moc = haplessToken.managedObjectContext;
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    // The substantial consideration:
    NSAssert( (nil == haplessToken.place),
             @"Some place still holds this token.");
    
    [moc deleteObject: haplessToken];

    [moc saveAnyChanges];
}
+(BOOL)deleteURLsAndURLToken: (PNURLToken*)haplessToken
                       error: (NSError**)error
{
    NSManagedObjectContext *moc = haplessToken.managedObjectContext;
    
    NSAssert( (nil != haplessToken), @"nil token.");
    NSAssert( ([haplessToken isKindOfClass:[PNURLToken class]]),
             @"Bogus token of class %@",
             NSStringFromClass([haplessToken class]) );
    
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    NSAssert( (nil == haplessToken.place), @"Some place still holds this token.");
    
    { // nil == haplessToken.place, so this passage is redundant.
        PNDocSupportNetController *dsnc =
        [PNDocSupportNetController sharedDocSupportNetController];
        
        PNDocSupportNet *dsn = [dsnc docSupportNet];
        
        enum PNUuidStatus uuidStatus =
        [dsn statusOfUuid: haplessToken.uuid];
        switch( uuidStatus ){
            case PNUuidStatusUnknown:
            {
                // Expected
                break;
            }
            case PNUuidStatusAssignedCloudURL:
            case PNUuidStatusAssignedLocalURL:
            case PNUuidStatusInvalid:
            case PNUuidStatusUnassignedCloudURL:
            case PNUuidStatusUnassignedLocalURL:
            default:
            {
                NSAssert( NO, @"inconsistent uuid status.");
                break;
            }
        }

    } // nil == haplessToken.place, so this passage is redundant.
    
    
    [haplessToken removeURLsWithError: error];
    
    if( nil == error ){
        [PNURLToken deleteURLToken: haplessToken];
        return YES;
    }else{
        return NO;
    }
}


+(PNURLToken*)demote: (PNCloudURLToken*)cloudToken
               error: (NSError**)error
{
    PNURLToken *localToken = nil; // the result.
    
    // Defensive programming:
    NSAssert( (nil != cloudToken), @"nil token.");
    NSAssert( ([cloudToken isMemberOfClass:[PNCloudURLToken class]]),
             @"Bogus token of class %@",
             NSStringFromClass([cloudToken class]) );
    
    NSManagedObjectContext *moc = cloudToken.managedObjectContext;
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    // The substantial consideration:
    NSAssert( (nil == cloudToken.place),
             @"Some place still holds this token: %@.",
             cloudToken.place.name);
    
    NSMutableDictionary *properties =
    [[PNURLToken propertiesFromFilePackageURL:cloudToken.localFilePackageURL] mutableCopy];
    
    NSString *documentClassName = cloudToken.documentClassName;
    if( (nil != documentClassName ) ){
        [properties setObject: documentClassName
                       forKey: PNDocumentClassNameKey];
    }

    
    localToken =
    [PNURLToken insertLocalURLTokenWithProperties: properties // ignores the value of PNContainerKey
                           inManagedObjectContext: moc];
    
    NSFileManager *fileMgr = [PNDocSupportNetController threadSafeFileManager];
    BOOL success =
    [fileMgr setUbiquitous: NO
                 itemAtURL: cloudToken.localFilePackageURL
            destinationURL: localToken.localFilePackageURL
                     error: error];
    
    if( success && (nil == *error) ){
        
        [moc deleteObject: cloudToken];
        
        [moc saveAnyChanges];

    }

    return localToken;
}


+(PNCloudURLToken*)promote: (PNURLToken*)localToken
                     error: (NSError**)error
{
    PNCloudURLToken *cloudToken = nil; // the result.
    
    // Defensive programming:
    NSAssert( (nil != localToken), @"nil token.");
    NSAssert( ([localToken isMemberOfClass:[PNURLToken class]]),
             @"Bogus token of class %@",
             NSStringFromClass([localToken class]) );
    
    NSManagedObjectContext *moc = localToken.managedObjectContext;
    NSAssert( (nil != moc), @"Failed to get moc.");
    
    // The substantial consideration:
    NSAssert( (nil == localToken.place),
             @"Some place still holds this token: %@.",
             localToken.place.name);
    
    /*
     DEPRECATED COMMENT:
     Note the asymmetry in the demote:error:/promote:error: pair.
     In +demote:error:, we set the source cloud token’s ubiquity to NO;
     in +promote:error: we NEGLECT to set the source local token’s ubiquity to YES.
     It seems that when we open a cloud token’s document a la Sadun,
     the appropriate ubiquity is set to YES as a side effect of its cloud options.
     */
    NSMutableDictionary *properties =
    [[PNURLToken propertiesFromFilePackageURL: localToken.localFilePackageURL] mutableCopy];
    
    NSString *documentClassName = localToken.documentClassName;
    if( (nil != documentClassName ) ){
        [properties setObject: documentClassName
                       forKey: PNDocumentClassNameKey];
    }

    cloudToken =
    [PNCloudURLToken insertCloudURLTokenWithProperties: properties // ignores the value of PNContainerKey
                                inManagedObjectContext: moc];
    
    NSFileManager *fMgr = [PNDocSupportNetController threadSafeFileManager];
    
    BOOL filePackageInTheWay = [fMgr fileExistsAtPath: cloudToken.localFilePackageURL.path];
    
    
    /*
     See:
     FEB 5, 2012
     Getting started with iCloud Storage
     http://samvermette.com/312
     
     "If there’s already a file located at destinationURL, the setUbitquitous:
     call will fail with Cocoa error 516.
     If you want to replace that file, you’ll need to remove the old version first by calling setUbiquitous:NO
     using the iCloud URL as sourceURL and your local sandbox as destinationURL)."
     
     Umm... I think it makes more sense here to try to evict the file that's in the way.
     
     */
    BOOL success = NO;
    
    if( filePackageInTheWay ){
        success = [fMgr evictUbiquitousItemAtURL: cloudToken.localFilePackageURL
                                           error: error];
        
    }
    if( success ){
        NSLog(@"setUbiquitous:YES begin");
        success = [fMgr setUbiquitous: YES
                            itemAtURL: localToken.localFilePackageURL
                       destinationURL: cloudToken.localFilePackageURL
                                error: error];
        NSLog(@"setUbiquitous:YES end");
    }
    
    if( success && (nil == *error) ){
        
        [moc deleteObject: localToken];
        
        [moc saveAnyChanges];
        
    }
    
    return cloudToken;
}


@end
