#import "_PNURLToken.h"

//#import <PNDocSupport/PNURLToken.h>
#import "PNURLToken.h"
// #import <PNDocSupport/PNUuidStatus.h>


extern NSString *PNLocalizedNameKey;
extern NSString *PNUuidStringKey;
extern NSString *PNMidPathComponentsKey;
extern NSString *PNContainerURLKey;

extern NSString *PNDocumentClassNameKey;
extern NSString *PNBaseCaseNumberKey;

extern NSString *PNNullUuidString;

/**
 PNUuidStatus enumerates values that identify the current use of a UUID.
 */
typedef enum PNUuidStatus

{
    PNUuidStatusInvalid = 0, /**< A nil object returns 0 for its status */
    
    PNUuidStatusUnknown = 1, /**< The associated UUID is not currently in use. The unknown status is NOT 0. */
    
    PNUuidStatusUnassignedLocalURL = 1 << 1, /**< A local file package with this UUID is available to be opened as a document.  */
    
    PNUuidStatusUnassignedCloudURL = 1 << 2, /**< A cloud file package with this UUID is available to be opened as a document. */
    
    PNUuidStatusAssignedLocalURL  = 1 << 3, /**< A local file package with this UUID is already opened as a document. */
    
    PNUuidStatusAssignedCloudURL  = 1 << 4  /**< A cloud file package with this UUID is already opened as a document. */
    
} PNUuidStatus;

/**
	PNURLToken wraps a URL suitable for a  file package of UIManagedDocument. It provides a consistent form for document URLs.
 
 
 PNURLToken supports URLs of the form
    file://<base>/<midPath>/<uuid>/<fileName>
 
 where:
 <base> is typically NSDocumentsDirectory()
 <midPath> is typically nil or empty, but can be a set sequence of path components useful to the adopting developer.
 <uuid> a UUID in string form
 <fileName> the document's file name
 
 
 
 */
@interface PNURLToken : _PNURLToken
{

    // required:
    NSArray *m_midPathComponents; // nil, or may determine document class
    NSString *m_uuid;
    NSString *m_localizedName;
    
    // default value is @"UIManagedDocument":
    NSString *m_documentClassName;
    
    //NSURL *m_logicalDocumentURL; // computed from ivars above
    NSURL *m_localFilePackageURL;

}
// Custom logic goes here.

@property (readonly, strong) NSURL *containerURL;

@property (readwrite, strong) NSArray *midPathComponents;
@property (readwrite, strong) NSString *uuid;
@property (readwrite, strong) NSString *localizedName;

@property (readonly, strong) NSDictionary *persistentStoreOptions;

@property (readwrite, strong) NSString *documentClassName; // default is @"UIManagedDocument"

// computed properties:
//@property (readonly, strong) NSString *midPath; // DEPRECATED, last used in -quickLook
@property (readonly, strong) NSDate *date;
@property (readonly, assign) enum PNUuidStatus uuidStatus;

// computed once, lazily instantiated:
//@property (readonly, strong) NSURL *filePackageURL;
@property (readonly, strong) NSURL *localFilePackageURL;


/**
	Returns an array of path components as strings, comprising the file package URL but omitting the base.
 */
@property (readonly, strong) NSArray *relativePathComponents;

/**
	Unused by the static library, possibly useful to adopting developers.
 */
@property (readwrite, strong) NSDictionary  *userInfo;

/**
	For debugging.
	@returns the class and a compressed version of the filePackageURL.
 */
-(NSString*)quickLook;

+(NSString*) nextUuid;
+(BOOL)validUuid: (NSString*)testString;

+(NSDictionary*)propertiesFromFilePackageURL: (NSURL*)url;
-(NSURL*)filePackageURLWithContainer: (NSURL*)container;

-(BOOL)removeURLsWithError:(NSError**)error;
+(NSURL*)containerUrl;

-(NSDictionary*)persistentStoreOptions;
-(NSDictionary*)localPersistentStoreOptions;

@end
