#import "PNURLToken.h"


#import "PNDocSupportNetController.h"
#import "PNDocSupportNet.h"
#import "PNURLTokenPlace.h"
#import "PNCloudURLToken.h"

#import "NSURL+NPAssisting.h"

#import "NSManagedObject+PNPermanentIDing.h"

#import "NSString+PNEllipsis.h"

#import <UIKit/UIKit.h>

const NSString *PNLocalizedNameKey = @"localizedName";
const NSString *PNUuidStringKey = @"uuid";
const NSString *PNMidPathComponentsKey = @"midPathComponents";
const NSString *PNContainerURLKey = @"containerURL";


const NSString *PNDocumentClassNameKey = @"documentClassName";
const NSString *PNBaseCaseNumberKey = @"baseCaseNumber";

const NSString *PNNullUuidString = @"00000000-0000-0000-0000-000000000000";

@interface PNURLToken ()
@property (readwrite, strong) NSURL *containerURL;
//@property (readwrite, strong) NSArray *midPathComponents;
//@property (readwrite, strong) NSString *uuid;
//@property (readwrite, strong) NSString *localizedName;
@property (readwrite, strong) NSDictionary *persistentStoreOptions;

//@property (readwrite, strong) NSString *percentEscapedName;
@end

@implementation PNURLToken

// primary properties from initializers:
@dynamic containerURL; // = m_containerURL;

@dynamic midPathComponents; // = m_midPathComponents;

@dynamic uuid;// = m_uuid;
@dynamic localizedName; // = m_localizedName;

@dynamic documentClassName;


// computed properties:
//@dynamic midPath; // DEPRECATED, last used in -quickLook
@dynamic date;

@dynamic isAvailable;
@dynamic uuidStatus;

// computed once, lazily instantiated:
@dynamic relativePathComponents;
@dynamic localFilePackageURL;

@dynamic persistentStoreOptions;

// optional property (may be nil):
@synthesize userInfo;

-(NSString*)localizedName
{
    return m_localizedName;
}
-(void)setLocalizedName:(NSString *)aName
{
    NSAssert( [aName isKindOfClass:[NSString class]], @"Bogus name not a string: %@", [aName description]);
    NSAssert( (0 != aName.length), @"Bogus empty name: %@", aName);
    m_localizedName = aName;
}
-(NSString*)uuid
{
    return m_uuid;
}
-(void)setUuid:(NSString *)aUuid
{
    NSAssert( [PNURLToken validUuid: aUuid], @"Bogus uuid = %@", aUuid);
    m_uuid = aUuid;
}
-(NSArray*)midPathComponents
{
    return m_midPathComponents;
}
-(void)setMidPathComponents:(NSArray *)someMidPathComponents
{
    NSArray *tmp = nil;
    if( nil == someMidPathComponents ){
        tmp = [NSArray array];
    }else{
        NSAssert( [someMidPathComponents isKindOfClass:[NSArray class]], @"Bogus midPathComponents not an array: %@",
                 [someMidPathComponents description]);
        tmp = someMidPathComponents;
    }
    m_midPathComponents = tmp;
}
//-(NSString*)midPath
//{
//    return [self.midPathComponents componentsJoinedByString: slash];
//    
//}
-(NSDate*)date
{
    NSDate *myDate = [NSDate date];
    
    enum PNURLDeploymentCase myCase = [self.localFilePackageURL deploymentCase];
    
    switch( myCase ){
        case PNURLDeploymentCaseUuidDirSlashExistingFile:
        case PNURLDeploymentCaseUuidDirSlashExistingRWDir:
        {
            NSError *error = nil;
            id x = nil;
            BOOL y =
            [self.localFilePackageURL getResourceValue: &x
                                                forKey: NSURLContentModificationDateKey
                                                 error: &error];
            
            if( y && (nil == error) && [x isKindOfClass:[NSDate class]]  ){
                myDate = x;
            }else{
                myDate = [NSDate distantPast];
            }
            
            break;
        }
        case PNURLDeploymentCaseInvalid:
        case PNURLDeploymentCaseNoUuidDir:
        case PNURLDeploymentCaseUuidDirSlashNoSuchFileOrDirectory:
        case PNURLDeploymentCaseDefective:
        default:
        {
            // The URL points to a file that does not yet exist.
            // myDate = [NSDate date];
            break;
        }
            
    }
    return myDate;
}

-(NSArray*)relativePathComponents
{
    NSMutableArray *rpComponents = [self.midPathComponents mutableCopy];
    [rpComponents addObject: self.uuid];
    [rpComponents addObject: self.localizedName];
    return  rpComponents;
}
-(NSURL*)filePackageURLWithContainer: (NSURL*)container
{
    NSMutableArray *components = [[container pathComponents] mutableCopy];
    [components addObjectsFromArray:[self relativePathComponents]];
    
    NSURL *result = [NSURL fileURLWithPathComponents: components];
    return [result pnNormalizedURL];
}

-(NSURL*)localFilePackageURL
{
    if( nil == m_localFilePackageURL ){
        
        // Works for PNURLToken and PNCloudURLToken:
        m_localFilePackageURL =
        [[self filePackageURLWithContainer:[PNURLToken containerUrl]] copy];
        
        // When we call
        //    [[UIDocument alloc] initWithFileURL: foo];
        // the first time on "foo", foo is a file URL.
        // Thereafter, it's a directory.
        
        NSAssert( [PNDocSupportNetController guaranteeDirectoryAtURL: [m_localFilePackageURL URLByDeletingLastPathComponent]],
                 @"Programming error: -localFilePackageURL");
        
        NSLog(@"-[%@ localFilePackageURL] = %@",
              NSStringFromClass([self class]),
              m_localFilePackageURL.absoluteString );
        
        /*
         Examples:
         
         -[PNCloudURLToken filePackageURL] = file://localhost/private/var/mobile/Library/Mobile%20Documents/TEAMCODE10~com~myCompany~TaggedLocationsDemo/Documents/4BAB95BD-EDB9-4A53-808A-3A9E7F2D0BB3/Untitled
         
         */
    }
    return m_localFilePackageURL;
}

//
//-(NSString*)localizedName
//{
//    NSString *result =
//    [self.percentEscapedName stringByReplacingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
//    return result;
//}
//-(void)setLocalizedName:(NSString *)displayName
//{
//    NSString *result =
//    [displayName stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
//    self.percentEscapedName = result;
//}
-(NSString *)documentClassName
{
    if( nil == m_documentClassName ){
        m_documentClassName = @"UIManagedDocument";
    }
    return m_documentClassName;
}
-(void)setDocumentClassName:(NSString *)aDocumentClassName
{
    NSString *tmp = nil;
    if( nil == aDocumentClassName ){
        tmp = @"UIManagedDocument"; 
    }else{
        tmp = aDocumentClassName;
    }
    
    BOOL valid =
    [NSClassFromString(tmp) isSubclassOfClass: [UIManagedDocument class]];
    
    if( !valid ){
        NSLog( @"bogus bogus documentClassName" );
    }
    
    NSAssert( valid, @"bogus documentClassName (%@) does not inherit from UIManagedDocument", aDocumentClassName);
    
    m_documentClassName = tmp;
}
-(enum PNUuidStatus)uuidStatus
{
    PNPlace *place = self.place;
    if( nil == place ) return PNUuidStatusUnknown;
    
    NSAssert([place isKindOfClass: [PNURLTokenPlace class]],
             @"token belongs to bogus place: %@",
             [place description] );
    
    return ((PNURLTokenPlace*)place).uuidStatus;
}

-(NSString*)quickLook
{
    NSMutableString *result = [NSMutableString stringWithCapacity: 128];
    [result appendFormat: @"(%@*) ", NSStringFromClass([self class])];
    
    NSString *mp = [self.midPathComponents componentsJoinedByString: slash];
    [result appendFormat: @" %@/%@/%@",
     [mp pnEllipsis],
     [self.uuid pnEllipsis],
     self.localizedName];
    return result;
}

+(NSString*) nextUuid
{
    // See: http://iphonedevelopertips.com/general/create-a-universally-unique-identifier-uuid.html
    CFUUIDRef puuid = CFUUIDCreate( nil );
    CFStringRef uuidString = CFUUIDCreateString( nil, puuid );
    
    NSString *result = [NSString stringWithString: (__bridge NSString *)uuidString];
    CFRelease(puuid);
    CFRelease(uuidString);
    
    // The result is 128-bits.
    
    // If you change -nextUuid or -validUuid:,
    // run this sanity check briefly:
    NSAssert( [self validUuid: result], @"Sanity check failed.");
    
    return result;
}

+(BOOL)validUuid: (NSString*)testString
{
    NSUUID *test = [[NSUUID alloc] initWithUUIDString: testString];
    return (nil != test);
}

+(NSSet*)requiredPropertyKeys
{
    static NSSet *mm_requiredProperties = nil;
    if( nil == mm_requiredProperties ){
        NSArray *tmp = [NSArray arrayWithObjects:
                        PNContainerURLKey,
                        PNUuidStringKey,
                        PNLocalizedNameKey,
                        nil];
        mm_requiredProperties = [[NSSet alloc] initWithArray: tmp];
    }
    return mm_requiredProperties;
}

+(NSDictionary*)propertiesFromFilePackageURL: (NSURL*)rawUrl
{
    /*
     This method must return identical results for two use cases:
     [1] file://localhost/var/mobile/Applications/yadda/yadda
     [2] file://localhost/private/var/mobile/Applications/yadda/yadda
     
     */
    
    NSURL *url = [rawUrl pnNormalizedURL];
    
    NSMutableArray *remainingComponents = [[url pathComponents] mutableCopy];
    if( 2 >= remainingComponents.count ){
        return nil;
    }
    
    //    NSString *urlString  = [url absoluteString];
    
    NSMutableDictionary *result = [NSMutableDictionary dictionaryWithCapacity:3];
    NSURL *container = nil;
    
    BOOL validBase = NO;
    
    // [1] Identify the container URL:
    PNUrlBaseCase baseCase = [url baseCase];
    switch( baseCase ){
        case PNUrlBaseCaseCloud:
        {
            //            NSURL *cloudBase = [NPCloudURLToken containerUrl];
            //            NSString *cloudBaseString = [cloudBase absoluteString];
            //
            //            hasBaseAsPrefix = [urlString hasPrefix: cloudBaseString];
            //            if( hasBaseAsPrefix ){
            //                container = cloudBase;
            //            }
            validBase = YES;
            container = [PNCloudURLToken containerUrl]; //
            break;
        }
        case PNUrlBaseCaseSimulator:
        case PNUrlBaseCaseLocal:
        {
            //            NSURL *localBase = [NPURLToken containerUrl];
            //            NSString *localBaseString = [localBase absoluteString];
            //
            //            hasBaseAsPrefix = [urlString hasPrefix: localBaseString];
            //            if( hasBaseAsPrefix ){
            //                container = localBase;
            //            }
            validBase = YES;
            container = [PNURLToken containerUrl];
            break;
        }
        case PNUrlBaseCaseInvalid:
        {
            result = nil;
            container = nil; // redundant
            validBase = NO; // redundant
            break;
        }
    }
    
    // [2] Build the result:
    if( validBase ){
        
        // [3] get the container:
        [result setObject: container
                   forKey: PNContainerURLKey];
        
        NSArray *containerComponents = [container pathComponents];
        
        NSRange containerRange = NSMakeRange(0, containerComponents.count);
        
        [remainingComponents removeObjectsInRange: containerRange];
        
        // [4] get the localized name:
        NSString *localizedName = [remainingComponents lastObject];
        [result setObject: localizedName
                   forKey: PNLocalizedNameKey];
        [remainingComponents removeLastObject];
        
        // [5] get uuid string:
        NSString *testUuid = [remainingComponents lastObject];
        BOOL validUuid = [PNURLToken validUuid: testUuid];
        
        if( validUuid ){
            [result setObject: testUuid
                       forKey: PNUuidStringKey];
            [remainingComponents removeLastObject];
            
            // [6] get mid path components:
            [result setObject: [remainingComponents copy]
                       forKey: PNMidPathComponentsKey];
        }else{
            result = nil;
        }
        
        return result;
    }else{
        // !hasBaseAsPrefix
        return nil;
    }
}


-(BOOL)removeURLsWithError:(NSError**)error
{
    NSURL *haplessURL = [self localFilePackageURL];
    
    NSURL *haplessUuidDir = [haplessURL URLByDeletingLastPathComponent];
    
    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    BOOL success = [fileMngr removeItemAtURL:haplessUuidDir
                                       error:error];
    
    return success;
}
-(NSURL*)containerUrl
{
    return [[self class] containerUrl];
}
+(NSURL*)containerUrl
{
    static NSURL *mm_containerUrl = nil;
    if( nil == mm_containerUrl ){
        
        NSFileManager *tsfm = [PNDocSupportNetController threadSafeFileManager];
        NSURL *result =
        [tsfm URLForDirectory:NSDocumentDirectory
                     inDomain:NSUserDomainMask
            appropriateForURL:nil
                       create:YES
                        error:nil];
        
        mm_containerUrl = [[result pnNormalizedURL] copy];
        
        NSLog(@"+[%@ containerUrl] = %@",
              NSStringFromClass(self),
              mm_containerUrl );
    }
    
    return mm_containerUrl;
}
+(NSDictionary*)localPersistentStoreOptions
{
    /* See: https://developer.apple.com/library/ios/#releasenotes/DataManagement/RN-iCloudCoreData/_index.html#//apple_ref/doc/uid/TP40011298
     
     [1] If you use a SQLite store, you need to provide a unique identifier for the store. You set the identifier using the NSPersistentStoreUbiquitousContentNameKey in the setPersistentStoreOptions: dictionary for the managed document.
     
     [2] You should also provide a ubiquitous content URL using the NSPersistentStoreUbiquitousContentURLKey.
     (The default URL is based on the bundle identifier. If you provide a custom URL for the transaction logs, you can share documents across applications with different identifiers. Typically you specify a directory called TransactionLogs in the ubiquity container.)
     So that you can retrieve it later if necessary (for example, when deleting a document), UIManagedDocument saves the store’s ubiquitous content name in a property list file named DocumentMetadata.plist in the document package.
     
     Note:
     Transaction logs for a document are stored in a directory:
     
     <Value of NSPersistentStoreUbiquitousContentURLKey>/<Value of NSPersistentStoreUbiquitousContentNameKey>.
     
     */
    
    static NSDictionary *mm_localPersistentStoreOptions = nil;
    if( nil == mm_localPersistentStoreOptions ){
        
        NSMutableDictionary *storeOptions =
        [NSDictionary dictionaryWithObjectsAndKeys:
         [NSNumber numberWithBool:YES],  NSMigratePersistentStoresAutomaticallyOption,
         [NSNumber numberWithBool:YES],  NSInferMappingModelAutomaticallyOption,
         nil];
        
        mm_localPersistentStoreOptions = [storeOptions copy];
    }
    return mm_localPersistentStoreOptions;
}
-(NSDictionary*)localPersistentStoreOptions
{
    return [[self class] localPersistentStoreOptions];
}
-(NSDictionary*)persistentStoreOptions
{
    // In this static library,
    // the method -[PNDocToken nqd_allocInitDocument] calls this method.

    return [[self class] localPersistentStoreOptions];
}

@end
