//
//  UIManagedDocument+PNDocTokenCallbackSupporting.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 4/25/12.
//  Copyright (c) 2012 NowPicture LLC. All rights reserved.
//

#import "UIManagedDocument+PNDocTokenCallbackSupporting.h"

#import "PNDocToken+CallbackSupporting.h"

#import "PNCloudURLToken.h"


@implementation UIManagedDocument (PNDocTokenCallbackSupporting)

/*
 
 This category just provides the basic UIDocument file operations, but with a PNDocToken argument added.
 The PNDocToken argument manages the callback.
 
 */ 
- (void)openWithToken: (PNDocToken*)docToken
{
    NSLog(@"[self openWithToken:] BEGIN ");
    NSLog(@"state = %@",
          docToken.documentStateAsString);

    [self openWithCompletionHandler: ^(BOOL success){
        
        NSLog(@"[self openWithToken:] in Handler, ");
        NSLog(@"state = %@",
              docToken.documentStateAsString);

        [docToken didOpenSuccessfully: success];
    }];
}

- (void)saveToURL:(NSURL *)url 
 forSaveOperation:(UIDocumentSaveOperation)saveOperation 
            token: (PNDocToken*)docToken
{
    self.fileModificationDate = [NSDate date];
    
    [self saveToURL: url
   forSaveOperation: saveOperation
  completionHandler: ^(BOOL success){
      
      /**
       I'm finding rather long latencies of tens of seconds
       for creating cloud-persistent documents.
       */
      [docToken didSaveSuccessfully: success 
                   forSaveOperation: saveOperation];
  }];
}

-(void)revertToContentsOfURL: (NSURL *)url
                       token: (PNDocToken*)docToken
{    
    [self revertToContentsOfURL: url 
              completionHandler:^(BOOL success){
                  
                  [docToken didRevertSuccessfully: success
                                  toContentsOfURL: url];      
              }];
    
}

-(void)closeWithToken: (PNDocToken*)docToken
{
    //[self.managedObjectContext processPendingChanges];
    /*
     You call this method (-closeWithCompletionHandler:) 
     to begin the sequence of method calls 
     that saves a document safely and asynchronously. 
     */
    [self closeWithCompletionHandler: ^(BOOL success){
        [docToken didCloseSuccessfully: success];
    }];
}

@end

