#import "_PNCreateDoc.h"

@class PNDocToken, PNDocTokenPlace, PNArcT2P;

#import "PNDocumentCreating.h"
#import "PNDocTokenMarking.h"

/**
 PNCreateDoc creates a PNDocToken from the parameters of its -enableFor:... method.
	The -transform calls  -createAndSaveDocument.  */
@interface PNCreateDoc : _PNCreateDoc
{
    NSURL *m_permID_createdURLToken;
    NSURL *m_permID_createdDocToken;
}
// Custom logic goes here.


// Arguments:
@property (readonly, strong) NSString *localizedName;
@property (readonly, strong) NSString *documentClassName;
@property (readonly, strong) NSArray *midPathComponents;
@property (readonly, strong) id <PNDocumentCreating> creationDelegate;

@property (readonly, strong) PNURLToken *createdURLToken;
@property (readonly, strong) PNDocToken *createdDocToken;


/**
 
 The creation delegate can initialize the new document's object graph.
 On success, the created doc token's url token goes to the appropriate assigned place (local or cloud). The new doc token goes to toPlace.
 
 @param transitionDelegate the transition delegate to inform of success or failure
 @param aLocalizedName a name for the document to be created. Needs not be unique--certainly not across devices.
 @param documentClassName nil, @"UIManagedDocument" or the name of a subclass. If nil, the creationDelegate will receive -documentClassNameForUrlToken: (called from nqd_allocInitDocumentWithLocalURL).
 @param midPathComponents nil, an empty array, or an array of strings suitable for path components of a document's file package URL.
 @param creationDelegate the creation delegate initializes the object graph of the newly instantiated document.
 @param toPlace the place that will receive the new doc token on success of the transition.
 
 */
-(void)          enableFor: (id<PNTransitionDelegate>)transitionDelegate
     documentLocalizedName: (NSString*)aLocalizedName
         documentClassName: (NSString*)documentClassName
         midPathComponents: (NSArray*)aMidPath
          creationDelegate: (id <PNDocumentCreating>)creationDelegate
                   toPlace: (PNDocTokenPlace*)toPlace;

@end
