#import "_PNDocTokenTransition.h"



#import "PNURLTokensTransition.h"
#import "PNTransition+Transforming.h"
#import "PNDocTokenMarking.h"


/**
 These keys that may appear in the info argument of -docTokenOperationDidFail:info:

 See: -(NSDictionary*)statusOfFailedOperation: (SEL)cmd
 
 */
extern const NSString *PNDocTokenFailedOperationKey;
extern const NSString *PNDocTokenFailedOperatorKey;
extern const NSString *PNDocTokenFailedStatusKey;
extern const NSString *PNDocTokenFailedNestedInfoKey;
extern const NSString *PNDocTokenFailedOperationLogKey;

@class PNDocToken, PNDocTokenPlace;


@interface PNDocTokenTransition : _PNDocTokenTransition
    <PNDocTokenMarking>
{
    // Fixed, internal:
    NSURL *m_permID_innerPlace;
    
    // Fixed, lazily instantitated:
    // {local, cloud} x {unassigned, assigned} x {from, to}
    NSURL *m_permID_arcFromUnassignedLocal;
    NSURL *m_permID_arcFromUnassignedCloud;

    NSURL *m_permID_arcToUnassignedLocal;
    NSURL *m_permID_arcToUnassignedCloud;

    NSURL *m_permID_arcFromAssignedLocal;
    NSURL *m_permID_arcFromAssignedCloud;
    
    NSURL *m_permID_arcToAssignedLocal;
    NSURL *m_permID_arcToAssignedCloud;
    
    // Transient: see -disconnectTransientArcs
    NSURL *m_permID_destinationPlace;
    NSURL *m_permID_arcToDestination;

    NSURL *m_permID_arcFromSource;
    NSURL *m_permID_sourcePlace;

}
// Custom logic goes here.
@property (readwrite, strong) NSSet *unassignedIn;
@property (readwrite, strong) NSSet *unassignedOut;

@property (readwrite, strong) NSSet *assignedIn;
@property (readwrite, strong) NSSet *assignedOut;



// Convenience properties:

// Arguments:
@property (readwrite, strong) PNDocToken *docToken;
@property (readonly,  strong) PNDocTokenPlace *innerPlace;
@property (readwrite, strong) NSString *sourceUuid;
@property (readwrite, strong) NSString *destinationUuid;


// Fixed:
@property (readonly, strong) PNArcP2T *arcFromUnassignedLocal;
@property (readonly, strong) PNArcP2T *arcFromUnassignedCloud;

@property (readonly, strong) PNArcT2P *arcToUnassignedLocal;
@property (readonly, strong) PNArcT2P *arcToUnassignedCloud;

@property (readonly, strong) PNArcP2T *arcFromAssignedLocal;
@property (readonly, strong) PNArcP2T *arcFromAssignedCloud;

@property (readonly, strong) PNArcT2P *arcToAssignedLocal;
@property (readonly, strong) PNArcT2P *arcToAssignedCloud;


// Transient:
@property (readonly, strong) PNDocTokenPlace *destinationPlace;
@property (readonly, strong) PNDocTokenPlace *sourcePlace;

@property (readonly, strong) PNArcP2T *arcFromSource;
@property (readonly, strong) PNArcT2P *arcToDestination;


-(void)disconnectTransientArcs;

-(void)launchTransform; // call a PNDocToken (FileOperations) method,
// expect -docTokenDidBecomeAvailable: or -docToken:didFailWithError:

@end

@interface PNDocTokenTransition (Transforming)
-(void)assertInputGuardExpressions;
-(void)gatherDocToken;
-(void)gatherURLTokens;

-(void)assertOutputGuardExpressions;
-(void)scatterDocToken;
-(void)scatterURLTokens;
@end
