#import "PNDocTokenTransition.h"

#import "PNDocTokenPlace.h"
#import "PNURLTokenPlace.h"

#import "PNDocToken.h"

#import "NSManagedObject+PNPermanentIDing.h"

#import "PNDocToken+FileOperations.h"

#import "PNArcT2P.h"

#import "PNDocSupportNet.h"

#import "PNCloudURLToken.h"

#import "PNDocToken+Managing.h"


// Keys that may appear in the info argument of -docTokenOperationDidFail:info:
const NSString *PNDocTokenFailedOperationKey = @"PNDocToken Failed Operation";
const NSString *PNDocTokenFailedOperatorKey = @"PNDocToken Failed Operator";
const NSString *PNDocTokenFailedStatusKey = @"PNDocToken Failed Status";
const NSString *PNDocTokenFailedNestedInfoKey = @"PNDocToken Failed Message";
const NSString *PNDocTokenFailedOperationLogKey = @"PNDocToken Failed Operation Log";


//@interface PNDocTokenTransition ()
//@property (readwrite, strong) NSSet *unassignedIn;
//@property (readwrite, strong) NSSet *unassignedOut;
//
//@property (readwrite, strong) NSSet *assignedIn;
//@property (readwrite, strong) NSSet *assignedOut;
//
//@end

@implementation PNDocTokenTransition


// Custom logic goes here.

@synthesize  unassignedIn, unassignedOut, assignedIn, assignedOut;

@synthesize docToken;
@synthesize sourceUuid, destinationUuid;
@dynamic innerPlace;

@dynamic arcFromUnassignedLocal;
@dynamic arcFromUnassignedCloud;
@dynamic arcToUnassignedLocal;
@dynamic arcToUnassignedCloud;

@dynamic arcFromAssignedLocal;
@dynamic arcFromAssignedCloud;
@dynamic arcToAssignedLocal;
@dynamic arcToAssignedCloud;

@dynamic sourcePlace;
@dynamic destinationPlace;


#pragma mark PNDocTokenMarking methods, called on the Main Thread
-(void)docTokenDidBecomeAvailable: (PNDocToken*)aDocToken
{
    NSAssert( (nil == aDocToken.error),
             @"-[%@ docTokenDidBecomeAvailable:] cross-wired error.",
             NSStringFromClass( [self class]) );

    NSAssert( (self.docToken == aDocToken),
             @"-[%@ docTokenDidBecomeAvailable:] cross-wired doc token.",
             NSStringFromClass( [self class]) );
  
    m_error = nil;
    
    [super transform]; // calls -didTransform on the main thread
}
-(void) docToken: (PNDocToken*)aDocToken
didFailWithError: (NSError*)error
{
    NSAssert( (nil != aDocToken.error),
             @"-[%@ docToken:didFailWithError:] cross-wired error.",
             NSStringFromClass( [self class]) );

    NSAssert( (self.docToken == aDocToken),
             @"-[%@ docToken:didFailWithError:] cross-wired doc token.",
             NSStringFromClass( [self class]) );
    
    m_error = error;
    
    [super transform]; // calls -didTransform on the main thread
}

-(void)didAddDocToken:(PNDocToken *)token
{
    
}
-(void)willRemoveDocToken:(PNDocToken *)token
{
    
}
-(void)launchTransform
{
    NSAssert( NO, @"Programming error: Abstract parent implements a crashin stub.");

    // callbacks: -docTokenDidBecomeAvailable: or -docToken:didFailWithError:
    // Call [super transform] from -docTokenDidBecomeAvailable: or -docToken:didFailWithError:.
}

-(PNDocTokenPlace*)innerPlace
{
    PNDocTokenPlace *p = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_innerPlace ){
        p = [PNDocTokenPlace insertInManagedObjectContext: moc];
        {
            p.capacityValue = 1;
            p.name = [NSString stringWithFormat: @"%@ inner place", self.name];
            p.marker = self;
        }
        [moc saveAnyChanges];
        m_permID_innerPlace = [p permanentID];
    }
    p = (PNDocTokenPlace*)[moc managedObjectForPermanentID: m_permID_innerPlace];
    return p;
}

-(PNArcP2T*)arcFromUnassignedLocal
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromUnassignedLocal ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedLocal =
            self.docSupportNet.unassignedLocalPlace;
            arc.outTransition = self;
            arc.inPlace = unassignedLocal;
            arc.targetTokenEntityName = @"PNURLToken";
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [moc saveAnyChanges];
        m_permID_arcFromUnassignedLocal = [arc permanentID];
    }
    arc = (PNArcP2T*)[moc managedObjectForPermanentID: m_permID_arcFromUnassignedLocal];
    return arc;
}
-(PNArcP2T*)arcFromUnassignedCloud
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromUnassignedCloud ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedCloud =
            self.docSupportNet.unassignedCloudPlace;
            arc.outTransition = self;
            arc.inPlace = unassignedCloud;
            arc.targetTokenEntityName = @"PNCloudURLToken";
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [moc saveAnyChanges];
        m_permID_arcFromUnassignedCloud = [arc permanentID];
    }
    arc = (PNArcP2T*)[moc managedObjectForPermanentID: m_permID_arcFromUnassignedCloud];
    return arc;
}

-(PNArcT2P*)arcToUnassignedLocal
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToUnassignedLocal ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedLocal = self.docSupportNet.unassignedLocalPlace;
            arc.inTransition = self;
            arc.outPlace = unassignedLocal;
            arc.targetTokenEntityName = @"PNURLToken";
        }
        [moc saveAnyChanges];
        m_permID_arcToUnassignedLocal = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToUnassignedLocal];
    return arc;
}
-(PNArcT2P*)arcToUnassignedCloud
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToUnassignedCloud ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedCloud = self.docSupportNet.unassignedCloudPlace;
            arc.inTransition = self;
            arc.outPlace = unassignedCloud;
            arc.targetTokenEntityName = @"PNCloudURLToken";
        }
        [moc saveAnyChanges];
        m_permID_arcToUnassignedCloud = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToUnassignedCloud];
    return arc;
}

-(PNArcP2T*)arcFromAssignedLocal
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromAssignedLocal ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *assignedLocal =
            self.docSupportNet.assignedLocalPlace;
            arc.outTransition = self;
            arc.inPlace = assignedLocal;
            arc.targetTokenEntityName = @"PNURLToken";
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [moc saveAnyChanges];
        m_permID_arcFromAssignedLocal = [arc permanentID];
    }
    arc = (PNArcP2T*)[moc managedObjectForPermanentID: m_permID_arcFromAssignedLocal];
    return arc;
}
-(PNArcP2T*)arcFromAssignedCloud
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromAssignedCloud ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *assignedCloud =
            self.docSupportNet.assignedCloudPlace;
            arc.outTransition = self;
            arc.inPlace = assignedCloud;
            arc.targetTokenEntityName = @"PNCloudURLToken";
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [moc saveAnyChanges];
        m_permID_arcFromAssignedCloud = [arc permanentID];
    }
    arc = (PNArcP2T*)[moc managedObjectForPermanentID: m_permID_arcFromAssignedCloud];
    return arc;
}

-(PNArcT2P*)arcToAssignedLocal
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToAssignedLocal ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *assignedLocal = self.docSupportNet.assignedLocalPlace;
            arc.inTransition = self;
            arc.outPlace = assignedLocal;
            arc.targetTokenEntityName = @"PNURLToken";
        }
        [moc saveAnyChanges];
        m_permID_arcToAssignedLocal = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToAssignedLocal];
    return arc;
}
-(PNArcT2P*)arcToAssignedCloud
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToAssignedCloud ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *assignedCloud = self.docSupportNet.assignedCloudPlace;
            arc.inTransition = self;
            arc.outPlace = assignedCloud;
            arc.targetTokenEntityName = @"PNCloudURLToken";
        }
        [moc saveAnyChanges];
        m_permID_arcToAssignedCloud = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToAssignedCloud];
    return arc;
}
-(PNArcT2P*)arcToDestination
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToDestination ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            arc.inTransition = self;
            arc.outPlace = self.destinationPlace;
            arc.targetTokenEntityName = @"PNDocToken";
        }
        [moc saveAnyChanges];
        m_permID_arcToDestination = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToDestination];
    return arc;
}

-(PNArcP2T*)arcFromSource
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromSource ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            arc.inPlace = self.sourcePlace;
            arc.outTransition = self;
            arc.targetTokenEntityName = @"PNDocToken";
            arc.uuids = nil;
        }
        [moc saveAnyChanges];
        m_permID_arcFromSource = [arc permanentID];
    }
    arc = (PNArcP2T*)[moc managedObjectForPermanentID: m_permID_arcFromSource];
    return arc;
}

-(PNDocTokenPlace*)sourcePlace
{
    NSManagedObjectContext *moc = self.managedObjectContext;
    return (PNDocTokenPlace*)[moc managedObjectForPermanentID: m_permID_sourcePlace];
}
-(void)disconnectArcFromSource
{
    PNArcP2T *arc = self.arcFromSource;
    
    if( nil == arc ) return; // subclasses may return nil;
    
    [self.destinationPlace removeOutArcsObject: arc];
    [self removeInArcsObject: arc];
    
    NSManagedObjectContext *moc = self.managedObjectContext;
    [moc deleteObject: arc];
    m_permID_arcFromSource = nil;
    [moc saveAnyChanges];
}

//-(void)setSourcePlace:(PNDocTokenPlace *)aSourcePlace
//{
//    [self disconnectArcFromSource];
//    m_permID_sourcePlace = [aSourcePlace permanentID];
//}
-(void)disconnectArcToDestination
{
    PNArcT2P *arc = self.arcToDestination;
    
    if( nil == arc ) return; // subclasses may return nil;
    
    [self.destinationPlace removeInArcsObject: arc];
    [self removeOutArcsObject: arc];
        
    NSManagedObjectContext *moc = self.managedObjectContext;
    [moc deleteObject: arc];
    m_permID_arcToDestination = nil;
    [moc saveAnyChanges];
}

-(PNDocTokenPlace*)destinationPlace
{
    NSManagedObjectContext *moc = self.managedObjectContext;
    return (PNDocTokenPlace*)[moc managedObjectForPermanentID: m_permID_destinationPlace];
}
//-(void)setDestinationPlace:(PNDocTokenPlace *)aDestinationPlace
//{
//    [self disconnectArcToDestination];
//    m_permID_destinationPlace = [aDestinationPlace permanentID];
//}


-(void)disconnectTransientArcs
{
    if( nil != self.arcFromSource ){
        [self disconnectArcFromSource];
    }
    if( nil != self.arcToDestination ){
        [self disconnectArcToDestination];
    }
    
    [self.innerPlace clear];
}

-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
{
    self.unassignedIn  = [NSSet set];
    self.assignedIn    = [NSSet set];

    self.unassignedOut = [NSSet set];
    self.assignedOut   = [NSSet set];
    
    [self.innerPlace clear];
    
    [super enableFor: transitionDelegate];
}
@end

@implementation PNDocTokenTransition (Transforming)

-(void)assertInputGuardExpressions
{
    
    NSSet *sourceUuidSet = nil;
    if( nil != self.sourceUuid ){
        sourceUuidSet = [NSSet setWithObject: self.sourceUuid];
    }
    
    self.arcFromSource.uuids = sourceUuidSet;
    
    self.arcFromUnassignedLocal.uuids = sourceUuidSet;
    self.arcFromUnassignedCloud.uuids = sourceUuidSet;
    
    self.arcFromAssignedLocal.uuids = sourceUuidSet;
    self.arcFromAssignedCloud.uuids = sourceUuidSet;
   
}
-(void)gatherTokens
{
    [self assertInputGuardExpressions];
    [self gatherURLTokens];
    [self gatherDocToken];
}

-(void)gatherDocToken
{
    // PNCreateDoc and PNOpenDoc override this method.
    if( nil != m_permID_sourcePlace ){
        
        NSSet *incoming = [self.arcFromSource pullTokens];
        NSAssert( (1 == (incoming.count) ),
                 @"Should receive exactly one PNDocToken.");
        
        self.docToken = [incoming anyObject];
        self.docToken.isAvailableValue = YES; // available a priori

        [self.innerPlace addToken: self.docToken];
        
    }
}

-(void)gatherURLTokens
{
    NSLog(@"-[%@ gatherURLTokens]", NSStringFromClass([self class]));
    
    NSLog(@"BEFORE: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);

    // Of each {local, cloud} pair of arcs, only one actually pulls any token(s).
    // They have complementary guard expressions for the entities they permit.

    NSMutableSet *fromUnassigned = [NSMutableSet setWithCapacity:1];
    
    [fromUnassigned unionSet: [self.arcFromUnassignedLocal pullTokens]];
    [fromUnassigned unionSet: [self.arcFromUnassignedCloud pullTokens]];
    
    self.unassignedIn = [fromUnassigned copy];
    
    NSMutableSet *fromAssigned = [NSMutableSet setWithCapacity:1];
    
    [fromAssigned unionSet: [self.arcFromAssignedLocal pullTokens]];
    [fromAssigned unionSet: [self.arcFromAssignedCloud pullTokens]];
    
    self.assignedIn = [fromAssigned copy];

    NSLog(@"AFTER: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);
}

-(void)transform
{
    [self launchTransform]; // expect a callback to either:
    // [1] -docTokenDidBecomeAvailable: or
    // [2] -docToken:didFailWithError:
}
-(void)didTransform
{
    [super didTransform];
    [self disconnectTransientArcs];
}


-(void)scatterTokens
{
    [self assertOutputGuardExpressions];
    [self scatterURLTokens];
    [self scatterDocToken];
}
-(void)assertOutputGuardExpressions
{
    NSSet *destUuidSet = nil;
    if( nil != self.destinationUuid){
        destUuidSet = [NSSet setWithObject: self.destinationUuid];
    }
    
    self.arcToDestination.uuids = destUuidSet;
    
    self.arcToUnassignedLocal.uuids = destUuidSet;
    self.arcToUnassignedCloud.uuids = destUuidSet;
    
    self.arcToAssignedLocal.uuids = destUuidSet;
    self.arcToAssignedCloud.uuids = destUuidSet;
}

-(void)scatterDocToken
{
    // PNCloseDoc overrides this method.
    if( nil != self.destinationPlace ){
        
        PNDocToken *resultDocToken = self.docToken;
        [self.innerPlace removeToken: resultDocToken];
        
        PNDocTokenPlace *p = (PNDocTokenPlace*)(self.destinationPlace);
        [p addToken: resultDocToken];
        [p.marker docTokenDidBecomeAvailable:resultDocToken];

    }
    [self.innerPlace clear];

}
-(void)scatterURLTokens
{
    NSLog(@"-[%@ scatterURLTokens]", NSStringFromClass([self class]));
    
    NSLog(@"BEFORE: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);

    // Of each {local, cloud} pair of arcs, only one actually pushes the token(s).
    // They have complementary guard expressions for the entities they permit.
    
    [self.arcToUnassignedLocal pushTokens: self.unassignedOut];
    [self.arcToUnassignedCloud pushTokens: self.unassignedOut];
    
    [self.arcToAssignedLocal pushTokens: self.assignedOut];
    [self.arcToAssignedCloud pushTokens: self.assignedOut];

    NSLog(@"AFTER: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);
}


-(void)scatterURLTokensSimpler
{
    NSLog(@"-[%@ scatterURLTokensSimpler]", NSStringFromClass([self class]));
    
    NSLog(@"BEFORE: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);
    
    // Of each {local, cloud} pair of arcs, only one actually pushes the token(s).
    // They have complementary guard expressions for the entities they permit.
    
    NSSet *outputURLTokens = nil; // should be a property, filled in -transform
        
    for( PNURLToken *next in outputURLTokens){
        if( [next isKindOfClass: [PNCloudURLToken class]] ){
            
            if( nil == next.docToken ){
                [self.arcToUnassignedCloud pushToken: next];
            }else{
                [self.arcToAssignedCloud pushToken: next];
            }
        }else{
            if( nil == next.docToken ){
                [self.arcToUnassignedLocal pushToken: next];
            }else{
                [self.arcToAssignedLocal pushToken: next];
            }
            
        }
    }
    [self.arcToUnassignedLocal pushTokens: self.unassignedOut];
    [self.arcToUnassignedCloud pushTokens: self.unassignedOut];
    
    [self.arcToAssignedLocal pushTokens: self.assignedOut];
    [self.arcToAssignedCloud pushTokens: self.assignedOut];
    
    NSLog(@"AFTER: docSupportNet.quickLook = \n%@",
          [[self docSupportNet] quickLook]);
}

@end
