#import "_PNPipeline.h"

#import "PNTransitionDelegate.h"



/**
	PNPipeline is a subclass of PNDocTokenTransition. It encapsulates a series of internal transitions. PNPipeline is the abstract parent of:
 PNCloseCreate
 PNCloseOpen
 PNCloseFlipOpen (speculative and untested, circa 2013 mid-April)
 
 Among iOS apps, some follow a user experience pattern that strives to keep some document open and accessible to the user. PNPipeline serves those apps.
 
 */
@interface PNPipeline : _PNPipeline
    <PNTransitionDelegate>
{
    // In this implementation, each component occurs once,sequentially.
    // No looping.
    NSMutableOrderedSet *m_componentTransitions;
}

// Custom logic goes here.
@property (readonly, weak) id <PNTransitionDelegate> outerTransitionDelegate;

// In -didFailTransition:rolledBackSuccessfully:
// the delegate may inspect these properties for damage-control purposes.
@property (readonly, strong) PNTransition *currentPipelinedTransition;
@property (readonly, strong) NSOrderedSet *componentTransitions;
@property (readonly, assign) NSUInteger currentIndex;

// For <PNTransitionDelegate>
@property (readwrite, strong) PNTransition *activeTransition;

// So far, the subclasses of PNPipeline include:
//    PNCloseCreate, PNCloseOpen, PNCloseMigrateOpen, PNCloseFlipOpen
// They share these properties:
@property (readonly, strong) NSString *closeUuid;
@property (readonly, strong) PNDocTokenPlace *sourcePlace;
@property (readonly, strong) PNDocTokenPlace *destinationPlace;

// Except for PNCloseCreate, the subclasses use this property:
@property (readonly, strong) NSString *openUuid;

-(void)buildPipeline;



/**
 
 Typically, the sourcePlace and destPlace are the same. But, sourcePlace and destPlace could well be distinct, in the case of master/detail view controllers.
 @param aPipelineTransitionDelegate the outer transition delegate (the receiver serves as the transition delegate for its internal transitions.
 @param uuidToClose identifies the target docToken in fromPlace
 @param fromPlace the place that contains the target doc token.
 @param toPlace the place that receives the product doc token.
 */
-(void)enableFor: (id <PNTransitionDelegate>)aPipelineTransitionDelegate
  closeUuid: (NSString*)uuidToClose
  fromPlace: (PNDocTokenPlace*)fromPlace
    toPlace: (PNDocTokenPlace*)toPlace;

@end

@interface PNPipeline (PrivateSubclassing)
-(void)fireNextTransition;
@end