#import "PNPipeline.h"

#import "PNTransition+Transforming.h"

@interface PNPipeline ()

@property (readwrite, weak) id <PNTransitionDelegate> outerTransitionDelegate;
@property (readwrite, strong) PNTransition *currentPipelinedTransition;
@property (readwrite, strong) NSOrderedSet *componentTransitions;
@property (readwrite, assign) NSUInteger currentIndex;

@property (readwrite, strong) NSString *closeUuid;

@property (readwrite, strong) PNDocTokenPlace *sourcePlace;

@property (readwrite, strong) PNDocTokenPlace *destinationPlace;

@end

@implementation PNPipeline

@synthesize outerTransitionDelegate;
@synthesize componentTransitions = m_componentTransitions;
@synthesize closeUuid, openUuid, sourcePlace, destinationPlace;
@synthesize currentIndex;

@dynamic  currentPipelinedTransition;

// For <PNTransitionDelegate>
@synthesize activeTransition;

// Custom logic goes here.

-(PNTransition*)currentPipelinedTransition
{
    NSUInteger count = m_componentTransitions.count;
    if( self.currentIndex < count ){
        return [m_componentTransitions objectAtIndex: self.currentIndex];
    }
    return nil;
}
-(void)buildPipeline
{
    // Abstract parent does nothing.
    
    if( nil == m_componentTransitions ){
        m_componentTransitions = [NSMutableOrderedSet orderedSetWithCapacity: 4];
    }
   
}
-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
{
    NSAssert( NO,
             @"-[PNPipeline enableFor:] aborts. Use -enableFor:closeUuid:fromPlace:toPlace: And this is an abstract parent anyway.");
}
-(void)enableFor: (id <PNTransitionDelegate>)aPipelineTransitionDelegate
  closeUuid: (NSString*)uuidToClose
  fromPlace: (PNDocTokenPlace*)fromPlace
    toPlace: (PNDocTokenPlace*)toPlace
{
    self.sourcePlace = fromPlace;
    self.closeUuid = uuidToClose;
    self.destinationPlace = fromPlace;
    
    self.currentIndex = 0;
    
    self.outerTransitionDelegate = aPipelineTransitionDelegate;
    [self.outerTransitionDelegate setActiveTransition: self];
    
    [self fireNextTransition];
}

#pragma mark PNTransitionDelegate methods:

-(void)didFireTransition: (PNTransition*) transition
{
    // Sanity check:
    NSAssert( (transition == self.currentPipelinedTransition),
             @"-[%@ didFireTransition: %@] not self.activeTransition %@",
             NSStringFromClass([self class]),
             transition.name,
             self.currentPipelinedTransition.name );
    
    // This method advances the pipeline:
    NSUInteger index = self.currentIndex;
    NSAssert( (NSNotFound != index),
             @"-[%@ didFireTransition: %@] self.currentIndex = NSNotFound",
             NSStringFromClass([self class]),
             transition.name);
    
    NSUInteger lastIndex = m_componentTransitions.count - 1;
    
    NSAssert( (NSNotFound != index), @"-didFireTransition: bogus transition");
    
    if( self.currentIndex != lastIndex ){
        self.currentIndex++;
        [self fireNextTransition];
        
    }else{
        
        // This call concludes the pipeline.
        [self.outerTransitionDelegate didFireTransition: self];
    }
}
-(BOOL)shouldAttemptToRollbackTransition: (PNTransition*) transition
{
    return NO;
}
-(void) didFailTransition: (PNTransition*) transition
   rolledBackSuccessfully: (BOOL)success
{
    // Pfft!
    [self.outerTransitionDelegate didFailTransition: self
                             rolledBackSuccessfully: success];
    // The delegate may inspect the two properties componentTransitions and activeTransition.
}

@end

@implementation PNPipeline (PrivateSubclassing)
-(void)fireNextTransition
{
    // Typically, subclasses of PNDocTokenTransition
    // Subclasses of PNPipeline can implement a switch/case
    // to fire each component transition with its appropriate arguments.
    switch( self.currentIndex ){
        default:
        {
            NSAssert( NO,
                     @"Abstract parent: -[PNPipeline fireNextTransition] aborts. Subclasses must override.");
            break;
        }
    }
}

@end

