#import "PNQueryUpdate.h"

#import "PNTransition+Transforming.h"

#import "PNArcP2T.h"
#import "PNArcT2P.h"

#import "PNURLToken.h"
#import "PNCloudURLToken.h"

#import "PNURLToken+LifeCycling.h"

#import "NSURL+NPAssisting.h"

#import "PNDocSupportNetController.h"
#import "PNDocSupportNet.h"

@interface PNQueryUpdate ()

// Private interface goes here.
@property (readwrite, strong) NSSet *discoveredItems;

@end


@implementation PNQueryUpdate

@synthesize callback;
@synthesize discoveredItems;

// Custom logic goes here.

/**
 +filteredQueryResults: takes as input the results from an NSMetadataQuery.
 [1] For each NSMetadataItem in the input array, 
 the method finds the UUID in its path.
 [2] For each UUID, the method determines if a cloud token with that uuid already exists.
 [3] if not, the method adds that NSMetadataItem to the output set.
 
 The method returns a set comprising any NSMetadataItems in the input array that do not correspond to an existing cloud url token.
 */
+(NSSet*)filteredQueryResults: (NSArray*)unfilteredQueryResults
{
    if( 0 == unfilteredQueryResults.count ){
        return [NSSet set];
    }
    
    NSMutableSet *filteredSet =
    [NSMutableSet setWithCapacity: unfilteredQueryResults.count];
    
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    PNDocSupportNet *dsn = dsnc.docSupportNet;

    for( NSMetadataItem *nextItem in unfilteredQueryResults ){
        
//        NSURL *url = [nextItem valueForKey: NSMetadataItemURLKey];
//        NSDictionary* metadata =
//        [NSDictionary dictionaryWithContentsOfURL:url];
//      if( 0 != metadata.count ){
            NSString *uuid = [self uuidFromMetadataItem: nextItem];
            
            NSAssert( ( nil != uuid ), @"Bogus metadata item: %@",
                     [[nextItem valueForKey: NSMetadataItemURLKey] description] );
            
            enum PNUuidStatus test = [dsn statusOfUuid: uuid];
            
            switch( test ){
                case PNUuidStatusUnassignedCloudURL:
                case PNUuidStatusAssignedCloudURL:
                {
                    // Typical case: an existing token has this uuid.
                    // Do nothing.
                    
                    break;
                }
                case PNUuidStatusUnknown:
                {
                    // We need to make a new cloud token for this metadata item.
                    [filteredSet addObject: nextItem];
                    
                    break;
                }
                case PNUuidStatusAssignedLocalURL:
                case PNUuidStatusUnassignedLocalURL:
                case PNUuidStatusInvalid:
                default:
                {
                    NSAssert( NO, @"Bad uuid status: %d", test);
                    break;
                }
                    
            } // end switch( test )

        //} // if( 0 != metadata.count )
        
    } // end for(){}
    
    return filteredSet;
}

-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
{
    NSAssert( NO, @"-[PNCloudUpdate enableFor:] aborts. Use -enableFor:metadataItems:callback:");
}
-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
   metadataItems: (NSSet*)metadataItemsToBuild
        callback: (NSInvocation*)aCallback
{
    self.discoveredItems = metadataItemsToBuild;
    self.callback = aCallback;
    
    [super enableFor: transitionDelegate
         targetUuids: [NSSet set]]; // enable all tokens to pass
}

@end

@implementation PNQueryUpdate (Transforming)

+(NSString*)uuidFromMetadataItem: (NSMetadataItem*)item
{
    NSString *result = nil;
    
    // We're expecting a url of the form
    //   */<uuid>/<name>/DocumentMetadata.plist
    
    NSURL *yaddaYaddaUuidNameDocMDataDotPlist = [item valueForKey: NSMetadataItemURLKey];
    NSString *test1 = [yaddaYaddaUuidNameDocMDataDotPlist lastPathComponent];
    BOOL foundDocMDataPlist = [test1 isEqualToString: (NSString*)PNDocMDataDotPlistKey];
    
    if( foundDocMDataPlist ){
        
        NSURL *yaddaYaddaUuidName =
        [yaddaYaddaUuidNameDocMDataDotPlist URLByDeletingLastPathComponent];
        NSURL *yaddaYaddaUuid = [yaddaYaddaUuidName URLByDeletingLastPathComponent];
        NSString *test2 = [yaddaYaddaUuid lastPathComponent];
        
        if( [PNURLToken validUuid: test2] ){
            result = test2;
        }
    }
    return result;
}
-(void)gatherTokens
{
    // Don't gather any tokens; we make output tokens from self.discoveredItems. 
}
-(void)scatterTokens
{
    
    self.arcToUnassignedCloud.uuids = [NSSet set];
    
    [super scatterTokens];
}

-(void)transform
{
    NSManagedObjectContext *moc = self.managedObjectContext;
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    PNDocSupportNet *dsn = dsnc.docSupportNet;
    
    for( NSMetadataItem *nextItem in self.discoveredItems ){
        
        NSString *uuid = [[self class] uuidFromMetadataItem: nextItem];
        
        NSAssert( ( nil != uuid ), @"Bogus metadata item: %@",
                [[nextItem valueForKey: NSMetadataItemURLKey] description] );
        
        enum PNUuidStatus test = [dsn statusOfUuid: uuid];
        
        switch( test ){
            case PNUuidStatusUnassignedCloudURL:
            case PNUuidStatusAssignedCloudURL:
            {
                // Typical case: an existing token has this uuid.
                // Do nothing.
                
                break;
            }
            case PNUuidStatusUnknown:
            {
                // We need to make a new cloud token.
                
                PNCloudURLToken *newToken =
                [PNCloudURLToken insertCloudURLTokenWithMetadataItem: nextItem
                                              inManagedObjectContext: moc];
                
                [self.cloudURLTokensOut addObject: newToken];
                
                break;
            }
            case PNUuidStatusAssignedLocalURL:
            case PNUuidStatusUnassignedLocalURL:
            case PNUuidStatusInvalid:
            default:
            {
                NSAssert( NO, @"Bad uuid status: %d", test);
                break;
            }
        
        } // end switch( test )
    } // end for(){}
    
    [super transform];
}
-(void)didTransform
{
    [super didTransform];
    
    if( nil == m_error ){
        [self.callback invoke];
        // -[PNDocSuportController pnQueryUpdateSucceeded]
    }
}
@end

