#import "PNRenameDoc.h"

#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"
#import "PNCloudURLToken.h"
#import "PNArcT2P.h"

#import "PNDocToken+FileOperations.h"
#import "NSManagedObject+PNPermanentIDing.h"

#import "PNDocTokenPlace.h"

#import "NSURL+NPAssisting.h"

#import "PNDocSupportNetController.h"

@interface PNRenameDoc ()

// Private interface goes here.
@property (readwrite, strong) NSString *differentName;
//@property (readwrite, strong) NSString *differentUuid;
@property (readwrite, strong) PNURLToken *originalURLToken;
@property (readwrite, strong) PNURLToken *differentURLToken;

@property (readwrite, assign) BOOL deleteOriginalValue;

@end


@implementation PNRenameDoc

// Custom logic goes here.
@synthesize differentName;
//@synthesize differentUuid;
@synthesize originalURLToken;
@dynamic differentURLToken;

@synthesize deleteOriginalValue;

-(PNArcP2T*)arcFromUnassignedLocal
{
    return nil;
}
-(PNArcP2T*)arcFromUnassignedCloud
{
    return nil;
}
//-(PNArcT2P*)arcToUnassignedLocal
//{
//    return nil;
//}
//-(PNArcT2P*)arcToUnassignedCloud
//{
//    return nil;
//}
//-(PNArcP2T*)arcFromAssignedLocal
//{
//    return nil;
//}
//-(PNArcP2T*)arcFromAssignedCloud
//{
//    return nil;
//}
//-(PNArcT2P*)arcToAssignedLocal
//{
//    return nil;
//}
//-(PNArcT2P*)arcToAssignedCloud
//{
//    return nil;
//}
//-(PNArcT2P*)arcToDestination
//{
//    return nil;
//}
//-(PNArcP2T*)arcFromSource
//{
//    return nil;
//}
//-(PNDocTokenPlace*)sourcePlace
//{
//    return nil;
//}
//
//-(void)setSourcePlace:(PNDocTokenPlace *)aSourcePlace
//{
//    NSAssert( NO, @"%@ has no source place",
//             NSStringFromClass([self class]) );
//    
//}
//-(PNDocTokenPlace*)destinationPlace
//{
//    return nil;
//}
//-(void)setDestinationPlace:(PNDocTokenPlace *)aDestinationPlace
//{
//    NSAssert( NO, @"%@ has no destination place",
//             NSStringFromClass([self class]) );
//}

-(void)   enableFor: (id <PNTransitionDelegate>)sender
       existingUuid: (NSString*)anExistingUuid
           renameTo: (NSString*)aNewName
          fromPlace: (PNDocTokenPlace*)aSourcePlace
            toPlace: (PNDocTokenPlace*)aDestPlace
     deleteOriginal: (BOOL)deleteOriginalFlag
{
    self.differentName = aNewName;
    self.deleteOriginalValue = deleteOriginalFlag;
    
    m_permID_sourcePlace = [aSourcePlace permanentID];
    m_permID_destinationPlace = [aDestPlace permanentID];
    m_permID_differentURLToken = nil;
    
    self.sourceUuid = anExistingUuid;
    
    [super enableFor: sender];
}

#pragma mark trap

//-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
//  closeUuid: (NSString*)uuid
//  fromPlace: (PNDocTokenPlace*)fromPlace
//{
//    NSLog(@"trapped");
//    NSAssert( NO, @"trapped");
//    
//}
-(PNURLToken*)differentURLToken
{
    PNURLToken *result = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_differentURLToken ){
        
        // [1] Mostly, copy the original:
        NSMutableDictionary *properties =
        [[PNURLToken propertiesFromFilePackageURL: self.originalURLToken.localFilePackageURL] mutableCopy];
        
        // [2] Assert different file name and UUID
        [properties setObject: self.differentName
                       forKey: PNLocalizedNameKey];
        [properties setObject: [PNURLToken nextUuid]
                       forKey: PNUuidStringKey];
        
        // [3] make the replacement url token with the modified properties:
        if( [self.originalURLToken isMemberOfClass:[PNURLToken class]] ){
            result = [PNURLToken insertLocalURLTokenWithProperties: properties
                                            inManagedObjectContext: moc];
        }
        if( [self.originalURLToken isKindOfClass:[PNCloudURLToken class]] ){
            result = [PNURLToken insertCloudURLTokenWithProperties: properties
                                            inManagedObjectContext: moc];
        }
       
        // [4] cleanup:
        result.isAvailableValue = YES;
        self.sourceUuid = nil;
        [moc processPendingChanges];
        
        m_permID_differentURLToken = [result permanentID];
    }
    result = (PNURLToken*)[moc managedObjectForPermanentID: m_permID_differentURLToken];
    
    return result;
}

@end

@implementation PNRenameDoc (Transforming)

-(void)gatherDocToken
{
    [super gatherDocToken];
    
    NSAssert( [self.docToken.urlToken isKindOfClass:[PNURLToken class]],
              @"PNRenameDoc only renames local documents.");
    
    self.docToken.urlToken = self.differentURLToken;
}
-(void)gatherURLTokens
{
    [super gatherURLTokens];
    
    if( 1 != self.assignedIn.count ){
        NSAssert( (1==self.assignedIn.count),
                 @"Should get just one url token");
    }
    
    self.originalURLToken = self.assignedIn.anyObject;
    
    self.unassignedOut = [NSSet setWithObject: originalURLToken];
    
}
-(void)launchTransform
{
    [self.docToken saveDocumentForRenaming];
    
    // callbacks: -docTokenDidBecomeAvailable: or -docToken:didFailWithError:
    // Call [super transform] from -docTokenDidBecomeAvailable: or -docToken:didFailWithError:.
    
    self.assignedOut = [NSSet setWithObject: self.docToken.urlToken];
    

}

-(void)didTransform
{
    if( nil == m_error ){
        NSError *localError = nil;
        
        if( self.deleteOriginalValue ){
            
            // Get rid of the UUID directory that contained the store.
            [self.originalURLToken removeURLsWithError: &localError];
            
        }
        
        m_error = localError;
    }
    [super didTransform];
}
-(void)scatterURLTokens
{
     
    if( self.deleteOriginalValue ){
        
        // Delete the original source URL token.
        enum PNURLDeploymentCase dCase = [self.originalURLToken.localFilePackageURL deploymentCase];

        NSAssert( (PNURLDeploymentCaseNoUuidDir==dCase),
                 @"Failed to remove Uuid Dir in -didTransform");
       
        [PNURLToken deleteURLToken: self.originalURLToken];
        self.originalURLToken = nil;
        self.unassignedOut = nil;
        
    }else{
                
        self.unassignedOut = [NSSet setWithObject: self.originalURLToken];

    }
    self.assignedOut = [NSSet setWithObject: self.differentURLToken];

    // Of each {local, cloud} pair of arcs, only one actually pushes the token(s).
    // They have complementary guard expressions for the entities they permit.
    
    [self.arcToUnassignedLocal pushTokens: self.unassignedOut];
    [self.arcToUnassignedCloud pushTokens: self.unassignedOut];
    
    [self.arcToAssignedLocal pushTokens: self.assignedOut];
    [self.arcToAssignedCloud pushTokens: self.assignedOut];
    
}
-(void)assertOutputGuardExpressions
{
    NSSet *originalUUIDSet = nil;
    if( !self.deleteOriginalValue ){
        originalUUIDSet = [NSSet setWithObject: self.originalURLToken.uuid];
    }
    self.arcToUnassignedLocal.uuids = originalUUIDSet;
    self.arcToUnassignedCloud.uuids = originalUUIDSet;

    NSSet *differentUuidSet = [NSSet setWithObject: self.differentURLToken.uuid];
    
    self.arcToDestination.uuids = differentUuidSet;
    
    self.arcToAssignedLocal.uuids = differentUuidSet;
    self.arcToAssignedCloud.uuids = differentUuidSet;
}

@end
