#import "PNSyncResources.h"

#import "PNTransition+Transforming.h"

#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"
#import "PNCloudURLToken.h"

#import "PNDocSupportNetController.h"
#import "PNDocSupportNetController+StorageControlling.h"
#import "NSURL+NPAssisting.h"

#import "PNDocSupportNet.h"

#import "PNArcT2P.h"

@interface PNSyncResources ()

// Private interface goes here.

@end


@implementation PNSyncResources

@synthesize callback;

// Custom logic goes here.

-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
     targetUuids: (NSSet*)targetUuids
        callback: (NSInvocation*)aCallback
{
    self.callback = aCallback;
    
    [super enableFor: transitionDelegate
         targetUuids: targetUuids];
    
    self.arcToUnassignedLocal.uuids = self.uuids;
    self.arcToUnassignedCloud.uuids = nil;
}
@end

@implementation PNSyncResources (Transforming)
-(void)pruneBadUuidDirectoriesInDirectory: (NSURL*)targetDirectory
{
    /**
     
     This method checks the target directory's URLs.
     
     The "good" pattern is:
     <container url>/<mid path>/<uuid>/<name>
     where
     <container url>/<mid path>/<uuid>/
     contains EXACTLY ONE subdirectory (not a file),
         <name>/
     
     
     */
    
    NSLog(@"-[PNSyncResources pruneBadUuidDirectoriesInDirectory:] BEGIN");

    NSFileManager *fileMngr =
    [PNDocSupportNetController threadSafeFileManager];
    
    /*
     See: http://www.techques.com/question/1-11546974/NSFileManager:enumeratorAtURL:-returns-a-different-form-of-URL-to-NSFileManager:URLForDirectory
     
     */
    
    NSDirectoryEnumerationOptions options =
    NSDirectoryEnumerationSkipsPackageDescendants | NSDirectoryEnumerationSkipsHiddenFiles;
    
    NSArray *propertyKeys =
    [NSArray arrayWithObjects:NSURLNameKey, NSURLIsDirectoryKey, NSURLTypeIdentifierKey, nil];
    
    NSDirectoryEnumerator *dirEnumerator =
    [fileMngr enumeratorAtURL: targetDirectory
   includingPropertiesForKeys: propertyKeys
                      options: options
                 errorHandler: nil];
    
    // Enumerate the results
    for (NSURL *nextRawURL in dirEnumerator) {
        
        NSURL *nextURL = [nextRawURL pnNormalizedURL];
        
        enum PNURLDeploymentCase myCase = [nextURL deploymentCase];
        
        switch( myCase ){
            case PNURLDeploymentCaseUuidDirSlashNoSuchFileOrDirectory: // delete this
            {
                NSFileCoordinator* coordinator =
                [[NSFileCoordinator alloc] initWithFilePresenter:nil];
                
                NSError __block *blockError = nil;
                
                [coordinator coordinateWritingItemAtURL:nextURL
                                                options:NSFileCoordinatorWritingForDeleting
                                                  error:nil
                                             byAccessor:^(NSURL *coordinatedURL) {
                                                 
                                                 [fileMngr removeItemAtURL:coordinatedURL
                                                                     error:&blockError];
                                                 
                                                 if( nil != blockError ){
                                                     m_error = blockError;
                                                     [self didTransform];
                                                 }
                                                 
                                             }];
                
                [dirEnumerator skipDescendants]; // because we just deleted the descendents.
                
                break;
            }
            case PNURLDeploymentCaseUuidDirSlashExistingRWDir: // DEFINITELY don't delete this
            {
                break;
            }
            case PNURLDeploymentCaseUuidDirSlashExistingFile: // treat this case specially?
            {
                break;
            }
                
                // These other cases are degenerate in this method.
                // We're actually misusing -deploymentCase.
                
            case PNURLDeploymentCaseInvalid: // some kind of error
            case PNURLDeploymentCaseDefective: // some other kind of error
            case PNURLDeploymentCaseNoUuidDir: // definitely deleted
            default:
            {
                
                break;
            }
        }
    }
    NSLog(@"-[PNSyncResources pruneBadUuidDirectoriesInDirectory:] END");

}
-(void)purgeBadLocalURLTokens
{
    NSLog(@"-[PNSyncResources purgeStaleURLTokens] BEGIN");
    
    // Identify and delete stale tokens
    
    for( PNURLToken *urlToken in self.localURLTokensIn ){
        
        enum PNURLDeploymentCase myCase = [urlToken.localFilePackageURL deploymentCase];
        
        switch( myCase ){
            case PNURLDeploymentCaseUuidDirSlashExistingRWDir: // don't delete this
            {
                NSLog(@"\t skipping: %@", urlToken.localFilePackageURL );
                break;
            }
            case PNURLDeploymentCaseInvalid: // some kind of error
            case PNURLDeploymentCaseDefective: // some other kind of error
            case PNURLDeploymentCaseUuidDirSlashExistingFile: // we'll need to open its doc
            case PNURLDeploymentCaseUuidDirSlashNoSuchFileOrDirectory: // deleted, but badly
            case PNURLDeploymentCaseNoUuidDir: // definitely deleted
            default:
            {
                NSAssert( (nil==urlToken.docToken), @"doc token should be nil");
                
                NSLog(@"\t purging: %@", urlToken.localFilePackageURL );
                
                //[urlToken.place removeToken: urlToken];
                
                [PNURLToken deleteURLsAndURLToken: urlToken
                                            error: nil];
                break;
            }
        }
    }
    
    NSLog(@"-[PNSyncResources purgeStaleURLTokens] END");
}

-(void)visitLocalFilePackageURL: (NSURL*)filePackageURL
                     localError: (NSError*)localError
{
    
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    PNDocSupportNet *dsn = [dsnc docSupportNet];
    NSFileManager *fileMngr = [dsnc threadSafeFileManager];

    NSURL *uuidDir = [filePackageURL URLByDeletingLastPathComponent];
    
    // [1] Test for sibling files in the <uuid> directory.
    //    
    
    
    NSArray *siblings = [fileMngr contentsOfDirectoryAtPath: [uuidDir path]
                                                      error:  &localError];
    if( nil != localError ){
        return;
    }
    NSUInteger count = siblings.count;
    if( 1 != count ){
        NSAssert( (1==count),
                 @"<uuid> dir %@ must have exactly one file: %@",
                 [uuidDir path],
                 [siblings description] );
        
    }

    NSString *uuidTest = [uuidDir lastPathComponent];
    
    enum PNUuidStatus status = [dsn statusOfUuid: uuidTest];
    
    switch (status) {
        case PNUuidStatusUnknown:
        {
            NSLog(@"Discovered doc URL: %@", filePackageURL);
            
            // A PNURLToken does not exist for thisURL.
            // Make a new one.
            
            PNURLToken *newToken = [PNURLToken urlTokenForURL: filePackageURL];
            
            if( [newToken isKindOfClass: [PNCloudURLToken class]] ){
                [self.cloudURLTokensOut addObject: newToken];
            }
            if( [newToken isMemberOfClass: [PNURLToken class]] ){
                [self.localURLTokensOut addObject: newToken];
            }
            
            
            
            break;
        }
        case PNUuidStatusUnassignedLocalURL:
        case PNUuidStatusUnassignedCloudURL:
        case PNUuidStatusAssignedLocalURL:
        case PNUuidStatusAssignedCloudURL:
        {
            // All standard cases:
            // A PNURLToken already exists for thisURL.
            // Don't make a new one.
            NSLog(@"\t skipping: %@", [filePackageURL description]);
            break;
        }
        case PNUuidStatusInvalid: // 0 == PNUuidStatusInvalid. See PNURLToken.h
        default:
        {
            // Bad value of status
            NSAssert( NO, @"bogus status = %d", status );
            break;
        }
    } // end switch(status)
}
-(void)buildURLTokensForDocURLsInDirectory: (NSURL*)targetDirectory
{
    NSLog(@"-[PNDocSupportNet buildURLTokensForDocURLsInDirectory: %@] BEGIN",
          [targetDirectory description]);
    NSFileManager *fileMngr = [PNDocSupportNetController threadSafeFileManager];
    
    /*
     See: http://www.techques.com/question/1-11546974/NSFileManager:enumeratorAtURL:-returns-a-different-form-of-URL-to-NSFileManager:URLForDirectory
     
     
     NSDirectoryEnumerationOptions options =
     NSDirectoryEnumerationSkipsPackageDescendants | NSDirectoryEnumerationSkipsHiddenFiles;
     
     NSArray *propertyKeys =
     [NSArray arrayWithObjects:NSURLNameKey, NSURLIsDirectoryKey, NSURLTypeIdentifierKey, nil];
     
     NSDirectoryEnumerator *dirEnumerator = [fileMngr enumeratorAtURL: targetDirectory
     includingPropertiesForKeys: propertyKeys
     options: options
     errorHandler: nil];
     
     Using the URL enumerator, the -[NSURL deploymentCase] method was receivng arguments like:
     file://localhost/private/var/mobile/Applications/7499A800-AB73-45D8-BC95-A67372C08B31/Documents/2C578AB8-9538-413C-900D-94702E98C98A/
     
     with .../private/... in the path.
     
     Or use URLByResolvingSymlinksInPath
     or [[url URLByResolvingSymlinksInPath] URLByStandardizingPath];
     
     
     NSDirectoryEnumerator *dirEnumerator = [fileMngr enumeratorAtPath: [targetDirectory path]];
     FAILS TO GO DEEP.
     
     */
    
    // Enumerate the dirEnumerator results
    NSDirectoryEnumerationOptions options =
    NSDirectoryEnumerationSkipsPackageDescendants | NSDirectoryEnumerationSkipsHiddenFiles;
    
    NSArray *propertyKeys =
    [NSArray arrayWithObjects:NSURLNameKey, NSURLIsDirectoryKey, NSURLTypeIdentifierKey, nil];
    
    NSDirectoryEnumerator *dirEnumerator = [fileMngr enumeratorAtURL: targetDirectory
                                          includingPropertiesForKeys: propertyKeys
                                                             options: options
                                                        errorHandler: nil];
    
    NSError *localError = nil; // break out of the for loop if we hit an error.
    
    for (NSURL *nextRawURL in dirEnumerator) {
        
        NSURL *nextURL = [nextRawURL pnNormalizedURL];
        
        enum PNURLDeploymentCase myCase = [nextURL deploymentCase];
        
        switch( myCase ){
            case PNURLDeploymentCaseUuidDirSlashExistingRWDir: // DEFINITELY don't delete this
            {
                [self visitLocalFilePackageURL: nextURL
                                    localError: localError];
                
                [dirEnumerator skipDescendants];
                
                break;
            }
                
                // These other cases are degenerate in this method.
                // We're actually misusing -deploymentCase.
                
            case PNURLDeploymentCaseUuidDirSlashNoSuchFileOrDirectory: // delete this
            case PNURLDeploymentCaseUuidDirSlashExistingFile: // treat his case specially?
            case PNURLDeploymentCaseInvalid: // some kind of error
            case PNURLDeploymentCaseDefective: // some other kind of error
            case PNURLDeploymentCaseNoUuidDir: // definitely deleted
            default:
            {
                
                break;
            }
        }
        
        if( nil != localError ){
            break;
        }
    }
    
    // broke out of the for loop because we hit an error?
    if( nil != localError ){
        m_error = localError;
        [self didTransform];
    }

    NSLog(@"-[PNDocSupportNet buildURLTokensForDocURLsInDirectory: %@] END",
          [targetDirectory description]);
    
}

-(void)syncLocalURLs
{
    NSURL *target = [PNURLToken containerUrl];

    [self pruneBadUuidDirectoriesInDirectory: target];
    
    // Purge existing local tokens
    [self purgeBadLocalURLTokens];
    
    [self buildURLTokensForDocURLsInDirectory: target];
}

-(void)gatherTokens
{
    [self.tokensToDeleteInScatter removeAllObjects];
    
    [self.tokensToDeleteInScatter addObjectsFromArray: [[self.arcFromUnassignedCloud pullTokens] allObjects]];
   
    self.localURLTokensIn = [self.arcFromUnassignedLocal pullTokens];
    self.cloudURLTokensIn = [NSSet set];
            
    return;
}

-(void)transform
{
    NSLog( @"-[%@ transform] for real", NSStringFromClass([self class]));
    
    [self syncLocalURLs];
    
    [super transform];
}

-(void)didTransform
{
    [super didTransform];
    
    if( nil == m_error ){
        [self.callback invoke];
    }
}

@end
