#import "_PNTransition.h"

#import "PNTransitionDelegate.h"

@class PNDocSupportNet;

/**
 General purpose parent class at the level of PNNet, PNTransition, PNPlace,  PNArc, PNArcP2T, PNArcT2P, PNToken.
 
 PNTransition encapsulates an operation. Generally, the operation accepts input tokens and emits output tokens. The tokens encapsulate resources. The abstract parent PNTransition provides an empty operation. PNTransition has an error property (NSError). In subclasses, the first occurance of an an error in the operation terminates the transition. 
 
 The input arcs to a transition are instances of PNArcP2T; the output arcs from a transition are instances of PNArcT2P.

 The -enableFor: method acquires a transition delegate for the receiver. It also adds the receiver to its net's firing sequence. The PNNet enqueues an NSInvocation in its sequence queue for the receiver transition. The invocation sends the -fire message to the receiver transition.
 
 See: -[PNNet addToFiringSequence:]
 
 The transition reports its success or first failure to its transition delegate in -didTransform.
 
 PNTransition has two subclasses: PNURLTransition and PNDocTokenTransition PNURLTransition and PNDocTokenTransition operate on PNURLTokens and PNDocTokens respectively. PNPipeline is a subclass of PNDocTokenTransition and encapsulates a series of PNTransitions.
 
 In the class hierarchy descendent from PNTransition, subclasses differentiate primarily in their implementations of: 
 -gatherTokens      how they gather their input tokens
 -scatterTokens     how they scatter their output tokens
 -transform         the operation that transforms the set of input tokens to the set of output tokens 
 In some cases the sets of input or output tokens may be empty.
 
 PNPipeline acts as the transition delegate of its internal transitions, but reports overall success or failure to its own transition delegate.
 
 
 */
@interface PNTransition : _PNTransition
{
    id <PNTransitionDelegate> m_transitionDelegate;

    NSError *m_error;
}

@property (readonly, strong) NSError *error;
@property (readonly, strong) PNDocSupportNet *docSupportNet;

/**
	Gathers the receiver's input tokens, sets the receiver's transition delegate, and adds the receiver to its PNNet's firing sequence.
 @param transitionDelegate handles 
 */
-(void)enableFor: (id <PNTransitionDelegate>)transitionDelegate;

/**
 The adopting developer should not call this method directly.
 
	See -[PNNet addToFiringSequence:].
 PNNet adds to its operation queue an NSInvocation that calls the transition's -fire method.
 
 */
-(void)fire;


/**
 The adopting developer should not call this method directly.
 */
-(void)muster: (NSManagedObject*)foo;
/**
 The adopting developer should not call this method directly.
 */
-(void)connectFixedArcs;
/**
 The adopting developer should not call this method directly.
 */
-(void)disconnectFixedArcs; // never called, for symmetry

/**
	For debugging.
	@returns a string with the receiver's name, its input arcs, and output arcs.
 */
-(NSString*)quickLook;

@end
