#import "PNURLTokensTransition.h"

#import "PNArcP2T.h"
#import "PNArcT2P.h"
#import "PNPlace.h"
#import "PNURLTokenPlace.h"

#import "PNURLToken.h"
#import "PNURLToken+LifeCycling.h"

#import "PNDocSupportNet.h"
#import "PNDocSupportNetController.h"

#import "NSManagedObject+PNPermanentIDing.h"
#import "PNTransition+Transforming.h"


//#import <PNDocSupport/PNURLToken.h>
#import "PNURLToken.h"

@interface PNURLTokensTransition ()

@end

@implementation PNURLTokensTransition

// Custom logic goes here.
@dynamic arcFromUnassignedLocal;
@dynamic arcFromUnassignedCloud;
@dynamic arcToUnassignedLocal;
@dynamic arcToUnassignedCloud;

@dynamic allURLTokensIn;

@synthesize uuids, localURLTokensIn, cloudURLTokensIn;
@dynamic localURLTokensOut, cloudURLTokensOut;

@dynamic tokensToDeleteInScatter;

-(NSSet*)allURLTokensIn // called once during -gatherTokens
{
    NSMutableSet *allTokens = [self.localURLTokensIn mutableCopy];
    [allTokens addObjectsFromArray: [self.cloudURLTokensIn allObjects]];
    
    return allTokens;
}

-(NSMutableSet*)tokensToDeleteInScatter
{
    if( nil == m_tokensToDeleteInScatter ){
        m_tokensToDeleteInScatter = [[NSMutableSet alloc] initWithCapacity:1];
    }
    return m_tokensToDeleteInScatter;
}
-(NSMutableSet*)localURLTokensOut
{
    if( nil == m_localURLTokensOut ){
        m_localURLTokensOut = [[NSMutableSet alloc] initWithCapacity:1];
    }
    return m_localURLTokensOut;
}
-(NSMutableSet*)cloudURLTokensOut
{
    if( nil == m_cloudURLTokensOut ){
        m_cloudURLTokensOut = [[NSMutableSet alloc] initWithCapacity:1];
    }
    return m_cloudURLTokensOut;
}
-(void)connectFixedArcs
{
    [self muster: self.arcFromUnassignedLocal];
    [self muster: self.arcFromUnassignedCloud];
    
    [self muster: self.arcToUnassignedLocal];
    [self muster: self.arcToUnassignedCloud];
}
-(PNArcP2T*)arcFromUnassignedLocal
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromUnassignedLocal ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            arc.inPlace = self.docSupportNet.unassignedLocalPlace;
            arc.outTransition = self;
            arc.uuids = nil;
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [self.managedObjectContext saveAnyChanges];
        m_permID_arcFromUnassignedLocal = [arc permanentID];
    }
    arc = (PNArcP2T*)[self.managedObjectContext managedObjectForPermanentID: m_permID_arcFromUnassignedLocal];
    return arc;
}
-(PNArcP2T*)arcFromUnassignedCloud
{
    PNArcP2T *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcFromUnassignedCloud ){
        arc = [PNArcP2T insertInManagedObjectContext: moc];
        {
            arc.inPlace = self.docSupportNet.unassignedCloudPlace;
            arc.outTransition = self;
            arc.uuids = nil;
            arc.multiplicityValue = 1; // requires at least 1 token upstream
        }
        [self.managedObjectContext saveAnyChanges];
        m_permID_arcFromUnassignedCloud = [arc permanentID];
    }
    arc = (PNArcP2T*)[self.managedObjectContext managedObjectForPermanentID: m_permID_arcFromUnassignedCloud];
    return arc;
}

-(PNArcT2P*)arcToUnassignedLocal
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToUnassignedLocal ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedLocal = self.docSupportNet.unassignedLocalPlace;
            arc.inTransition = self;
            arc.outPlace = unassignedLocal;
            arc.targetTokenEntityName = @"PNURLToken";
            arc.uuids = nil;
        }
        [moc saveAnyChanges];
        m_permID_arcToUnassignedLocal = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToUnassignedLocal];
    return arc;
}
-(PNArcT2P*)arcToUnassignedCloud
{
    PNArcT2P *arc = nil;
    NSManagedObjectContext *moc = self.managedObjectContext;
    if( nil == m_permID_arcToUnassignedCloud ){
        arc = [PNArcT2P insertInManagedObjectContext: moc];
        {
            PNURLTokenPlace *unassignedCloud = self.docSupportNet.unassignedCloudPlace;
            arc.inTransition = self;
            arc.outPlace = unassignedCloud;
            arc.targetTokenEntityName = @"PNCloudURLToken";
            arc.uuids = nil;
        }
        [moc saveAnyChanges];
        m_permID_arcToUnassignedCloud = [arc permanentID];
    }
    arc = (PNArcT2P*)[moc managedObjectForPermanentID: m_permID_arcToUnassignedCloud];
    return arc;
}

-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
{
    NSAssert( NO, @"-[PNURLTokensTransition enableFor:] aborts. Use -enableFor:targetUuids:");
}

-(void)enableFor: (id<PNTransitionDelegate>)transitionDelegate
     targetUuids: (NSSet*)targetUuids
{
    self.uuids = targetUuids;
    
    self.arcFromUnassignedLocal.uuids = self.uuids;
    self.arcFromUnassignedCloud.uuids = self.uuids;
    [super enableFor: transitionDelegate];
}



@end

@implementation PNURLTokensTransition (Transforming)
-(void)gatherTokens
{
    [self.tokensToDeleteInScatter removeAllObjects];
    
    self.localURLTokensIn = [self.arcFromUnassignedLocal pullTokens];
    self.cloudURLTokensIn = [self.arcFromUnassignedCloud pullTokens];

//    BOOL nonZero = (0 != (self.localURLTokensIn.count + self.cloudURLTokensIn.count));
//    NSAssert( nonZero, @"Gathered no tokens. That's an error.");
    
    // During -transform in subclasses,
    // we remove from [self.tokensToDelete
    // every token that does not raise an error:
    //[self.tokensToDeleteInScatter addObjectsFromArray: self.allURLTokensIn.allObjects];
    
    return;
}
-(void)scatterTokens
{
    
    { // Debug passage:
        NSManagedObjectContext *moc = self.managedObjectContext;
        [moc saveAnyChanges];
    } // Debug passage:
    
    [self.arcToUnassignedLocal pushTokens: self.localURLTokensOut];
    [self.localURLTokensOut removeAllObjects];
    
    { // Debug passage:
        NSManagedObjectContext *moc = self.managedObjectContext;
        [moc saveAnyChanges];
    } // Debug passage:
    
    [self.arcToUnassignedCloud pushTokens: self.cloudURLTokensOut];
    [self.cloudURLTokensOut removeAllObjects];
    
    { // Debug passage:
        NSManagedObjectContext *moc = self.managedObjectContext;
        [moc saveAnyChanges];
    } // Debug passage:
   
    while( 0 != self.tokensToDeleteInScatter.count ){
        PNURLToken *haplessToken = self.tokensToDeleteInScatter.anyObject;
        [self.tokensToDeleteInScatter removeObject: haplessToken];
    
        [PNURLToken deleteURLToken: haplessToken];

    }
    
    { // Debug passage:
        NSManagedObjectContext *moc = self.managedObjectContext;
        [moc saveAnyChanges];
    } // Debug passage:

}
@end
