//
//  NSManagedObject+PNPermanentIDing.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/7/12.
//
//

#import <CoreData/CoreData.h>

/**
	Used to reduce fetching and complexity. The managed object context must have retainsRegisteredObjects == YES.
 Suitable for data models limited to a few hundred instances.
 See also: 
 NSManagedObjectContext (PNPermanentIDing) -managedObjectForPermanentID:
	@returns a permanent ID in the form of a URL.
 */
@interface NSManagedObject (PNPermanentIDing)
-(NSURL*)permanentID;
@end

/**
 Used to reduce fetching and complexity. The receiver must have retainsRegisteredObjects == YES.
 Suitable for data models limited to a few hundred instances.
 See also:
 NSManagedObject (PNPermanentIDing) -permanentID
 @param permanentID a permanent ID as returned by NSManagedObject (PNPermanentIDing) -permanentID.
 @returns the managed object associated with the permanent ID.
 */
@interface NSManagedObjectContext (PNPermanentIDing)
-(NSManagedObject*)managedObjectForPermanentID:(NSURL*)permanentID;
@end

@interface NSManagedObjectContext (PNWrangling)
/**
	Convenience method hides the error argument to [self save:&error];
 */
-(void)saveAnyChanges;
/**
	Convenience method eturns the receiver's data model
	@returns the receiver's data model
 */
-(NSManagedObjectModel*)managedObjectModel;
@end