//
//  NSManagedObject+PNPermanentIDing
//  TaggedLocationsDemo
//
//  Created by DevGuy on 11/7/12.
//
//

#import "NSManagedObject+PNPermanentIDing.h"

@implementation NSManagedObject (PNPermanentIDing)
-(NSURL*)permanentID
{
	NSURL *result = nil;
	NSManagedObjectContext *moc = [self managedObjectContext];
    NSAssert( [moc retainsRegisteredObjects],
             @"NO==[moc retainsRegisteredObjects]. Might work initially, but fail later.");
	NSError *error = nil;
	BOOL ok = NO;
	{
		/*
		 Special Considerations (for obtainPermanentIDsForObjects:)
		 This method results in a transaction with the underlying store which changes the file’s modification date.
		 
		 This results an additional consideration if you invoke this method on the managed object context associated with an instance of NSPersistentDocument.
		 Instances of NSDocument need to know that they are in sync with the underlying content.
		 To avoid problems, after invoking this method you must therefore update the document’s modification date (using setFileModificationDate:).
		 
		 */
		ok = [moc obtainPermanentIDsForObjects: [NSArray arrayWithObject: self]
										 error: &error];
		/* error =
		 Error Domain=NSCocoaErrorDomain Code=134020 UserInfo=0x10b6220
		 "The model configuration used to open the store is incompatible with the one that was used to create the store."
		 */
	}
	
	if( nil != error ){
		NSString *yelp =
		[NSString stringWithFormat: @"-[%@ permanentID] failed to obtain permanent IDs",
         NSStringFromClass([self class] )];
        NSAssert( ( nil == error ), yelp );

	}
	if( ok ){
		
		NSManagedObjectID *objectID = [self objectID];
		result = [objectID URIRepresentation];
		NSString *help = [NSString stringWithFormat:@"-[%@ permanentID] can't get URL for objectID %@",
						  NSStringFromClass([self class] ),
                          [objectID description]];
		NSAssert( result, help );
		
	}else{
		NSAssert( ok, @"can't make objectIDs permanent." );
	}
	
    NSManagedObject *test = [moc managedObjectForPermanentID: result];
    
    NSAssert( (test == self), @"Bogus result");
	return result;
}

@end

@implementation NSManagedObjectContext (PNPermanentIDing)
-(NSManagedObject*)managedObjectForPermanentID:(NSURL*)permanentID
{
    NSAssert( [self retainsRegisteredObjects],
             @"NO==[self retainsRegisteredObjects]. Might work initially, but fail later.");

    NSManagedObject *result = nil;
    if( nil == permanentID ){
        // The if(){...} statement lets me set a breakpoint:
        NSAssert( (nil != permanentID), @"Can't resolve nil permanentID");
    }

    NSPersistentStoreCoordinator *psc = [self persistentStoreCoordinator];
    NSManagedObjectID *objectID = [psc managedObjectIDForURIRepresentation: permanentID];
    NSAssert( (nil != objectID), @"Failed to get objectID for permanentID");
    
    result = [self objectRegisteredForID: objectID];
    if( nil == result ){
        NSLog(@"-[%@ managedObjectForPermanentID: %@]",
              NSStringFromClass([self class]),
              [permanentID description] );
        
        NSLog(@"No object registered for objectID, crashing");
        NSLog(@"  objectID = %@", [objectID description]);
    }
    NSAssert( (nil != result), @"No object registered for objectID");
	return result;
}

@end

@implementation NSManagedObjectContext (PNWrangling)
-(void)logMocSaveError: (NSError*)error
{
    if ([[error domain] isEqualToString:@"NSCocoaErrorDomain"]) {
        // ...check whether there's an NSDetailedErrors array
        NSDictionary *userInfo = [error userInfo];
        if ([userInfo valueForKey:@"NSDetailedErrors"] != nil) {
            // ...and loop through the array, if so.
            NSArray *errors = [userInfo valueForKey:@"NSDetailedErrors"];
            for (NSError *anError in errors) {
                
                NSDictionary *subUserInfo = [[anError userInfo] copy];
                // Granted, this indents the NSValidation keys rather a lot
                // ...but it's a small loss to keep the code more readable.
                NSLog(@"Core Data Save Error\n\n \
                      NSValidationErrorKey\n%@\n\n \
                      NSValidationErrorPredicate\n%@\n\n \
                      NSValidationErrorObject\n%@\n\n \
                      NSLocalizedDescription\n%@",
                      [subUserInfo valueForKey:@"NSValidationErrorKey"],
                      [subUserInfo valueForKey:@"NSValidationErrorPredicate"],
                      [subUserInfo valueForKey:@"NSValidationErrorObject"],
                      [subUserInfo valueForKey:@"NSLocalizedDescription"]);
            }
        }
        // If there was no NSDetailedErrors array, print values directly
        // from the top-level userInfo object. (Hint: all of these keys
        // will have null values when you've got multiple errors sitting
        // behind the NSDetailedErrors key.
        else {
            NSLog(@"Core Data Save Error\n\n \
                  NSValidationErrorKey\n%@\n\n \
                  NSValidationErrorPredicate\n%@\n\n \
                  NSValidationErrorObject\n%@\n\n \
                  NSLocalizedDescription\n%@",
                  [userInfo valueForKey:@"NSValidationErrorKey"],
                  [userInfo valueForKey:@"NSValidationErrorPredicate"],
                  [userInfo valueForKey:@"NSValidationErrorObject"],
                  [userInfo valueForKey:@"NSLocalizedDescription"]);
            
        }
    }
    // Handle mine--or 3rd party-generated--errors
    else {
        NSLog(@"Custom Error: %@", [error localizedDescription]);
    }

}
-(BOOL)pnSaveAnyChanges
{
    /*
     This method 
     */
    [self processPendingChanges];
    
    NSError *error = nil;
    
    [self save:&error];
    
    if (error) {
        // handle the error.
        [self logMocSaveError: error];
        return NO;
    }else{
        return YES;
    }
}


-(void)saveAnyChanges
{
    [self pnSaveAnyChanges];
}
/*

 */
-(NSManagedObjectModel*)managedObjectModel
{
    NSPersistentStoreCoordinator *psc = self.persistentStoreCoordinator;
    NSManagedObjectModel *mom = psc.managedObjectModel;
    return mom;
}
-(NSFetchRequest *)fetchRequestForEntitiesByName: (NSString*)entityName
{
    NSManagedObjectModel *mom = [self managedObjectModel];

	NSEntityDescription *entityDescription =
    [[mom entitiesByName] objectForKey: entityName];
	
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	[fetchRequest setEntity: entityDescription];
	
	return fetchRequest;
}

@end
