//
//  NSURL+NPAssisting.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 9/10/12.
//
//

#import <Foundation/Foundation.h>

/**
	An enumeration of cases for the <base> part of file package urls. For the convenience of adopting developers, the value encodes whether or not local storage occurs on a real device or the simulator.
 */
typedef enum PNUrlBaseCase
{
	PNUrlBaseCaseInvalid = 0, /**< The degenerate value */
	PNUrlBaseCaseLocal = 1, /**< file package url has NSHomeDirectory() as its base, on a device */
	PNUrlBaseCaseCloud = 2, /**< file package url has the cloud container as its base */
	PNUrlBaseCaseSimulator = 3 /**< file package url has NSHomeDirectory() as its base, on the simulator */
} PNUrlBaseCase;

typedef enum PNURLDeploymentCase
{
    PNURLDeploymentCaseInvalid = 0,
    /**< Not the other cases
   Invalid base: neither local (device or simulator) nor cloud.
    A nil token returns PNURLDeploymentCaseInvalid from the -urlDeploymentCase method. */
    
    PNURLDeploymentCaseNoUuidDir,
    /**< valid base, no valid uuid dir */

    PNURLDeploymentCaseUuidDirSlashNoSuchFileOrDirectory,
    /**< appropriate base url
     uuid dir exists, and its name is a valid uuid string
     no existing file */
        
    PNURLDeploymentCaseUuidDirSlashExistingFile,
    /**< [1] appropriate base url: local or cloud.
    [2] uuid directory is valid and exists;
    [3] file exists, not directory */
    
    PNURLDeploymentCaseDefective,
    /**< [1] appropriate base url: local or cloud.
     [2] uuid directory is valid and exists;
     [3] directory exists, not file;
     [4] has siblings */
  
    PNURLDeploymentCaseUuidDirSlashExistingRWDir
     /**< [1]  [1] appropriate base url: local or cloud.
      [2] uuid directory is valid and exists;
      [3] directory exists, not file;
      [4] readable and writeable. */
    

} PNURLDeploymentCase;

extern NSString *slash;

/**
	A category on NSURL that encapsulates helper methods for managing URLs for file packages.
 */
@interface NSURL (NPAssisting)

/**
	Inspects the path components and returns a value that identifies the leading path component(s).
	@returns the appropriate value of PNUrlBaseCase.
 */
-(enum PNUrlBaseCase)baseCase;


/**
	Provides a consistent form for file package URLs.
 
See: https://github.com/omnigroup/OmniGroup/blob/master/Frameworks/OmniUI/iPad/OUIDocumentPreview.m
 
    Example:
    /private/var/mobile/Applications/CB7B8FC9-8756-4123-B624-C0C510DC438A/Documents/4FF18CF9-1392-4250-B3EA-2518FAB42FD5/Untitled ->
            /var/mobile/Applications/CB7B8FC9-8756-4123-B624-C0C510DC438A/Documents/4FF18CF9-1392-4250-B3EA-2518FAB42FD5/Untitled

	@returns a consistent version of the URL.
 */
-(NSURL*)pnNormalizedURL;

+(NSString*)shibbolethForBaseCase:(enum PNUrlBaseCase)baseCase;

-(enum PNURLDeploymentCase)deploymentCase;

-(NSURL*)URLByAppendingPathComponents: (NSArray*)components
                          isDirectory: (BOOL)flag;
-(NSArray*)siblingsWithError: (NSError**)error;
@end
