//
//  PNAlertViewHelper.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/19/13.
//
//

#import <UIKit/UIKit.h>

/**
 The PNAlertViewHelper implements the minimum UIAlertViewDelegate method, -alertView:didDismissWithButtonIndex:.
 
 Sometimes a given class uses UIAlertView in different ways. 
 For example, PNDocSupportNetController uses UIAlertView in a few categories.
 
 Each use case requires its own delegate.
 
 Each of the UIAlertView's button titles must be unique. Sorry. (I just haven't yet understood the usefulness of an alert view with redundant button titles.) 
 
 After I wrote and used this, I came across UIBAlertView.
 It's probably better.
 See:
 http://stavash.wordpress.com/2013/01/31/quick-tip-uialertview-with-a-block-callback/
 https://github.com/stavash/UIBAlertView
 
 */
@interface PNAlertViewHelper : NSObject
<UIAlertViewDelegate>

@property (readwrite, strong) NSDictionary *mapButtonTitlesToInvocations;

/** 
    A convenience method. Builds an NSInvocation with no arguments, to associate with the button title. 
 Calls -setInvocation:forButtonTitle:.
 
 @param target the intended receiver of the invocation
 @param selector the selector on the target used to build the NSInvocation
 @param buttonTitle the title for the associated alert view button. When the user touches this button, the receiver invokes the NSInvocation.
 */
-(void)setTarget: (NSObject*)target
        selector: (SEL)selector
  forButtonTitle: (NSString*)buttonTitle;

/** 
    This method is more general than -setTarget:selector:forButtonTitle:. Use this method if you need an invocation with one or more arguments.
 @param invocation the NSInvocation to invoke when the user touches the button.
 @param buttonTitle the title for the alert view button. When the user touches this button, the receiver invokes the NSInvocation.
 */
-(void)setInvocation: (NSInvocation*)invocation
      forButtonTitle: (NSString*)buttonTitle;

/**
	The required UIAlertViewDelegate method.
	@param alertView the receiver's alert view
	@param buttonIndex the index of the button the user touched.
 */
- (void)        alertView:(UIAlertView *)alertView
didDismissWithButtonIndex:(NSInteger)buttonIndex;

@end
