
#import <UIKit/UIKit.h>
#import <CoreData/CoreData.h>

/**
 
 PNDocumentEntityTVController is based on CoreDataTableViewController
 from Stanford CS193p Winter 2013,
 and on Ray Wenderlich's example:
 http://www.raywenderlich.com/999/core-data-tutorial-how-to-use-nsfetchedresultscontroller
 
 
 In many cases for table views, adopting developers can subclass PNDocumentEntityTVController for their custom view controllers. Be sure to call -setDocument:entityName:sortDescriptors: when:
 [1] your custom view controller first receives a managed document (in a PNDocToken); and
 [2] if ever the document changes (e.g, from PNCloseOpen).
 
 */
@interface PNDocumentEntityTVController : UITableViewController <NSFetchedResultsControllerDelegate>
{
    Class _tableViewCellClass;
}
@property (strong, nonatomic) NSFetchedResultsController *fetchedResultsController;
@property (readonly, strong, nonatomic) UIManagedDocument *document;
@property (readonly, strong, nonatomic) NSString *entityName;

@property (readwrite) NSString *tableViewCellClassName;
@property (readwrite, strong) NSString *tableViewCellReuseID;


/**
	Forces the fetchedResultsController to refetch the data. The fetchedResultsController observes the managed object context for changes, so you rarely need to call this method.
 */
- (void)performFetch;

/**
	This property is the secret sauce for this class from from CS193. See the CS193 documentation: 
 "   Turn this on before making any changes in the managed object context that
 are a one-for-one result of the user manipulating rows directly in the table view.
 Such changes cause the context to report them (after a brief delay),
 and normally our fetchedResultsController would then try to update the table,
 but that is unnecessary because the changes were made in the table already (by the user)
 so the fetchedResultsController has nothing to do and needs to ignore those reports.
 Turn this back off after the user has finished the change.
 Note that the effect of setting this to NO actually gets delayed slightly
 so as to ignore previously-posted, but not-yet-processed context-changed notifications,
 therefore it is fine to set this to YES at the beginning of, e.g., tableView:moveRowAtIndexPath:toIndexPath:,
 and then set it back to NO at the end of your implementation of that method.
 It is not necessary (in fact, not desirable) to set this during row deletion or insertion
 (but definitely for row moves)."
 */
@property (nonatomic) BOOL suspendChangeTrackingInMoc;

/**
	Set to YES to get some debugging output in the console.
 */
@property BOOL debug;

-(UITableViewCell*)newTableViewCell;

/**
	This method adopts the CS193 CoreDataTableViewController class for more convenient use with the PNDocSupport static library. We build the fetchedResultsController's fetch request from the entity name and sort descriptors. The NSFetchedResultsController uses the document's monaged object context.
 
 @param aDocument Developers adopting PNDocSupport will provide the document from a PNDocToken.
 @param anEntityName An entity from aDocument's data model
 @param someSortDescriptors Determines the order of items in the table view.
 
 Be sure to call -setDocument:entityName:sortDescriptors: when:
 [1] your custom view controller first receives a managed document (in a PNDocToken); and
 [2] if ever the document changes (e.g, from PNCloseOpen).
 */
-(void)setDocument: (UIManagedDocument*)aDocument
        entityName: (NSString*)anEntityName
   sortDescriptors: (NSArray*)someSortDescriptors;

@end
