//
//  CoreDataTableViewController.m
//
//  Created for Stanford CS193p Winter 2013.
//  Copyright 2013 Stanford University. All rights reserved.
//

#import "PNDocumentEntityTVController.h"

@interface PNDocumentEntityTVController()
@property (nonatomic) BOOL beganUpdates;

@end

@implementation PNDocumentEntityTVController

#pragma mark - Properties

@synthesize fetchedResultsController = _fetchedResultsController;
@synthesize suspendChangeTrackingInMoc = _suspendAutomaticTrackingOfChangesInManagedObjectContext;
@synthesize debug = _debug;
@synthesize beganUpdates = _beganUpdates;

@synthesize document = _document;
@synthesize entityName = _entityName;

@dynamic tableViewCellClassName;

@synthesize tableViewCellReuseID = _tableViewCellReuseID;


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}


-(Class)tableViewCellClass
{
    if( nil == _tableViewCellClass ){
        
        _tableViewCellClass =
        NSClassFromString(@"UITableViewCell"); //[UITableViewCell class];
        
    }
    return _tableViewCellClass;
}
-(void)setTableViewCellClassName:(NSString*)aTableViewCellClassName
{
    Class c = NSClassFromString( aTableViewCellClassName );
    
    NSObject *test =
    [[c alloc] initWithStyle: UITableViewCellStyleDefault
             reuseIdentifier: nil];
    
    NSAssert( [test isKindOfClass: [UITableViewCell class]],
             @"Bogus tableViewCellClass = %@",
             aTableViewCellClassName);

    _tableViewCellClass = c;
}

-(UITableViewCell*)newTableViewCell
{
    Class c = [self tableViewCellClass];
    
    UITableViewCell *newCell =
    [[c alloc] initWithStyle: UITableViewCellStyleDefault
             reuseIdentifier: self.tableViewCellReuseID];
    return newCell;
}

#pragma mark - Fetching

- (void)performFetch
{
    if (self.fetchedResultsController) {
        
        // Refactored from CS193 for clarity:
        if (self.debug){
            // This passage is all about debug messages:
            if (self.fetchedResultsController.fetchRequest.predicate) {
                NSLog(@"[%@ %@] fetching %@ with predicate: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), self.fetchedResultsController.fetchRequest.entityName, self.fetchedResultsController.fetchRequest.predicate);
            } else {
                NSLog(@"[%@ %@] fetching all %@ (i.e., no predicate)", NSStringFromClass([self class]), NSStringFromSelector(_cmd), self.fetchedResultsController.fetchRequest.entityName);
            }
        }        
        
        NSError *error;
        [self.fetchedResultsController performFetch:&error];
        if (error) NSLog(@"[%@ %@] %@ (%@)", NSStringFromClass([self class]), NSStringFromSelector(_cmd), [error localizedDescription], [error localizedFailureReason]);

        
    } else {
        if (self.debug) NSLog(@"[%@ %@] no NSFetchedResultsController (yet?)", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
    }
    [self.tableView reloadData];
}

- (void)setFetchedResultsController:(NSFetchedResultsController *)newfrc
{
    NSFetchedResultsController *oldfrc = _fetchedResultsController;
    if (newfrc != oldfrc) {
        _fetchedResultsController = newfrc;
        newfrc.delegate = self;
        if ((!self.title || [self.title isEqualToString:oldfrc.fetchRequest.entity.name]) && (!self.navigationController || !self.navigationItem.title)) {
            self.title = newfrc.fetchRequest.entity.name;
        }
        if (newfrc) {
            if (self.debug) NSLog(@"[%@ %@] %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), oldfrc ? @"updated" : @"set");
            [self performFetch]; 
        } else {
            if (self.debug) NSLog(@"[%@ %@] reset to nil", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
            [self.tableView reloadData];
        }
    }
}

-(void)setDocument: (UIManagedDocument*)aDocument
        entityName: (NSString*)anEntityName
   sortDescriptors: (NSArray*)someSortDescriptors
{
    self.fetchedResultsController = nil;
    
    if( nil == aDocument ) return;
    
    _document = aDocument;
    NSAssert( [_document isKindOfClass:[UIManagedDocument class]],
             @"Bogus document");
    _entityName = anEntityName;
    NSAssert( [_entityName length],
             @"Bogus entityName");
    
    NSManagedObjectContext *moc = self.document.managedObjectContext;
    if( nil != moc ){
        NSFetchRequest *request =
        [NSFetchRequest fetchRequestWithEntityName: self.entityName];
        request.sortDescriptors = someSortDescriptors;
        
        self.fetchedResultsController =
        [[NSFetchedResultsController alloc] initWithFetchRequest:request
                                            managedObjectContext:moc
                                              sectionNameKeyPath:nil
                                                       cacheName:nil];

    }

}
#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [[self.fetchedResultsController sections] count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [[[self.fetchedResultsController sections] objectAtIndex:section] numberOfObjects];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	return [[[self.fetchedResultsController sections] objectAtIndex:section] name];
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index
{
	return [self.fetchedResultsController sectionForSectionIndexTitle:title atIndex:index];
}

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView
{
    return [self.fetchedResultsController sectionIndexTitles];
}
- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Subclasses must override:
    
    return nil;
    
}

#pragma mark - NSFetchedResultsControllerDelegate

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    if (!self.suspendChangeTrackingInMoc) {
        [self.tableView beginUpdates];
        self.beganUpdates = YES;
    }
}

- (void)controller:(NSFetchedResultsController *)controller
  didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
		   atIndex:(NSUInteger)sectionIndex
	 forChangeType:(NSFetchedResultsChangeType)type
{
    if (!self.suspendChangeTrackingInMoc)
    {
        switch(type)
        {
            case NSFetchedResultsChangeInsert:
                [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
                break;
                
            case NSFetchedResultsChangeDelete:
                [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
                break;
        }
    }
}


- (void)controller:(NSFetchedResultsController *)controller
   didChangeObject:(id)anObject
	   atIndexPath:(NSIndexPath *)indexPath
	 forChangeType:(NSFetchedResultsChangeType)type
	  newIndexPath:(NSIndexPath *)newIndexPath
{		
    if (!self.suspendChangeTrackingInMoc)
    {
        switch(type)
        {
            case NSFetchedResultsChangeInsert:
                [self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
                break;
                
            case NSFetchedResultsChangeDelete:
                [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
                break;
                
            case NSFetchedResultsChangeUpdate:
                [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
                break;
                
            case NSFetchedResultsChangeMove:
                [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
                [self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
                break;
        }
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    if (self.beganUpdates) [self.tableView endUpdates];
}

- (void)endSuspensionOfUpdatesDueToContextChanges
{
    _suspendAutomaticTrackingOfChangesInManagedObjectContext = NO;
}

- (void)setSuspendChangeTrackingInMoc:(BOOL)suspend
{
    if (suspend) {
        _suspendAutomaticTrackingOfChangesInManagedObjectContext = YES;
    } else {
        [self performSelector:@selector(endSuspensionOfUpdatesDueToContextChanges)
                   withObject:0
                   afterDelay:0];
    }
}

@end

