//
//  DocumentController+IBActions.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/17/12.
//
//

#import "DocumentController+IBActions.h"
#import "DocumentController+TokenManaging.h"
#import "DocumentController+ViewUpdating.h"

#import "PNDeleteUUIDs.h"

#import "PNURLToken.h"

#import "PNDocToken.h"
#import "PNDocToken+FileOperations.h"

#import "PNCloudURLToken.h"

#import "PNDocSupportNetController.h"

#import "PNTransition.h"

#import "PNDocTokenTransition.h"

#import "RootViewController.h"
#import "PNURLTokenPlace.h"

  
#import "PNDocSupportNetController+TokenVending.h"
#import "PNDocSupportNetController+TransitionFiring.h"
#import "PNDocSupportNetController+PipelineTransitionFiring.h"

#import "NSURL+NPAssisting.h"

#import "PNOpenDoc.h"

#import "PNDocSupportNet.h"

@implementation DocumentController (IBActions)
#pragma mark Action methods

-(IBAction)deleteURLAction:(id)sender
{
    NSLog(@"\n\n+++++++\n -deleteURLAction: \n+++++++\n\n");
    
    PNURLToken *haplessURLToken = [self selectedToken];
    if( nil == haplessURLToken ) return;
    
    NSAssert( [(NSObject*)haplessURLToken isKindOfClass:[PNURLToken class]],
             @"Bogus selection--should be a url token");
    
    
    BOOL isAvailable = ((PNURLToken*)haplessURLToken).isAvailableValue;
    NSAssert( isAvailable, @"Don't try to delete the URL of an open document.");
    
    NSSet *uuids = [NSSet setWithObject:haplessURLToken.uuid];
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    [dsnc deleteURLsWithUuids: uuids
                       sender: self];
    
}

-(IBAction)newDocAction:(id)sender
{
    NSLog(@"\n\n+++++++\n -newDocAction: \n+++++++\n\n");
    
    RootViewController *rvc = self.rootViewController;
    
    // Make a new document:
    NSString *newDocClassName = NSStringFromClass([UIManagedDocument class]);
        
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    

    [dsnc     closeDocToken: self.docToken
                  fromPlace: rvc.docTokensPlace
 createDocWithLocalizedName: @"[New]"
          documentClassName: newDocClassName
          midPathComponents: nil
           creationDelegate: rvc
                    toPlace: rvc.docTokensPlace
                     sender: self];

}

-(IBAction)openDocAction:(id)sender
{
    NSLog(@"\n\n+++++++\n -openDocAction: \n+++++++\n\n");
    RootViewController *rvc = self.rootViewController;
    
    
    PNURLToken *selectedToken = [self selectedToken];
    if( nil == selectedToken ) return;
    
    NSAssert( [(NSObject*)selectedToken isKindOfClass:[PNURLToken class]],
             @"Bogus selection--should be a url token");
        
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    
    // hack for -installDummyDocToken
    if( nil == rvc.docToken ){
        [dsnc.docSupportNet.openDoc enableFor: self
                                         uuid: selectedToken.uuid
                                      toPlace: rvc.docTokensPlace];
    }else{
        [dsnc closeDocToken: rvc.docToken
                  fromPlace: rvc.docTokensPlace
            openDocWithUUID: selectedToken.uuid
                    toPlace: rvc.docTokensPlace
                     sender: self];

    }
    
}

-(IBAction)renameDocAction:(id)sender
{
    NSLog(@"\n\n+++++++\n -renameDocAction: \n+++++++\n\n");
    
    NSString *newDocName = self.docNameTextField.text;
    NSLog(@"  %@ -> %@ ",
          self.docToken.urlToken.localizedName,
          newDocName);
    
    self.docNameTextField.enabled = NO;
    
    if( [newDocName isEqualToString: self.docToken.urlToken.localizedName] ) {
        //[self.docToken saveDocumentForOverwriting]; // don't do this:
        // Saving hoarks cloud docs.
    }else{
        
        PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
        RootViewController *rvc = self.rootViewController;
        
        NSUserDefaults *stdUserDefaults = [NSUserDefaults standardUserDefaults];
        
        // See Settings.bundle/Root.plist
        // We expect a switch tied to @"deleteOriginal".
        BOOL flag = [stdUserDefaults boolForKey: @"deleteOriginal"];
        NSLog(@" stdUserDefaults.deleteOriginal = %@ ",
              flag ? @"YES" : @"NO" );

        [dsnc renameDocToken: self.docToken
                     newName: newDocName
                   fromPlace: rvc.docTokensPlace
                     toPlace: rvc.docTokensPlace
              deleteOriginal: flag
                      sender: self ];
        
    }
    return;
}

@end
