//
//  DocumentController+TokenManaging.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/17/12.
//
//

#import "DocumentController+TokenManaging.h"
#import "DocumentController+IBActions.h"
#import "DocumentController+ViewUpdating.h"

#import "PNURLToken.h"

#import "PNDocToken.h"
#import "PNDocToken+FileOperations.h"

#import "PNURLToken.h"
#import "PNCloudURLToken.h"

#import "PNDocSupportNetController.h"
#import "PNDocSupportNet.h"

#import "PNTransition.h"
#import "PNTransition+Transforming.h"

#import "RootViewController.h"
#import "PNURLTokenPlace.h"

  

@implementation DocumentController (TokenManaging)

-(PNURLToken*)selectedToken
{
    NSIndexPath *selectedPath = [self.tableView indexPathForSelectedRow];
    
    PNURLToken *selectedToken = [self urlTokenAtIndexPath: selectedPath];
    return selectedToken;
}
-(void)selectCurrentDocument
{
    PNURLToken *currentURLToken = self.rootViewController.docToken.urlToken;
    
    if( nil == currentURLToken) return;
    
    NSIndexPath *indexPath = [self indexPathOfURLToken: currentURLToken];
    
    BOOL inRange = NO;
    
    NSUInteger nSections = [self.tableView numberOfSections];
    if( indexPath.section < nSections ){
        NSUInteger nRows = [self.tableView numberOfRowsInSection: indexPath.section];
        if( indexPath.row < nRows ){
            inRange = YES;
        }
    }
    
    if( !inRange ) return;
    
    if( (NSNotFound != indexPath.row) && (NSNotFound != indexPath.section) ){
        [self.tableView selectRowAtIndexPath: indexPath
                                    animated: YES
                              scrollPosition: UITableViewScrollPositionMiddle];
    }
    [self qualifyButtonsAndNameField];
}
-(PNURLToken*)tokenForIndexPath: (NSIndexPath *)indexPath
{
    PNURLToken *result = nil;
    
    NSUInteger row = indexPath.row;
    NSUInteger section = indexPath.section;
    
    switch ( section ) {
        case 0: // device
        {
            result =
            [m_currentDeviceURLs objectAtIndex: row];
            break;
        }
        case 1: // iCloud
        {
            result =
            [m_currentCloudURLs objectAtIndex: row];
            break;
        }
        default:
        {
            break;
        }
    }
    
    return result;
    
}
-(NSIndexPath*)indexPathOfURLToken: (PNURLToken*)urlToken
{
    NSUInteger section = NSNotFound;
    NSUInteger row = NSNotFound;
    
    if( nil != urlToken) {
        
        if( [m_currentDeviceURLs containsObject: urlToken]) {
            section = 0;
            row = [m_currentDeviceURLs indexOfObject: urlToken];
        }
        if( [m_currentCloudURLs containsObject: urlToken]) {
            section = 1;
            row = [m_currentCloudURLs indexOfObject: urlToken];
        }
        
    }
    
    return [NSIndexPath indexPathForRow: row
                              inSection: section];
}
-(PNURLToken*)urlTokenAtIndexPath: (NSIndexPath*)indexPath
{
    if( nil == indexPath ) return nil;
    
    NSUInteger row = indexPath.row;
    NSUInteger section = indexPath.section;
    
    PNURLToken *urlToken = nil;
    
    switch ( section ) {
        case 0: // device
        {
            urlToken =
            [m_currentDeviceURLs objectAtIndex: row];
            break;
        }
        case 1: // iCloud
        {
            urlToken =
            [m_currentCloudURLs objectAtIndex: row];
            break;
        }
        default:
        {
            break;
        }
    }
    
    return urlToken;
    
}


#pragma mark UIDataSourceModelAssociation methods -- maintaining the selection and visible rows.
#pragma mark Use UUIDs as identifiers.
#pragma mark These methods require that we assert restorationIdentifier on the table view and its entire view hierarchy. ... Um, later.

- (NSIndexPath *) indexPathForElementWithModelIdentifier:(NSString *)identifier
                                                  inView:(UIView *)view
{
    NSAssert( (view == self.tableView), @"Bogus view");
    /*
     The view uses the information to ensure that the rows
     with the same data are once again selected or scrolled into view,
     even if those rows are in a different location now.
     */
    BOOL validUuid = [PNURLToken validUuid: identifier];
    NSAssert( validUuid,
             @"Oops... There was a misunderstanding. We're using uuid strings for identifiers");
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    PNURLToken *element = [dsnc.docSupportNet urlTokenForUuid: identifier];
    
    NSAssert( (nil != element),
             @"Fooled me. I thought we'd treat relocated rows, not missing ones." );
    
    NSIndexPath *indexPath = [self indexPathOfURLToken: element];
    return indexPath;
}

- (NSString *) modelIdentifierForElementAtIndexPath:(NSIndexPath *)idx
                                             inView:(UIView *)view
{
    NSAssert( (view == self.tableView), @"Bogus view");
    /*
     The view uses the information to ensure that the rows
     with the same data are once again selected or scrolled into view,
     even if those rows are in a different location now.
     */
    
    PNURLToken *target = [self urlTokenAtIndexPath: idx];
    NSAssert( (nil != target),
             @"Fooled me. I thought we'd treat relocated rows, not missing ones." );
    
    NSString *uuid = target.uuid;
    
    return uuid;
}



@end
