//
//  DocumentController+ViewUpdating.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 10/17/12.
//
//

#import "DocumentController+ViewUpdating.h"
#import "DocumentController+TokenManaging.h"
#import "RootViewController.h"
#import "PNDocSupportNetController.h"
#import "PNDocSupportNetController+TokenVending.h"
#import "PNDocSupportNetController+StorageControlling.h"

#import "PNTransition.h"
#import "PNDocSupportNet.h"

// #import <PNDocSupport/PNUuidStatus.h>
#import "PNURLTokenPlace.h"

@implementation DocumentController (ViewUpdating)
-(void)updateTableViewData
{
    // Refresh the ordered URLs
    m_currentDeviceURLs = [[self nextLocalURLs] copy];
    if( nil == m_currentDeviceURLs ){
        m_currentDeviceURLs = [[NSOrderedSet alloc] init];
    }
    
    m_currentCloudURLs  = [[self nextCloudURLs] copy];
    if( nil == m_currentDeviceURLs ){
        m_currentCloudURLs = [[NSOrderedSet alloc] init];
    }
    
    // [self.tableView reloadData] should be called only in this method.
    // In general, the URL array ivars should be updated before the
    // table view reloads its data.
    [self.tableView reloadData];
    
}
-(void)updateViewFromModel
{
    NSLog(@"-[DocumentController updateViewFromModel]");
    
    PNDocToken *docToken = self.rootViewController.docToken;
    
    if( nil == docToken ){
        NSLog(@"self.rootViewController.docToken == nil");
    }else{
        NSString *localizedName = docToken.urlToken.localizedName;
        if( ![localizedName isEqualToString:self.docNameTextField.text] ){
            
            NSLog(@"self.rootViewController.docToken.urlToken.localizedName = %@", localizedName);
        }
        self.docNameTextField.text = localizedName;
    }
}
-(void)qualifyButtonsAndNameField
{
    BOOL transitionPending = (nil != self.activeTransition);
    
    self.closeCreateDocButton.hidden  = transitionPending;
    self.closeCreateDocButton.enabled = !transitionPending;
    
    // 2013 April 03
    // Rename for iCloud won't work until we get more clues from Apple.
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    BOOL canRename = [dsnc canRenameDocuments];
    self.docNameTextField.enabled = canRename && !transitionPending;
    
        
    PNURLToken *selectedToken = [self selectedToken];
    
//    if( nil == selectedToken){
//        // Hackish, but the UI is not my main focus.
//        // I want to implement/debug the framework.
//        [self performSelectorOnMainThread:@selector(selectCurrentDocument)
//                               withObject:nil
//                            waitUntilDone: YES];
//    }
    // This passage controls the [Open] and [Delete] buttons:
    // If the selected token is assigned, the user can't open or delete it.
    // The user can only open or delete the selected token if it is available.
    PNDocSupportNetController *dscn =
    [PNDocSupportNetController sharedDocSupportNetController];
    enum PNUuidStatus status =
    [dscn.docSupportNet statusOfUuid: selectedToken.uuid];
    
    BOOL assigned =
    (PNUuidStatusAssignedLocalURL == status) ||
    (PNUuidStatusAssignedCloudURL == status);
    
    self.openDocButton.hidden  = self.deleteDocButton.hidden  =  assigned; // current doc url is assigned
    self.openDocButton.enabled = self.deleteDocButton.enabled = !assigned;
    
}
-(void)updateUI
{
    [self updateTableViewData];
    [self updateViewFromModel];
    [self qualifyButtonsAndNameField];
}

-(NSOrderedSet*)nextLocalURLs
{
    PNDocSupportNetController *dscn = [PNDocSupportNetController sharedDocSupportNetController];

    NSOrderedSet *localURLs = [dscn orderedLocalURLTokens];
    return localURLs;
}
-(NSOrderedSet*)nextCloudURLs
{
    PNDocSupportNetController *dscn = [PNDocSupportNetController sharedDocSupportNetController];
    
    NSOrderedSet *cloudURLs = [dscn orderedCloudURLTokens];
    return cloudURLs;
}

@end
