//
//  DocumentController
//  TaggedLocationsDemo
//
//  Created by DevGuy on 3/26/12.
//  Copyright (c) 2012 NowPicture LLC. All rights reserved.
//

#import "DocumentController.h"
#import "DocumentController+TokenManaging.h"
#import "DocumentController+ViewUpdating.h"

#import "PNDocToken.h"
#import "PNDocToken+FileOperations.h"

#import "PNURLToken.h"
#import "PNCloudURLToken.h"

#import "PNTransition.h"

#import "PNDocSupportNetController.h"
#import "PNDocSupportNet.h"

#import "PNTransition.h"

#import "RootViewController.h"
#import "PNURLTokenPlace.h"

#import "NSString+PNEllipsis.h"

#import "PNCreateDoc.h"
#import "PNDocTokenPlace.h"

#import "PNDocSupportNetController.h"
  

@interface DocumentController (Tracking) 

-(void)mocObjectsDidChange: (NSNotification*)notification;
-(void)documentDidSave: (NSNotification*)notification;
-(void)documentStateDidChange: (NSNotification*)notification;

-(void)markingDidChange;

@end

@implementation DocumentController (Tracking) 

-(void)mocObjectsDidChange: (NSNotification*)notification
{
    NSLog(@"-[%@ mocObjectsDidChange:", NSStringFromClass([self class]) );
    
    [self updateUI];
}
-(void)documentDidSave: (NSNotification*)notification
{
    [self updateUI];
    
}
-(void)documentStateDidChange: (NSNotification*)notification
{
    [self updateUI];
    
}


-(void)markingDidChange
{
    NSAssert( [[NSThread currentThread] isMainThread],
             @"Not main thread.");
    
    
    [self updateUI];
    
}
@end



@implementation DocumentController

//@dynamic docToken;

@synthesize rootViewController;

@synthesize docNameTextField;

@synthesize closeCreateDocButton;
@synthesize deleteDocButton;
@synthesize openDocButton;
@synthesize tableView;

@synthesize notificationObservers;
@synthesize activeTransition;

@dynamic docToken;

-(PNDocToken *)docToken
{
    // A convenience method:
    return self.rootViewController.docToken;
    // OrganizerController and RootViewController both show aspects of the same document.
    // But, a doc token can have only one delegate,
    // and in this case, it's RootViewController.
}

#pragma mark UIViewController stuff: 

- (void)viewDidLoad
{
    
    [super viewDidLoad];
	// Do any additional setup after loading the view.

    self.tableView.delegate = self;
    self.tableView.allowsSelection = YES;
    self.tableView.allowsMultipleSelection = NO;

    [self setActiveTransition: nil];
    
    [self docTokenDidBecomeAvailable: self.rootViewController.docToken];
    
    self.notificationObservers = [NSMutableArray arrayWithCapacity:2];
    
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    PNDocSupportNet *dsNet = [PNDocSupportNetController sharedDocSupportNetController].docSupportNet;
    id observer =
    [dfc addObserverForName: PNNetDidChangeMarkings
                     object: dsNet
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self performSelectorOnMainThread: @selector(markingDidChange)
                                withObject: nil
                             waitUntilDone: YES];
     }];
    [self.notificationObservers addObject: observer];
    
    observer =
    [dfc addObserverForName: TLDDocTokenDidBecomeAvailable // TLD means TaggedLocationsDemo
                     object: self.rootViewController 
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self updateUI];
     }];
    [self.notificationObservers addObject: observer];

}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    
    m_currentDeviceURLs = nil;
    m_currentCloudURLs = nil;
}

-(void)removeAllNotificationObservers
{
    NSNotificationCenter *dfC = [NSNotificationCenter defaultCenter];
    
    for( id observer in self.notificationObservers ){
        [dfC removeObserver: observer];
    }
    [self.notificationObservers removeAllObjects];
}

-(void)dealloc
{
    [self removeAllNotificationObservers];
    self.notificationObservers = nil;
    self.closeCreateDocButton = nil;
    self.openDocButton = nil;
    self.deleteDocButton = nil;
}

#pragma mark PNDocTokenMarking methods

-(void)docTokenDidBecomeAvailable: (PNDocToken*)aDocToken
{
    [self updateUI];
}

- (void)docToken: (PNDocToken*)aDocToken 
didFailWithError: (NSError*)error
{
    [self updateUI];
}

// Typically, the marker performs some damage-control and/or clean-up:



#pragma mark PNTransitionDelegate methods:

-(void)didFireTransition: (PNTransition*)transition
{

    NSAssert( (transition == self.activeTransition), @"Bogus transition.");
    
    self.activeTransition = nil;
    
    [self markingDidChange];
}
-(BOOL)shouldAttemptToRollbackTransition: (PNTransition*)transition
{
    // The initial version of the framework implements exactly one very simple rollback case:
    
    BOOL isMine =  (transition == self.activeTransition);
    BOOL isNewDoc = [transition isKindOfClass:[PNCreateDoc class]];
    
    return (isMine && isNewDoc);
}
-(void)didFailTransition: (PNTransition*)transition
  rolledBackSuccessfully: (BOOL)success
{
    [self setActiveTransition: nil];
    [self markingDidChange];
}

#pragma mark UITableViewDataSource methods:

static NSString *reuseID = @"DocumentNameCell";

-(NSString*)tableView:(UITableView *)tableView 
titleForHeaderInSection:(NSInteger)section
{
    NSString *title = @"??";
    switch ( section) {
            
        case 0:
        {
            title = @"Device:";
            break; 
        }
        case 1:
        {
            title = @"Cloud:";
            break; 
        }
        default:
        {
            break;
        }
    }
    return title;
}
- (NSInteger)tableView:(UITableView *)aTableView 
 numberOfRowsInSection:(NSInteger)section
{
    NSInteger result = NSNotFound;
        
    switch ( section ) {
        case 0: // Device
        {
            result = [m_currentDeviceURLs count];
            break;
        }
        case 1: // iCloud
        {
            result = [m_currentCloudURLs count];
            break;
        }
        default:
        {
            break;
        }
            
    }
    return result;
}

-(NSString*)cellTextForURLToken: (PNURLToken*)haplessToken
{
    NSString *cellText =
    [NSString stringWithFormat: @"%@ <%@>",
     haplessToken.localizedName,
     [haplessToken.uuid pnEllipsis] ];
    
    return cellText;
}
-(void)configureCell:(UITableViewCell *) cell
         atIndexPath:(NSIndexPath *)indexPath
{

    //PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    
    NSUInteger section = indexPath.section;
    NSUInteger row = indexPath.row;
    
    switch ( section) {
        case 0: // Device
        {
            PNURLToken *token = 
            [m_currentDeviceURLs objectAtIndex: row];
            enum PNUuidStatus status = token.uuidStatus;
            
            //cell.textLabel.text = token.localizedName;
            cell.textLabel.text = [self cellTextForURLToken: token];
            
            switch( status ){
                case PNUuidStatusAssignedLocalURL:
                {
                    cell.textLabel.enabled = NO;
                    break;
                }
                case PNUuidStatusUnassignedLocalURL:
                {
                    cell.textLabel.enabled = YES;
                    break;
                }
                case PNUuidStatusInvalid:
                case PNUuidStatusUnknown:
                case PNUuidStatusAssignedCloudURL:
                case PNUuidStatusUnassignedCloudURL:
                default:
                {
                    NSLog(@"-configureCell:atIndexPath: [r=%d, s=%d]",
                          [indexPath row],
                          [indexPath section]);
                    NSLog(@"  token = %@",  
                          [token quickLook] );
                    
                    NSAssert( NO, @"Bogus status %d of token %@",
                             status,
                             [token quickLook] );
                    break;
                }
            }
            break;
        }
        case 1: // iCloud
        {
            PNURLToken *token = 
            [m_currentCloudURLs objectAtIndex: row];
            enum PNUuidStatus status = token.uuidStatus;
           
            //cell.textLabel.text = token.localizedName;
            cell.textLabel.text = [self cellTextForURLToken: token];
            
            switch( status ){
                case PNUuidStatusAssignedCloudURL:
                {
                    cell.textLabel.enabled = NO;
                    break;
                }
                case PNUuidStatusUnassignedCloudURL:
                {
                    cell.textLabel.enabled = YES;
                    break;
                }
                case PNUuidStatusInvalid:
                case PNUuidStatusUnknown:
                case PNUuidStatusAssignedLocalURL:
                case PNUuidStatusUnassignedLocalURL:
                default:
                {
                    NSLog(@"-configureCell:atIndexPath: [r=%d, s=%d]",
                          [indexPath row],
                          [indexPath section]);
                    NSLog(@"  token = %@",  
                          [token quickLook] );
                    
                    NSAssert( NO, @"Bogus status %d of token %@",
                             status,
                             [token quickLook] );
                    break;
                }
            }
            break;
        }
        default:
        {
            break;
        }
    }
    cell.textLabel.hidden = NO;
    cell.detailTextLabel.hidden = YES;
    
    return;
}


// Row display. Implementers should *always* try to reuse cells by setting each cell's reuseIdentifier and querying for available reusable cells with dequeueReusableCellWithIdentifier:
// Cell gets various attributes set automatically based on table (separators) and data source (accessory views, editing controls)

- (UITableViewCell *)tableView:(UITableView *)aTableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
        
    UITableViewCell *cell = 
    [self.tableView dequeueReusableCellWithIdentifier: reuseID];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                      reuseIdentifier:reuseID];
    }
    
    [self configureCell: cell
            atIndexPath: indexPath];
    
	return cell;
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView             // Default is 1 if not implemented
{
    return 2;
}
- (NSString *)tableView:(UITableView *)aTableView 
titleForFooterInSection:(NSInteger)section
{
    return nil;
}

// Editing

// Individual rows can opt out of having the -editing property set for them. If not implemented, all rows are assumed to be editable.
- (BOOL)tableView:(UITableView *)aTableView 
canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}


// Moving/reordering

// Allows the reorder accessory view to optionally be shown for a particular row. By default, the reorder control will be shown only if the datasource implements -tableView:moveRowAtIndexPath:toIndexPath:
- (BOOL)tableView:(UITableView *)aTableView 
canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

// Index
//
//- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)aTableView;                                                    // return list of section titles to display in section index view (e.g. "ABCD...Z#")
//- (NSInteger)tableView:(UITableView *)aTableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index;  // tell table which section corresponds to section title/index (e.g. "B",1))
//
// Data manipulation - insert and delete support

// After a row has the minus or plus button invoked (based on the UITableViewCellEditingStyle for the cell), the dataSource must commit the change
- (void)tableView:(UITableView *)aTableView 
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"-tableView:commitEditingStyle:forRowAtIndexPath:");

}

// Data manipulation - reorder / moving support

- (void)tableView:(UITableView *)aTableView 
moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath 
      toIndexPath:(NSIndexPath *)destinationIndexPath
{
    
}
#pragma mark UITableViewDelegate methods for selections.
- (void)   tableView:(UITableView *)tableView
didEndDisplayingCell:(UITableViewCell *)cell
   forRowAtIndexPath:(NSIndexPath *)indexPath
{
    // This gets called more often than I thought.
    // Good opportunity for thrashing in the UI.
    // Don't do anything.
}
-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{    
    [self qualifyButtonsAndNameField];
}
-(void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Don't update anything.
}

#pragma mark UITextFieldDelegate for self.docNameTextField
- (void)textFieldDidEndEditing:(UITextField *)textField
{
    NSAssert( ( textField == self.docNameTextField ), @"Bogus text field");

}
- (void)textFieldDidBeginEditing: (UITextField *)textField
{
    NSAssert( ( textField == self.docNameTextField ), @"Bogus text field");
   
    
}
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    NSAssert( ( textField == self.docNameTextField ), @"Bogus text field");
    
    [textField resignFirstResponder];
    
    return NO;
}

// Tell the system what we support
- (NSUInteger)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskAllButUpsideDown;
}

// Tell the system It should autorotate
- (BOOL) shouldAutorotate {
    return YES;
}

//// Tell the system which initial orientation we want to have
//- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation {
//    return UIInterfaceOrientationPortrait;
//}
@end
