//
//  RootViewController+Observing.m
//  TaggedLocationsDemo
//
//  Created by Don Briggs on 6/29/12.
//  Copyright (c) 2012 Directed Graph Software LLC (AB). All rights reserved.
//

#import "RootViewController+Observing.h"
#import "RootViewController+ViewUpdating.h"

#import "PNDocToken.h"
#import "PNURLToken.h"
#import "PNCloudURLToken.h"


#import "PNDocSupportNetController+StorageControlling.h"
#import "RootViewController+StorageTracking.h"

#import "RootViewController+UuidTokenArchiving.h"

@implementation RootViewController (Observing)


-(void)observeDocument
{
    PNDocToken *myDocToken = [self docToken];
    UIManagedDocument *myDoc = [myDocToken document];
    NSManagedObjectContext *myDocMoc = myDoc.managedObjectContext;
    
    if(nil == myDocMoc) return; // Hack for -installDummyDocToken
    
    NSAssert( (nil !=myDocMoc), @"Bogus nil == myDocMoc");
    
//    NSLog(@"self.document.managedObjectContext = %@",
//          [myDocMoc description] );
    
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    id observer =
    [dfc addObserverForName: NSManagedObjectContextObjectsDidChangeNotification 
                     object: myDocMoc 
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self performSelectorOnMainThread: @selector(updateViewFromModel)
                                withObject: nil
                             waitUntilDone: YES];
     }];
    [self.documentObservers addObject: observer];
    
    observer =
    [dfc addObserverForName: NSManagedObjectContextDidSaveNotification
                     object: myDocMoc
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification)
     {
         [self performSelectorOnMainThread: @selector(updateViewFromModel)
                                withObject: nil
                             waitUntilDone: YES];
     }];
    [self.documentObservers addObject: observer];
    
    observer =
    [dfc addObserverForName: UIDocumentStateChangedNotification
                     object: myDoc
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self performSelectorOnMainThread: @selector(updateViewFromModel)
                                withObject: nil
                             waitUntilDone: YES];
     }];
    [self.documentObservers addObject: observer];
    
    if( [(self.docToken.urlToken) isKindOfClass: [PNCloudURLToken class]] ){
        
        observer =
        [dfc addObserverForName:
         NSPersistentStoreDidImportUbiquitousContentChangesNotification
                         object: [myDocMoc persistentStoreCoordinator]
                          queue: nil
                     usingBlock: ^(NSNotification *note) {
                         
                         [self.document.managedObjectContext performBlock:^{
                             
                             NSLog(@"%@: NSPersistentStoreDidImportUbiquitousContentChangesNotification: Merging changes",
                                   [[UIDevice currentDevice] model] );
                             
                             
                             NSManagedObjectContext* moc =
                             self.document.managedObjectContext;
                             [moc performBlockAndWait:^{
                                 
                                 NSUndoManager * undoManager = [moc undoManager];
                                 [undoManager disableUndoRegistration];{
                                     [moc mergeChangesFromContextDidSaveNotification:note];
                                     [moc processPendingChanges];
                                       
                                 }[undoManager enableUndoRegistration];
                                 
                                  
                             }]; // moc performBlockAndWait:
                         }]; //self.document.managedObjectContext performBlock:
                         
                         [self performSelector:@selector(updateViewFromModel)
                                    withObject:nil
                                    afterDelay: 0.5];

                     }]; // -addObserverForName:... block
        
        [self.documentObservers addObject: observer];
    } // if( [(self.docToken.urlToken) isKindOfClass: [PNCloudURLToken class]] )
    
}
-(void)ignoreDocument
{
    NSNotificationCenter *dfC = [NSNotificationCenter defaultCenter];
    
    for( id observer in self.documentObservers ){
        [dfC removeObserver: observer];
    }
    [m_documentObservers removeAllObjects];
}

-(void)observeSharedApp
{
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    UIApplication *sharedApp = [UIApplication sharedApplication];
    
    [dfc addObserver: self 
            selector: @selector(appStatusDidChange:)
                name:UIApplicationDidBecomeActiveNotification 
              object:sharedApp];
    
    [dfc addObserver: self 
            selector: @selector(appStatusDidChange:)
                name:UIApplicationDidEnterBackgroundNotification 
              object:sharedApp];
    
    id observer =
    [dfc addObserverForName: UIApplicationDidBecomeActiveNotification 
                     object: self.document.managedObjectContext 
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self appStatusDidChange: notification];
     }];
    [self.appObservers addObject: observer];
    
    observer =
    [dfc addObserverForName: UIApplicationDidEnterBackgroundNotification 
                     object: self.document.managedObjectContext 
                      queue: [NSOperationQueue mainQueue]
                 usingBlock: ^(NSNotification __strong *notification) 
     {
         [self appStatusDidChange: notification];
     }];
    [self.appObservers addObject: observer];
}

-(void)ignoreSharedApp
{
    NSNotificationCenter *dfC = [NSNotificationCenter defaultCenter];
    
    for( id observer in self.appObservers ){
        [dfC removeObserver: observer];
    }
    [m_appObservers removeAllObjects];
}

-(void)observeStorage
{
    
    
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    PNDocSupportNetController *dsnc =
    [PNDocSupportNetController sharedDocSupportNetController];
    
    id mainQueue = [NSOperationQueue mainQueue];
    
    id observer =
    [dfc addObserverForName: PNStorageDidInitializeNotification
                     object: dsnc
                      queue: mainQueue
                 usingBlock: ^(NSNotification __strong *notification)
     {
         [self openMostRecentOrCreateNewDocument];
     }];
    [self.storageObservers addObject: observer];
    
    observer = 
    [dfc addObserverForName: PNCloudAccountSignedInNotification
                     object: dsnc
                      queue: mainQueue
                 usingBlock: ^(NSNotification __strong *notification)
     {
         [self cloudAccountDidSignIn];
     }];
    [self.storageObservers addObject: observer];

 
    observer =
    [dfc addObserverForName: PNCloudAccountSignedOutNotification
                     object: dsnc
                      queue: mainQueue
                 usingBlock: ^(NSNotification __strong *notification)
     {
         [self cloudAccountDidSignOut];
     }];
    [self.storageObservers addObject: observer];

    observer =
    [dfc addObserverForName: PNCloudAccountDidChangeNotification
                     object: dsnc
                      queue: mainQueue
                 usingBlock: ^(NSNotification __strong *notification)
     {
         [self cloudAccountDidChange]; 
     }];
    [self.storageObservers addObject: observer];

}
-(void)ignoreStorage
{
    NSNotificationCenter *dfC = [NSNotificationCenter defaultCenter];
    
    for( id observer in self.storageObservers ){
        [dfC removeObserver: observer];
    }
    [m_storageObservers removeAllObjects];
}

@end
