//
//  RootViewController+UuidTokenArchiving.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 5/5/12.
//  Copyright (c) 2012 NowPicture LLC. All rights reserved.
//

#import "RootViewController+UuidTokenArchiving.h"

#import "PNDocToken.h"
#import "PNURLToken.h"
#import "PNCloudURLToken.h"

#import "PNURLToken+LifeCycling.h"
#import "PNDocTokenPlace.h"
#import "PNDocToken+LifeCycling.h"

// #import <PNDocSupport/PNUuidStatus.h>

#import "PNDocSupportNetController.h"
#import "PNDocSupportNetController+TokenVending.h"
#import "PNDocSupportNetController+StorageControlling.h"
#import "PNDocSupportNetController+TransitionFiring.h"


#import "PNDocSupportNet.h"

#import "PNURLTokenPlace.h"

#import "PNCreateDoc.h"
#import "PNOpenDoc.h"

#import "NSURL+NPAssisting.h"

#import "PNAlertViewHelper.h"


static NSString *TLDemo_Archive_DateKey     = @"TLDemo_Archive_Date";
static NSString *TLDemo_Archive_UserInfoKey = @"TLDemo_Archive_UserInfo";
static NSString *TLDemo_Archive_UUIDKey     = @"TLDemo_Archive_UUID";

@implementation RootViewController (UuidTokenArchiving)

NSString *TLDRootViewControllerMostRecentDocTokenDotUrlTokenDotUuidKey =
@"TaggedLocationDemo's UUID of the most recent document's URL token";

-(void)developmentCycleResetUbiquitousKeyValueStore
{
    /**
     Developers may wish to reset the app's NSUbiquitousKeyValueStore.
     From the debugger:
     > p [self developmentCycleResetUbiquitousKeyValueStore]
     > 
     */
    NSUbiquitousKeyValueStore *kvs = [NSUbiquitousKeyValueStore defaultStore];
    NSDictionary *dict = [kvs dictionaryRepresentation];
    NSArray *allKeys = [dict allKeys];
    
    for( NSString *nextKey in allKeys ){
        [kvs removeObjectForKey: nextKey];
    }
    [kvs synchronize];
}

-(void)archiveURLToken: (PNURLToken*)urlToken
               withKey: (NSString*)aKey;
{
    NSAssert( (nil != urlToken), @"nil url token");
    NSAssert( [urlToken isKindOfClass: [PNURLToken class]],
               @"bogus url token: %@", urlToken);
               
    NSAssert( (nil != aKey),  @"nil key");
    
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    
    NSDictionary *userInfo = urlToken.userInfo;
    if( nil == userInfo ){
        userInfo = [NSDictionary dictionary];
    }
    
    // Required keys: PNDateKey, PNUserInfoKey, PNUUIDKey
    NSDictionary *uuidDict =
    [NSDictionary dictionaryWithObjectsAndKeys:
     
     [NSDate date], TLDemo_Archive_DateKey,  // most recently archived, not most recently changed token, as in urlToken.date
     userInfo,      TLDemo_Archive_UserInfoKey,
     urlToken.uuid, TLDemo_Archive_UUIDKey,
     
     nil ];
    
    /* WRONG-HEADED
    if( [urlToken isKindOfClass:[PNCloudURLToken class]] ){
        NSUbiquitousKeyValueStore *kvs = [NSUbiquitousKeyValueStore defaultStore];

        [kvs setDictionary: uuidDict
                    forKey: aKey];
        
    }else{
        
        [ud setObject: uuidDict
               forKey: aKey];
    }
     */
    [ud setObject: uuidDict
           forKey: aKey];

    
    // -archiveURLToken:withKey: is only called from -docTokenDidBecomeAvailable:
    // -[PNDocSupportNetController scheduleRefreshCloudUsage] is all about NSUserDefaults,
    // and may invoke -localToCloud or -cloudToLocal.
    // But, RootViewController can't fire PNCloseFlipOpen until it has a document.
//    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
//    [dsnc scheduleRefreshCloudUsage];
}
-(PNURLToken*)unarchivedUrlTokenWithKey: (NSString*)aKey
{
    // [1] get the date stored in device user defaults in a dictionary stored under aKey:
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    
    NSDictionary *sandboxDict = [ud objectForKey: aKey]; 
    NSDate *sandboxDate = [sandboxDict objectForKey: TLDemo_Archive_DateKey];
    if( nil == sandboxDate ){
        sandboxDate = [NSDate distantPast];
    }
    
    // [2] get the date stored in cloud default store in a dictionary stored under aKey:
   NSUbiquitousKeyValueStore *kvs = [NSUbiquitousKeyValueStore defaultStore];
    
    NSDictionary *cloudDict = [kvs objectForKey: aKey];
    NSDate *cloudDate = [cloudDict objectForKey: TLDemo_Archive_DateKey];
    if( nil == cloudDate ){
        cloudDate = [NSDate distantPast];
    }
    
    NSDictionary *uuidDict = nil;
        
    // [3] use the most recent:
    NSComparisonResult comparison = [sandboxDate compare: cloudDate];

    switch( comparison ){
            /*
             - (NSComparisonResult)compare:(NSDate *)anotherDate
             
             Parameters
             anotherDate
             The date with which to compare the receiver.
             This value must not be nil. If the value is nil, the behavior is undefined and may change in future versions of Mac OS X.
             Return Value
             If:
             
             The receiver and anotherDate are exactly equal to each other, NSOrderedSame
             The receiver is later in time than anotherDate, NSOrderedDescending
             The receiver is earlier in time than anotherDate, NSOrderedAscending.
             */
        case NSOrderedAscending:
        {
            uuidDict = cloudDict;
            break;
        }
        case NSOrderedDescending:
        {
            uuidDict = sandboxDict;
            break;
        }
        case NSOrderedSame:
        {
            // A tie is infeasible.
            // The benefit of the doubt goes to the sandbox.
            uuidDict = sandboxDict;
            // They're probably both nil, anyway.
            
            break;
        }
    }

    if( nil == uuidDict ){
        // [4] if we didn't find an archive, return nil;
        return nil;
    }else{
        
        // [5] get the uuid from the archive:
        NSAssert( [uuidDict isKindOfClass: [NSDictionary class]], @"Bogus archive");
        
        NSArray *allKeys = [uuidDict allKeys];
        // Required keys: PNDateKey (already found, to get here), PNUserInfoKey, PNUUIDKey
        
        NSAssert( [allKeys containsObject: TLDemo_Archive_UserInfoKey], @"Unexpected archive, no user info.");
        NSAssert( [allKeys containsObject: TLDemo_Archive_UUIDKey], @"Unexpected archive, no uuid.");
        
        NSString *uuid = [uuidDict objectForKey: TLDemo_Archive_UUIDKey];
        NSAssert( [PNURLToken validUuid: uuid], @"Invalid uuid string");
        
        
        // [6] get the url token for the uuid string.
        //     the url token may be nil,
        PNDocSupportNetController *dscn =
        [PNDocSupportNetController sharedDocSupportNetController];
        PNURLToken *urlToken = [dscn.docSupportNet urlTokenForUuid: uuid];
        
        // [7] tack on the stored user info.
        //     user info may contain a document class name or other information
        //     helpful to the adopting developer
        //     (but the framework ignores the user info contents).
        NSDictionary *userInfo = [uuidDict objectForKey: TLDemo_Archive_UserInfoKey];
        urlToken.userInfo = userInfo;
        
        return urlToken;
    }
}
-(void)userAcknowledgedMissingDoc
{

}
-(void)postMissingDocumentAlert
{
    // post alert:
    NSString *title =
    NSLocalizedStringFromTable( @"Missing document",
                               @"-[NSObject(UuidTokenArchiving) postMissingDocumentAlert]",
                               @"Title: Missing document.");
    NSString *msg =
    NSLocalizedStringFromTable(@"The most recent document is missing. Opening a new default document",
                               @"-[NSObject(UuidTokenArchiving) postMissingDocumentAlert]",
                               @"Message: Tell the user we couldn't find the last file used, so we'll make a new empty document.");
    
    NSString *cancelButtonTitle =
    NSLocalizedStringFromTable(@"Oh, well...",
                               @"-[NSObject(UuidTokenArchiving) postMissingDocumentAlert]",
                               @"Cancel: Tell the user we couldn't find the last file used, so we'll make a new empty document.");
    
    PNAlertViewHelper *helper = [[PNAlertViewHelper alloc] init];
    
    [helper setTarget: self
             selector:@selector( userAcknowledgedMissingDoc )
       forButtonTitle:cancelButtonTitle];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle: title
                                message: msg
                               delegate: helper
                      cancelButtonTitle: cancelButtonTitle
                      otherButtonTitles: nil];
    
    [alert show];


}
//-(void)installDummyDocToken
//{
//    NSDictionary *properties =
//    [NSDictionary dictionaryWithObjectsAndKeys:
//     [NSArray array],        PNMidPathComponentsKey,
//     [PNURLToken nextUuid],  PNUuidStringKey,
//     @"DummyDocToken",     PNLocalizedNameKey,
//     @"UIManagedDocument", PNDocumentClassNameKey,
//     nil];
//    
//    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
//    
//
//    NSManagedObjectContext *moc = dsnc.docSupportNet.managedObjectContext;
//    
//    PNURLToken *urlToken =
//    [PNURLToken insertCloudURLTokenWithProperties: properties
//                           inManagedObjectContext: moc];
//    
//    PNDocToken *dummy =
//    [PNDocToken docTokenWithURLToken: urlToken
//                    creationDelegate: self];
//    
//    [self.docTokensPlace addToken: dummy];
//    
//}
-(void)openMostRecentOrCreateNewDocument
{
    NSAssert( [self conformsToProtocol: @protocol(PNDocumentCreating)],
             @"Receiver does NOT conform to PNDocumentCreating.");
    
    PNURLToken *urlToken = [self unarchivedUrlTokenWithKey: (NSString*)TLDRootViewControllerMostRecentDocTokenDotUrlTokenDotUuidKey];
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    
    // if nil == urlToken, mdc returns PNUuidStatusInvalid
    enum PNUuidStatus status = [dsnc.docSupportNet statusOfUuid: urlToken.uuid];    

    switch( status ){
        case PNUuidStatusInvalid: // 0 == PNUuidStatusInvalid, nil == urlToken?
        {
            // Either we have no urlToken archived under TLDRootViewControllerMostRecentDocTokenDotUrlTokenDotUuidKey,
            // or we have a urlToken with a bogus uuid.
            
            if( nil != urlToken ){
                
                [self postMissingDocumentAlert];
                
            }
            // No break; Fall through.
        }
        case PNUuidStatusUnknown:
        {
            PNDocSupportNetController *dsnc =
            [PNDocSupportNetController sharedDocSupportNetController];

            if([dsnc canCreateDocuments]){
                // Make a new document:
                NSString *newDocClassName = NSStringFromClass([UIManagedDocument class]);
                
                [dsnc createDocWithLocalizedName: @"Untitled"
                               documentClassName: newDocClassName
                               midPathComponents: [NSArray array]
                                creationDelegate: self
                                         toPlace: self.docTokensPlace
                                          sender: self];

            }else{
                UIAlertView *alert = [[UIAlertView alloc]
                                      initWithTitle:@"NSUbiquitousBummer" 
                                      message: @"iPad doesn't create cloud docs, circa 2013 April 03."
                                      delegate: nil 
                                      cancelButtonTitle: @"Sad"
                                      otherButtonTitles: nil];
                [alert show];
                
                //[self installDummyDocToken];
                
            }
             
            break;
        }
        case PNUuidStatusUnassignedLocalURL: // no break;
        case PNUuidStatusUnassignedCloudURL:
        {
            [dsnc openDocWithUUID: urlToken.uuid
                          toPlace: self.docTokensPlace
                           sender: self];

            break;
        }
        case PNUuidStatusAssignedLocalURL:
        case PNUuidStatusAssignedCloudURL:
        {
            NSAssert( NO, @"The doc token has already been assigned.");
            break;
        }
        default:
        {
            NSAssert( NO, @"Bogus uuid status.");
            break;
            
        }
    }
}

@end
