//
//  RootViewController+ViewUpdating.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/22/13.
//
//

#import "RootViewController+ViewUpdating.h"
#import "PNDocToken.h"
#import "PNURLToken.h"

@implementation RootViewController (ViewUpdating)

-(void)appStatusDidChange: (NSNotification*)n
{
    [self updateEditAddButtonsAndTitle];
}

-(void)updateEditAddButtonsAndTitle
{
    
    UIApplication *sharedApp = [UIApplication sharedApplication];
    UIApplicationState sharedAppState = sharedApp.applicationState;
    
    if(UIApplicationStateActive != sharedAppState){
        
        [self.navigationItem setRightBarButtonItem: nil
                                          animated: NO];
        self.editButtonItem.enabled = NO;
        return;
    }
    
    // Called from:
    //  -[RootViewController setDocController:], (when the document becomes available)
    //  -[RootViewController documentStateDidChange:], (the document may become unavailable)
    //  -[RootViewController viewDidLoad], and
    //  -[RootViewController addEvent]
    
    BOOL documentAvailable = self.docToken.isAvailableValue;
    BOOL documentReadable = documentAvailable && self.docToken.documentReadable;
    if( documentReadable ){
        //[self.fetchedResultsController.fetchedObjects setArray: self.fetchedObjects];
    }
    BOOL documentEditable = documentAvailable && self.docToken.documentWriteable;
    
    // Show or hide the [+] button:
    if( documentEditable ){
        if(nil == self.navigationItem.rightBarButtonItem) {
            [self.navigationItem setRightBarButtonItem: self.addButton
                                              animated: YES];
        }
        
    }else{
        [self.navigationItem setRightBarButtonItem: nil
                                          animated: YES];
    }
    
    // Set the title, with explanation, if necessary:
    NSString *titleReGPS = nil;
    BOOL gotGPS = (nil != m_currentLocation);
    if( gotGPS ){
        titleReGPS = NSLocalizedStringFromTable(@"Locations",
                                                @"RootViewController",
                                                @"Nav bar title, GPS OK");
        
    }else{
        titleReGPS = NSLocalizedStringFromTable(@"Locations (no GPS)",
                                                @"RootViewController",
                                                @"Nav bar title, no GPS");
    }
    
    // Set the title.
    NSString *docName = self.docToken.urlToken.localizedName;
    if( nil == docName ){
        // Shouldn't happen:
        docName = @"<no document>";
    };
    
    NSString *locationsGpsDocNameTitle =
    [NSString stringWithFormat: @"%@: %@", titleReGPS, docName];
    
    self.title = locationsGpsDocNameTitle;
    
    BOOL documentNotEmpty = NO;
    if( documentReadable ){
        documentNotEmpty = ( 0 != self.fetchedResultsController.fetchedObjects.count );
    }
    
    // Two considerations determine whether we can edit events or add new events.
    //
    // [1] We need a valid non-empty document to edit.
    //     When we launch this app fresh from an iPad reboot,
    //     a noticeable delay may elapse after the view appears,
    //     but before we have a valid document ready.
    //     We can't edit or add events before then.
    //
    // [2] We need GPS available to add new events.
    //     With a valid document but no GPS, we can edit existing events,
    //     but we can't add new ones.
    //     One way to alert the user is to alter the title in the navigation bar.
    
    self.editButtonItem.enabled = documentEditable && documentNotEmpty;
    // self.editButtonItem is a property of UIViewController
    // self.navigationItem.leftBarButtonItem.enabled = documentEditable;
    
    BOOL canAddEvents = gotGPS && documentEditable  && ![self isEditing] ;
    self.addButton.enabled = canAddEvents;
    
    //[self.tableView reloadData];
    //[self scheduleReloadSection: 0];
    
}
-(void)setUIEnabled: (BOOL)flag
{
    // This method should dismiss the keyboard.
    
    BOOL okToEdit = flag && self.docToken.isAvailable;
    self.addButton.enabled =
    self.editButtonItem.enabled = okToEdit;
}

-(void)updateViewFromModel
{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        [self performFetch];
        [self updateEditAddButtonsAndTitle];

    });
}

@end
