
/*
     File: RootViewController.m
 Abstract: The table view controller responsible for displaying the list of events, supporting additional functionality:
 * Addition of new new events;
 * Deletion of existing events using UITableView's tableView:commitEditingStyle:forRowAtIndexPath: method.
 * Editing an event's name.
 
  Version: 1.1
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple
 Inc. ("Apple") in consideration of your agreement to the following
 terms, and your use, installation, modification or redistribution of
 this Apple software constitutes acceptance of these terms.  If you do
 not agree with these terms, please do not use, install, modify or
 redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software.
 Neither the name, trademarks, service marks or logos of Apple Inc. may
 be used to endorse or promote products derived from the Apple Software
 without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or
 implied, are granted by Apple herein, including but not limited to any
 patent rights that may be infringed by your derivative works or by other
 works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2010 Apple Inc. All Rights Reserved.
 
 */

#import "RootViewController.h"
#import "RootViewController+UuidTokenArchiving.h"
#import "RootViewController+ViewUpdating.h"


#import "DocumentController.h"
#import "RootViewController+Observing.h"
#import "PNDocSupportNetController.h"

#import "PNDocSupportNetController+TokenVending.h"

#import "PNDocSupportNet.h"
#import "NSManagedObject+PNPermanentIDing.h"
#import "PNDocTokenPlace.h"

#import "TagSelectionController.h"

#import "PNDocSupportNetController+GreatBugReporting.h"

// Custom View stuff:
#import "EventTableViewCell.h"

// Custom Model stuff:
#import "Event.h"
#import "Tag.h"

// PNDocSupport stuff:
#import "PNCreateDoc.h" // a transition
#import "PNOpenDoc.h"

#import "PNDocTokenPlace.h"

#import "PNURLToken.h"
#import "PNDocToken.h"
#import "PNDocToken+FileOperations.h"

#import "PNDocSupportNetController.h"

#import "PNAlertViewHelper.h"

const NSString *segueRootToTagsID = @"RootToTags";

NSString *TLDDocTokenDidBecomeAvailable = @"TLDDocTokenDidBecomeAvailable";


@implementation RootViewController

@synthesize locationManager, addButton; //  eventTableViewCell;

@synthesize organizerBarButtonItem;

@dynamic docToken;

@dynamic docTokensPlace;
@dynamic documentObservers;
@dynamic appObservers;
@dynamic storageObservers;

@synthesize activeTransition;

-(NSMutableArray*)documentObservers
{
    if( nil == m_documentObservers ){
        
        m_documentObservers = [[NSMutableArray alloc] initWithCapacity: 5];
    }
    return m_documentObservers;
}
-(NSMutableArray*)appObservers
{
    if( nil == m_appObservers ){
        
        m_appObservers = [[NSMutableArray alloc] initWithCapacity: 5];
    }
    return m_appObservers;
}
-(NSMutableArray*)storageObservers
{
    if( nil == m_storageObservers ){
        
        m_storageObservers = [[NSMutableArray alloc] initWithCapacity: 5];
    }
    return m_storageObservers;
}

-(NSString*)entityName
{
    return @"Event";
}
-(NSArray*)fetchResultsSortDescriptors
{
    // Create the sort descriptors array.
    NSSortDescriptor *dateSortDescriptor = 
    [[NSSortDescriptor alloc] initWithKey:@"creationDate" 
                                ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:dateSortDescriptor, nil];
    
    return sortDescriptors;
}
-(Class)tableViewCellClass
{
    return [EventTableViewCell class];
}
-(NSString*)tableViewCellReuseID
{
    /**
     This string returned must match with the entry in the storyboard document.
     See:
     Root View Controller - Root View Controller ... ->
     Root View Controller - Root View Controller ... ->
     Table View ->
     Event Table View Cell - RootView...->
     (Attributes Inspector)
     Style: Custom
     Identifier: "RootViewController ReuseID"
     */

    return @"RootViewController ReuseID";
}

-(PNDocTokenPlace*)docTokensPlace
{
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    PNDocSupportNet *dsn = dsnc. docSupportNet;
    
    PNDocTokenPlace *p = nil;
    NSManagedObjectContext *moc = dsn.managedObjectContext;
    if( nil == m_permID_docTokensPlace ){
        
        p = [PNDocTokenPlace insertInManagedObjectContext: moc];
        {
            p.name = @"RVC";
            p.capacityValue = 1; // One document at a time.
            p.marker = self;
            
         }
        [moc saveAnyChanges];
        m_permID_docTokensPlace = [p permanentID];
    }
    p = (PNDocTokenPlace*)[moc managedObjectForPermanentID: m_permID_docTokensPlace];
    return p;
}

-(PNDocToken*)docToken
{
    PNDocToken *result = nil;
    
    // Bogus: when we first receive the doc token, it's unavailable.
    //NSSet *available = [self.docTokensPlace availableTokens];
    
    NSSet *allTokens = [self.docTokensPlace tokens];
    NSUInteger count = allTokens.count;
    switch( count ){
        case 0:
        {
            result = nil;
            break;
        }
        case 1:
        {
            result = [allTokens anyObject];
            break;
       }
        default:
        {
            NSAssert( NO, @"Overfull");
            break;
        }
    }
    return result;
}
//-(UIManagedDocument*)document
//{
//    return self.docToken.document;
//}


//-(NSManagedObjectContext*)managedObjectContext
//{
//    return self.docToken.document.managedObjectContext;
//}
#pragma mark -
#pragma mark Location manager

/**
 Return a location manager -- create one if necessary.
 */
- (CLLocationManager *)locationManager {
	
    if ( nil == locationManager) {
        locationManager = [[CLLocationManager alloc] init];
        [locationManager setDesiredAccuracy:kCLLocationAccuracyNearestTenMeters];
        [locationManager setDelegate:self];
	}
	
	return locationManager;
}


/**
 Conditionally enable the Add button:
 If the location manager is generating updates, then enable the button;
 If the location manager is failing, then disable the button.
 */
- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation {
    
    BOOL nowHaveNoGPS = (nil == newLocation );
    BOOL didHaveNoGPS = (nil == m_currentLocation );
    m_currentLocation = newLocation;

    BOOL gpsAvailablitityDidChange = (nowHaveNoGPS != didHaveNoGPS);
    if ( gpsAvailablitityDidChange ) { // && !self.editing
		[self updateEditAddButtonsAndTitle];
	}
}

- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error {
    
    m_currentLocation = nil;
    [self updateEditAddButtonsAndTitle];
    
}



#pragma mark -
#pragma mark View lifecycle
-(void)documentDidSave: (NSNotification*)notification
{
    //[super documentDidSave: notification];
    [self updateEditAddButtonsAndTitle];
   
}
-(void)documentStateDidChange: (NSNotification*)notification
{
    // An iCloud-persistent document change its state based on ectivity on another device.
    // Editing may be disabled, conflicts may arise.
    //[super documentStateDidChange: notification];
    [self updateEditAddButtonsAndTitle];
}

- (void)viewDidLoad {
	
    [super viewDidLoad]; // super sets the doc controller for this view controller
        
    // Add a toolbar item (UIBarButtonSystemItemOrganize) to "Organize":
    // * localized name 
    // * sandbox/iCloud persistence
    // * open existing documents in sandbox/iCloud
    // * delete/evict existing documents in sandbox/iCloud
    
    // initWithBarButtonSystemItem:(UIBarButtonSystemItem)systemItem target:(id)target action:
    self.organizerBarButtonItem = 
    [[UIBarButtonItem alloc] initWithBarButtonSystemItem: UIBarButtonSystemItemOrganize
                                                  target: self 
                                                  action: @selector(showOrganizer)];
    NSArray *toolbarItemsArray = [NSArray arrayWithObject: self.organizerBarButtonItem];
    self.toolbarItems = toolbarItemsArray;
    
    [self.navigationController setToolbarHidden: NO
                                       animated: NO];

    [self observeSharedApp];
    [self observeStorage];
    
    
	self.tableView.rowHeight = 77;
	
	
	// Configure the add and edit buttons.
    self.navigationItem.leftBarButtonItem = self.editButtonItem;
    
    UIBarButtonItem *aButton = 
    [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd 
                                                  target:self 
                                                  action:@selector(addEvent)];
	self.addButton = aButton;
	//[aButton release];
	
	// self.addButton.enabled = NO;
    //self.navigationItem.rightBarButtonItem = addButton;
    [self updateEditAddButtonsAndTitle];
    
	// Start the location manager.
	[[self locationManager] startUpdatingLocation];
	        
}


-(void)dealloc
{
    [self.docTokensPlace.managedObjectContext deleteObject: self.docTokensPlace];
    m_permID_docTokensPlace = nil;
    
    self.addButton = nil;
    
}
- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
    
    self.navigationItem.rightBarButtonItem.enabled = YES;
    
    // Debugging:
    [PNDocSupportNetController copyContainerToSandbox];
    [self updateEditAddButtonsAndTitle];
}

- (void)viewDidUnload {
	// Release any properties that are loaded in viewDidLoad 
    // or can be recreated lazily.
    
    [super viewDidUnload];
    [self.navigationItem setRightBarButtonItem: nil
                                      animated: NO];

    [self ignoreSharedApp];
	self.locationManager = nil;
	self.addButton = nil;
    //[m_docTokensPlace removeToken: self.docToken];
}
-(void)showOrganizer
{
    [self performSegueWithIdentifier: @"EventsToOrganizer"
                              sender: self]; 
    
}

// Tell the system what we support
- (NSUInteger)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskAllButUpsideDown;
}

// Tell the system It should autorotate
- (BOOL) shouldAutorotate {
    return YES;
}

// Tell the system which initial orientation we want to have
//- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation {
//    return UIInterfaceOrientationLandscape;
//}

#pragma mark -

#pragma mark Table view data source methods

- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    EventTableViewCell *cell =
    [self.tableView dequeueReusableCellWithIdentifier: self.tableViewCellReuseID];
    
    if (cell == nil) {
        cell = [[EventTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                         reuseIdentifier:self.tableViewCellReuseID];
        //cell.locationLabel = [UILabel al]
        cell = (EventTableViewCell*)[self newTableViewCell];
        NSAssert( [cell isKindOfClass: [EventTableViewCell class] ], @"Bogus table view cell class");
    }
    
    [self configureCell: cell
            atIndexPath: indexPath];
    
    return cell;
}
//- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
//	// Only one section.
//    return 1;
//}
//
//- (NSInteger)tableView:(UITableView *)tableView 
// numberOfRowsInSection:(NSInteger)section 
//{
//    NSInteger numberOfRows = 0;
//    if( 0 == section ) {
//        numberOfRows = self.fetchedResultsController.fetchedObjects.count;
//    }
//    return numberOfRows;
//}

-(void)configureCell:(UITableViewCell *) cell
         atIndexPath:(NSIndexPath *)indexPath
{
    
    // A date formatter for the creation date.
    static NSDateFormatter *dateFormatter = nil;
	if (dateFormatter == nil) {
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setTimeStyle:NSDateFormatterShortStyle];
		[dateFormatter setDateStyle:NSDateFormatterShortStyle];
	}
	
	// A number formatter for the latitude and longitude.
	static NSNumberFormatter *numberFormatter = nil;
	if (numberFormatter == nil) {
		numberFormatter = [[NSNumberFormatter alloc] init];
		[numberFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
		[numberFormatter setMaximumFractionDigits:3];
	}
    
    NSUInteger count = self.fetchedResultsController.fetchedObjects.count;
    if( indexPath.row < count ) {
        
        // Get the event corresponding to the current index path and configure the table view cell.
        //Event *event = (Event *)[self.fetchedResultsController.fetchedObjects objectAtIndex:indexPath.row];
        Event *event = (Event *)[self.fetchedResultsController.fetchedObjects objectAtIndex:indexPath.row];
        
       
        if( [cell isKindOfClass:[EventTableViewCell class]] ){
            
            EventTableViewCell *etvCell = (EventTableViewCell*)cell;
            etvCell.nameField.text = event.name;
            etvCell.creationDateLabel.text = [dateFormatter stringFromDate:[event creationDate]];

            NSString *string = [NSString stringWithFormat:@"%@, %@",
                                [numberFormatter stringFromNumber:[event latitude]],
                                [numberFormatter stringFromNumber:[event longitude]]];
            etvCell.locationLabel.text = string;
            
            NSMutableArray *eventTagNames = [NSMutableArray array];
            for (Tag *tag in event.tags) {
                [eventTagNames addObject:tag.name];
            }
            
            NSString *tagsString = @"";
            if ([eventTagNames count] > 0) {
                tagsString = [eventTagNames componentsJoinedByString:@", "];
            }
            etvCell.tagsField.text = tagsString;
            
            etvCell.nameField.tag = indexPath.row;
            etvCell.tagsButton.tag = indexPath.row;
            
            etvCell.nameField.delegate = self;
            
            NSIndexPath *selPath = [self.tableView indexPathForSelectedRow];
            if( selPath.row == indexPath.row ){
                if( [self isEditing] && [etvCell.nameField canBecomeFirstResponder] ){
                    [etvCell.nameField becomeFirstResponder];
                }  
            }
            
        }else{
            cell.textLabel.text = event.name;
            cell.detailTextLabel.text = [dateFormatter stringFromDate:[event creationDate]];
        }
    }else{
        NSLog(@"exceeded range.");
    }
}

-     (BOOL)tableView:(UITableView *)tableView 
canMoveRowAtIndexPath:(NSIndexPath *)indexPath 
{
    // The table view should not be re-orderable.
    return NO;
}


#pragma mark -
#pragma mark Editing


/**
 Handle deletion of an event.
 */
- (void)tableView:(UITableView *)tableView 
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
		
		// Ensure that if the user is editing a name field then the change is committed before deleting a row -- this ensures that changes are made to the correct event object.
		[tableView endEditing:YES];
		
        // Delete the managed object at the given index path.
		NSManagedObject *eventToDelete = [self.fetchedResultsController.fetchedObjects  objectAtIndex:indexPath.row];
		[self.document.managedObjectContext deleteObject:eventToDelete];
		
		[self performSelector:@selector(updateRowTags)
                   withObject:nil 
                   afterDelay:0.0];
    }   
}


- (void)setEditing:(BOOL)editing 
          animated:(BOOL)animated 
{
    BOOL wasEditing = self.isEditing;
    
    
    // The method -setEditing:animated: is not implemented in NPTableViewController.
    // The call to "super" goes to UIViewController.
    [super setEditing:editing
             animated:animated];
	self.navigationItem.rightBarButtonItem.enabled = !editing;
    
    if( wasEditing && !editing ){
        // If we're done editing, save the document.
        //[self.docToken logRegisteredObjects];
        
        [self.docToken.managedObjectContext processPendingChanges];
        //[self.docToken saveDocumentForOverwriting];
    }

}
	

- (IBAction)editTags:(UIButton *)button
{
	//[self updateEvents];
	m_rowForSegue = button.tag;
	
	// Ensure that if the user is editing the name field then the change is 
    // committed before pushing the new view controller.
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:m_rowForSegue 
                                                inSection:0];
	EventTableViewCell *cell = 
    (EventTableViewCell *)[self.tableView 
                           cellForRowAtIndexPath:indexPath];
	[cell.nameField endEditing:YES];
								
    [self performSegueWithIdentifier: @"EventsToTags"
                              sender: self]; 
    
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue 
                 sender:(id)sender
{
    if(self == sender){
        if( [segue.identifier isEqualToString: @"EventsToTags"] ){
            
            TagSelectionController *tagSelectionController = 
            segue.destinationViewController;
            
            tagSelectionController.rootViewController = self;
            //tagSelectionController.docToken = self.docToken;
            tagSelectionController.event =
            [self.fetchedResultsController.fetchedObjects objectAtIndex:m_rowForSegue];
            
            NSSortDescriptor *sortNames =
            [NSSortDescriptor sortDescriptorWithKey: @"name"
                                          ascending: YES];
            
            [tagSelectionController setDocument: self.document
                                     entityName: @"Tag"
                                sortDescriptors:[NSArray arrayWithObject: sortNames]];
            
            return;
        }
        if( [segue.identifier isEqualToString: @"EventsToOrganizer"] ){
            
            DocumentController *organizerController = 
            segue.destinationViewController;
            
            organizerController.rootViewController = self;
            return;
        }
    }
}
- (void)updateRowTags {
	NSArray *visibleCells = [self.tableView visibleCells];	
	for (EventTableViewCell *cell in visibleCells) {
		NSInteger tag = [[self.tableView indexPathForCell:cell] row];
		cell.nameField.tag = tag;
		cell.tagsButton.tag = tag;
	}
}


#pragma mark -
#pragma mark Add an event
- (void)addEvent {
	
	// If it's not possible to get a location, then return.
	m_currentLocation = [locationManager location];
	if (!m_currentLocation) {
        [self updateEditAddButtonsAndTitle];
		return;
	}
	
	/*
	 Create a new instance of the Event entity.
	 */
    
    NSString *entityName = [self entityName];
    //NSManagedObjectContext *moc = self.docToken.document.managedObjectContext;
    
    NSManagedObjectContext *moc = self.document.managedObjectContext;
    
    NSEntityDescription *entity = [NSEntityDescription entityForName: entityName
                                              inManagedObjectContext: moc];
    
    NSAssert( entity, @"Drat. No model.");
    
	Event *event = 
    (Event *)[NSEntityDescription insertNewObjectForEntityForName: entityName
                                           inManagedObjectContext: moc];
	
	// Configure the new event with information from the location.
	CLLocationCoordinate2D coordinate = [m_currentLocation coordinate];
	[event setLatitude:[NSNumber numberWithDouble:coordinate.latitude]];
	[event setLongitude:[NSNumber numberWithDouble:coordinate.longitude]];
	
	// Should be the location's timestamp, but this will be constant for simulator.
	// [event setCreationDate:[location timestamp]];
	[event setCreationDate:[NSDate date]];
    
    /*
     * The Event entity requires the name attribute.
     * The event should have a non-nil name before the event is saved.
     * Otherwise, the document save operation will fail, but with skimpy diagnostics.
     * You might see "UIDocumentStateSavingError" ... and nothing more.
     */
    event.name = @"<event name>"; //[NSString stringWithFormat: @""];

	/*
	 Since this is a new event, and events are displayed with most recent events at the top of the list, add the new event to the beginning of the events array, then:
	 * Add a new row to the table view
	 * Scroll to make the row visible
	 * Start editing the name field
	 */
	NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 
                                                inSection:0];
	[self updateRowTags];
    
    
	[self setEditing:YES animated:YES];
	EventTableViewCell *cell =
    (EventTableViewCell *)[self.tableView cellForRowAtIndexPath:indexPath];
    
    cell.nameField.text = event.name;
    
	[cell.nameField becomeFirstResponder];
    
	/*
	 Don't save yet -- the name is not optional:
	 * The user should add a name before the event is saved.
	 * If the user doesn't add a name, it will be set to @"" when they press Done.
	 */
    
    [self updateViewFromModel];

}

#pragma mark -
#pragma mark Editing text fields

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    return YES;
}
-(void)updateEventFromTextField: (UITextField *)textField
{
    NSUInteger row = textField.tag;
    
    NSManagedObjectContext *moc = self.document.managedObjectContext;
    
    [moc performBlockAndWait: ^{
        
        Event *event =
        [self.fetchedResultsController.fetchedObjects objectAtIndex:row];
        event.name = textField.text;
        
        [moc processPendingChanges];
        
    }]; //[moc performBlockAndWait:

}
- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
{

    [self performSelectorOnMainThread: @selector(updateEventFromTextField:)
                           withObject:textField
                        waitUntilDone: NO];
    return YES;
}


- (void)textFieldDidEndEditing:(UITextField *)textField {
	// Ensure that a text field for a row for a newly-inserted object is disabled when the user finishes editing.
	textField.enabled = self.editing;
}


- (BOOL)textFieldShouldReturn:(UITextField *)textField {	
	[textField resignFirstResponder];
    [self setEditing: NO 
            animated: YES];
	return YES;	
}

#pragma mark -
#pragma mark NPPlaceDelegate methods:

-(NSString*)documentClassNameForUrlToken: (PNURLToken*)existingUrlToken
{
    // PNDocToken calls this delegate method when it opens an existing URL 
    // and must instantiate an instance of UIManagedDocument or one of its subclasses.
    //
    // PNURLToken uses the following form for its document URL:
    //
    //      file://<base>/<relative path>/<uuid>/<localized name>

    // Cases:
    
    // [1] A useful degenerate case: all your documents are instances of UIManagedDocument.
    return @"UIManagedDocument";

    // [2] All your documents are instances of a custom class.
    //return [MyDocument class];
    
    // [3] If your application uses two or more subclasses of UIManagedDocument, 
    //     consider using the relative path to distinguish their URLs.
    // return [self documentClassForRelativePath: existingUrlToken.relativePath];

    // [3.a] You could encode the appropriate class name in the relative path directly:
    //      e.g.  file://<base>/MyDocumentClass/<uuid>/Untitled_1
    //      and   file://<base>/MyOtherDocumentClass/<uuid>/Untitled_2
    
    // [4] Once you've specified a document class for your new document, 
    //     you could add the class name to the url token's userInfo for archiving.
    //     See NSObject+UuidTokenArchiving.h

    // [x] Too bad NSMetadataQuery can't deal with path extensions.
    //     This obvious, preferred option is unavailable for iCloud.
    //
    //     If your app is sandbox-only, 
    //     path extensions provide a natural way to determine 
    //     appropriate UIManagedDocument subclasses.
    //     (But... sandbox-only apps don't need PNDocSupport.)

}

-(void)didAddDocToken: (PNDocToken*)token
{
    NSAssert( (1==self.place.tokens.count),
             @"Bogus count of tokens in place (should be 1): %d",
             self.place.tokens.count);
    
    BOOL needsBonusUpdate =
    ( token.document != self.document ) ||
    (nil == self.fetchedResultsController);
    
    if( needsBonusUpdate ){
        [self ignoreDocument];{
            
            UIManagedDocument *newDoc = token.document;
            NSString *entityName = [self entityName];
            NSArray *frsds = [self fetchResultsSortDescriptors];
            
            [self setDocument: newDoc
                   entityName: entityName
              sortDescriptors: frsds];
            
        }[self observeDocument];
    }
    [self updateViewFromModel];
    
    [self setUIEnabled: token.isAvailableValue];
}
-(void)willRemoveDocToken:(PNDocToken*)aDocToken
{
    [self ignoreDocument];
    [self setUIEnabled: NO];
}
-(PNDocTokenPlace*)place
{
    return self.docTokensPlace;
}
static NSUInteger mm_docTokenDidBecomeAvailableCount = 0;

-(void)docTokenDidBecomeAvailable: (PNDocToken*)aDocToken
{
    mm_docTokenDidBecomeAvailableCount++;
    
    NSAssert( [aDocToken isAvailable], @"Doc Token is not available, should be.");
    //NSAssert( (aDocToken == self.docToken), @"Wrong doc token is available.");
    BOOL isMainThread = [[NSThread currentThread] isMainThread];
    NSAssert( isMainThread, @"-docTokenDidBecomeAvailable: called in background thread.");
    
    [aDocToken logRegisteredObjects];
    NSLog(@"aDocToken.urlToken.localFilePackageURL = %@", aDocToken.urlToken.localFilePackageURL);

    [self updateViewFromModel];
    
    [self archiveURLToken: self.docToken.urlToken
                  withKey: (NSString*)TLDRootViewControllerMostRecentDocTokenDotUrlTokenDotUuidKey];
    
    // This notification goes to DocumentController.
    NSNotificationCenter *dfc = [NSNotificationCenter defaultCenter];
    [dfc postNotificationName: TLDDocTokenDidBecomeAvailable 
                       object: self];
    
    
    
    /*
     time   count of mm_docTokenDidBecomeAvailableCount
      1.0   2
     10.0   2
     20.0   2
    // Does the second save fail every time?

    [self.docToken performSelector:@selector(saveDocumentForOverwriting)
                        withObject: nil
                        afterDelay: 20.00];
     */
 
}

-(void)userAcknowledgedDocTokenDidFail
{
    // Try recovery?
}
-(void)postAlertDidFailWithDocToken: (NSString*)name
                              error: (NSError*)error
{
    // post alert:
    NSString *title =
    NSLocalizedStringFromTable( @"An Doc Token Error Occurred",
                               @"-[RootViewController docToken:didFailWithError:]",
                               @"Title: An Error Occurred.");
    NSString *msg =
    NSLocalizedStringFromTable(@"Error: ",
                               @"-[RootViewController docToken:didFailWithError:]",
                               @"Message: Tell the user we couldn't find the last file used, so we'll make a new empty document.");
    
    msg = [msg stringByAppendingString: [error description]];
    
    NSString *cancelButtonTitle =
    NSLocalizedStringFromTable(@"Yikes!",
                               @"-[RootViewController docToken:didFailWithError:]",
                               @"Cancel: Yikes!");
    
    PNAlertViewHelper *helper = [[PNAlertViewHelper alloc] init];
    
    [helper setTarget: self
             selector:@selector( userAcknowledgedDocTokenDidFail )
       forButtonTitle:cancelButtonTitle];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle: title
                          message: msg
                          delegate: helper
                          cancelButtonTitle: cancelButtonTitle
                          otherButtonTitles: nil];
    
    [alert show];
}
-(void)docToken: (PNDocToken*)aDocToken
didFailWithError: (NSError *)error
{
//    Bogus: typically at this stage, self.docToken == nil.
//    NSAssert( (aDocToken == self.docToken),
//             @"Bogus doc token");
    BOOL isMainThread = [[NSThread currentThread] isMainThread];
    NSAssert( isMainThread, @"-docToken:didFailWithError: called in background thread.");

    [aDocToken logRegisteredObjects];
    
    NSLog(@"mm_docTokenDidBecomeAvailableCount = %d", mm_docTokenDidBecomeAvailableCount);

    NSLog(@"aDocToken.urlToken.localFilePackageURL = %@", aDocToken.urlToken.localFilePackageURL);
    

    [self postAlertDidFailWithDocToken: aDocToken.urlToken.localizedName
                                 error: error];
}

#pragma mark PNTransitionDelegate methods:

-(void)didFireTransition: (PNTransition*)transition{ self.activeTransition = nil; }
-(BOOL)shouldAttemptToRollbackTransition: (PNTransition*)transition
{
    // The initial version of the framework implements exactly one very simple rollback case:
    
    BOOL isMine =  ( transition == self.activeTransition );
    BOOL isNewDoc = [transition isKindOfClass:[PNCreateDoc class]];
    
    return (isMine && isNewDoc);
}
-(void)userAcknowledgedTransitionDidFail
{
    // Try recovery?
    self.activeTransition = nil;
    
    abort();

}

-(void)postAlertDidFailWithTransition: (NSString*)name
                                error: (NSError*)error
{
    // post alert:
    NSString *title =
    NSLocalizedStringFromTable( @"A Transition Error Occurred",
                               @"-[RootViewController docToken:didFailWithError:]",
                               @"Title: An Error Occurred.");
 //   NSString *msg =
//    NSLocalizedStringFromTable(@"Error: ",
//                               @"-[RootViewController docToken:didFailWithError:]",
//                               @"Message: Tell the user we couldn't find the last file used, so we'll make a new empty document.");
    
    // Raw, unlocalized string for alert view 
    NSString *msg = [NSString stringWithFormat:
           
           @"Transition = %@, ",
           NSStringFromClass([self.activeTransition class]) ];
    
    msg = [msg stringByAppendingFormat:
           
           @"Error = %@,",
           [error description] ];

    NSString *cancelButtonTitle =
    NSLocalizedStringFromTable(@"Yikes!",
                               @"-[RootViewController docToken:didFailWithError:]",
                               @"Cancel: Yikes!");
    
    PNAlertViewHelper *helper = [[PNAlertViewHelper alloc] init];
    
    [helper setTarget: self
             selector:@selector( userAcknowledgedTransitionDidFail )
       forButtonTitle:cancelButtonTitle];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle: title
                          message: msg
                          delegate: helper
                          cancelButtonTitle: cancelButtonTitle
                          otherButtonTitles: nil];
    
    [alert show];
}
-(void)didFailTransition:(PNTransition *)transition
  rolledBackSuccessfully:(BOOL)success
{
    [self postAlertDidFailWithTransition: transition.name
                                   error: transition.error];
    
}


-(void)didChangeMocOfDoc:(UIManagedDocument *)document
{
    if( document == self.document ){
        [self updateViewFromModel];
    }
}
-(void)initializeDocumentObjectGraphInDocToken:(PNDocToken *)docToken 
{
    UIManagedDocument *document = docToken.document;
    
    // build the initial graph of managed objects in a newly instantiated document
    
    NSString *entityName = [self entityName];
    //NSManagedObjectContext *moc = self.docToken.document.managedObjectContext;
    
    NSManagedObjectContext *moc = document.managedObjectContext;
    
//    NSEntityDescription *entity = [NSEntityDescription entityForName: entityName
//                                              inManagedObjectContext: moc];
//    
//    NSAssert( entity, @"Drat. No model.");
    
	Event *event = 
    (Event *)[NSEntityDescription insertNewObjectForEntityForName: entityName
                                           inManagedObjectContext: moc];
	NSAssert( event,@"Drat! nil event");
    
	// Fake the location--we may not have a location manager available:
    [event setLatitude: [NSNumber numberWithDouble: +42.0]];
	[event setLongitude:[NSNumber numberWithDouble: -42.0]];
	
	// Should be the location's timestamp, but this will be constant for simulator.
	// [event setCreationDate:[location timestamp]];
	[event setCreationDate:[NSDate date]];
    
    /*
     * The Event entity requires the name attribute.
     * The event should have a non-nil name before the event is saved.
     * Otherwise, the document save operation will fail, but with skimpy diagnostics.
     * You might see "UIDocumentStateSavingError" ... and nothing more.
     */
    UIDevice *device = [UIDevice currentDevice];
    NSString *devModel = device.model; // may get @"iPad Simulator"
    if( nil == devModel ){
        devModel = @"Simulator?!"; // never happens, defensive programming
    }
    event.name = [NSString stringWithFormat: @"%@: initial event",
                  devModel];

}


-(void)willFinalizeDocToken: (PNDocToken*)token
{
    // After calling this method, but before calling -didFinalizeDocument
    // the sharedPNDocTokensController will call -removeToken
}
-(void)didFinalizeDocToken: (PNDocToken*)token
{
    [self updateEditAddButtonsAndTitle];
}

@end


