
/*
     File: TagSelectionController.m
 Abstract: The table view controller responsible for displaying all available tags.
 The controller is also given an Event object. Row for tags related to the event display a checkmark.  If the user taps a row, the corresponding tag is added to or removed from the event's tags relationship as appropriate.
 
  Version: 1.1
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple
 Inc. ("Apple") in consideration of your agreement to the following
 terms, and your use, installation, modification or redistribution of
 this Apple software constitutes acceptance of these terms.  If you do
 not agree with these terms, please do not use, install, modify or
 redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software.
 Neither the name, trademarks, service marks or logos of Apple Inc. may
 be used to endorse or promote products derived from the Apple Software
 without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or
 implied, are granted by Apple herein, including but not limited to any
 patent rights that may be infringed by your derivative works or by other
 works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2010 Apple Inc. All Rights Reserved.
 
 */


#import "TagSelectionController.h"
#import "PNDocumentEntityTVController.h"

#import "Event.h"
#import "Tag.h"
#import "EditableTableViewCell.h"

#import "PNDocToken.h"
#import "PNDocToken+FileOperations.h"

#import "RootViewController.h"
#import "PNDocumentEntityTVController.h"

@implementation TagSelectionController

@synthesize headerView = m_headerView, headerLabel = m_headerLabel;
//@synthesize initialEditTagCount;

//@synthesize docToken;
@synthesize rootViewController;

@dynamic document;
@dynamic docToken;

@dynamic newInsertionCell, event;
-(PNDocToken*)docToken
{
    return self.rootViewController.docToken;
}
-(Class)tableViewCellClass
{
    return [EditableTableViewCell class];
}

-(NSString*)tableViewCellReuseID
{
    /**
     This string returned must match with the entry in the storyboard document.
     See:
     Tag Selection Controller Scene ->
     Tag Selection Controller ->
     Table View ->
     Editable Table View Cell ->
        (Attributes Inspector)
        Style: Custom
        Identifier: "EditableTableViewCell ReuseID"
     */
    return @"EditableTableViewCell ReuseID";
}

-(NSString*)insertionCellReuseID
{
    return @"InsertionCell ReuseID";
}

-(UITableViewCell*)newTableViewCell
{
    EditableTableViewCell *newCell = (EditableTableViewCell*)[super newTableViewCell];
    
    CGRect r = newCell.bounds;
    NSLog( @"newCell.bounds = %@", NSStringFromCGRect(r) );
    return newCell;
}

-(UITableViewCell*)newInsertionCell
{
    NSString *insertionCellReuseID = [self insertionCellReuseID];
    
    UITableViewCell *newCell = 
    [[UITableViewCell alloc] initWithStyle: UITableViewCellStyleDefault 
                           reuseIdentifier: insertionCellReuseID];
    
    // These inherited ivars may obscure your custom ivars:
    newCell.textLabel.text = @"Add tag";
    newCell.textLabel.hidden = NO;
    newCell.detailTextLabel.hidden = YES;

    return newCell;
}
+(NSString*)entityName
{
    return @"Tag";
}
+(NSArray*)fetchResultsSortDescriptors
{
    // Create the sort descriptors array.
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"name"
																   ascending:YES];
	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    
    return sortDescriptors;
}

-(void)setEvent:(Event *)newEvent
{
    m_event = newEvent;
    
    if( nil != self.tableView ){
        
        
        NSString *eventName = self.event.name;
        if (![eventName length] > 0) {
            eventName = @"Unnamed event";
        }
        self.headerLabel.text = [NSString stringWithFormat:
                            @"Tags for \"%@\" are indicated by a check mark.",
                            eventName];
    }
    
}

-(Event*)event
{
    return m_event;
}
/*
-(void)updateModelFromView
{
    for( Tag *tag in self.fetchedResultsController.fetchedObjects){
        NSUInteger row = [self.fetchedResultsController.fetchedObjects indexOfObject: tag];
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:row
                                                    inSection: 0];
        EditableTableViewCell *cell = (EditableTableViewCell*)[self.tableView cellForRowAtIndexPath: indexPath];
        
        // Attributes:
        tag.name = cell.textField.text;
        
        // Relationships:
        if( UITableViewCellAccessoryCheckmark == cell.accessoryType ){
            [self.event addTagsObject: tag];
        }else{
            [self.event removeTagsObject: tag];
        }
        
    }
    //[self.docToken saveDocumentForOverwriting];
}
 */
- (void)viewDidLoad {
	
	// Configure the table view and controller.
	[super viewDidLoad];	
	self.tableView.allowsSelectionDuringEditing = YES;
	
	// Set up the navigation bar.
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
	self.navigationItem.title = @"Tags";
	
    // Header view should come from storyboard:    
    CGRect headerLabelFrame = CGRectInset((self.headerView.frame), 3.0, 3.0 );
    self.headerLabel = [[UILabel alloc] initWithFrame:headerLabelFrame];

    [self.headerView addSubview: self.headerLabel];
    
    self.tableView.tableHeaderView = self.headerView;
    
    //self.initialEditTagCount = NSNotFound;
}
- (void)viewDidAppear:(BOOL)animated {
	
	[super viewDidAppear:animated];
	if ( 0 == self.fetchedResultsController.fetchedObjects.count ) {
		// There are no tags, and presumably the user wants to add one.
		// Create and edit one straight away.
		[self setEditing:YES 
                animated:NO];
		[self insertTagAnimated:NO];
	}
}
-(void)viewWillDisappear:(BOOL)animated
{
    // It's probably not necessary,
    // but we'll make sure to update the model
    // before the view disappears.
        
    [super viewWillDisappear: animated];
}


#pragma mark -
#pragma mark Table view data source

//- (NSInteger)tableView:(UITableView *)tableView 
// numberOfRowsInSection:(NSInteger)section {
//	// Return the number of tags in the array, adding one if editing (for the Add Tag row).
//	NSUInteger count = [self.currentObjectsArray count];
//	if (self.editing) {
//		count++;
//	}
//    return count;
//}
-(void)configureCell:(UITableViewCell *) cell
         atIndexPath:(NSIndexPath *)indexPath
{
    cell.textLabel.hidden = YES;
    cell.accessoryType = UITableViewCellAccessoryNone;
    
    NSUInteger row = indexPath.row;
    
    EditableTableViewCell *etvCell = nil;
    if( [cell isKindOfClass:[EditableTableViewCell class]]){
        
        etvCell = (EditableTableViewCell*)cell;
        
        // Set the tag on the text field to the row number so it can be identified later if edited.
        etvCell.textField.tag = row;
        etvCell.textField.delegate = self;
        
        [etvCell configureTextField];
        
        Tag *tag = [self.fetchedResultsController.fetchedObjects objectAtIndex:row];
        // If the tag at this row in the tags array is related to the event, display a checkmark, otherwise remove any checkmark that might have been present.
        etvCell.textField.text = tag.name;
        
        if ([self.event.tags containsObject:tag]) {
            etvCell.accessoryType = UITableViewCellAccessoryCheckmark;
        }else{
            etvCell.accessoryType = UITableViewCellAccessoryNone;
        }
        
        if( self.isEditing ){
            NSArray *indexPathsForSelectedRows = [self.tableView indexPathsForSelectedRows];
            if( [indexPathsForSelectedRows containsObject: indexPath] ){
                
                etvCell.textField.delegate = self;
                [etvCell.textField becomeFirstResponder];
            }
        }
    }
}

- (NSInteger)tableView:(UITableView *)tableView 
 numberOfRowsInSection:(NSInteger)section {
    
    if( nil == self.event ) return 0;
    
	// Return the number of tags in the array, adding one if editing (for the Add Tag row).
	NSUInteger count = self.fetchedResultsController.fetchedObjects.count;
	if ( self.isEditing ) {
		count++;
	}
    return count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
    
    UITableViewCell *resultCell = nil;
	NSUInteger row = indexPath.row;
	
	if (row == self.fetchedResultsController.fetchedObjects.count) {
        
		// This is the insertion cell.
		NSString *insertionCellReuseID = [self insertionCellReuseID];
		
		resultCell = [tableView dequeueReusableCellWithIdentifier:insertionCellReuseID];
		if (resultCell == nil) {		
			resultCell = [self newInsertionCell];
		}
        
        // Configure the (+) cell:
        resultCell.textLabel.text = @"Add Tag";
        resultCell.textLabel.hidden = NO;
        resultCell.accessoryType = UITableViewCellAccessoryNone;

	}else{
        NSString *tableViewCellReuseID = [self tableViewCellReuseID];
		
        resultCell = [tableView dequeueReusableCellWithIdentifier:tableViewCellReuseID];
        
        if( nil == resultCell ){
            resultCell = [self newTableViewCell];
        }
        
        [self configureCell: resultCell
                atIndexPath: indexPath];
    }
	
    return resultCell;
 }


#pragma mark -
#pragma mark Editing rows

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView 
           editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	// The add row gets an insertion marker, the others a delete marker.
	if (indexPath.row == self.fetchedResultsController.fetchedObjects.count) {
		return UITableViewCellEditingStyleInsert;
	}
    return UITableViewCellEditingStyleDelete;
}


- (void)setEditing:(BOOL)editing 
          animated:(BOOL)animated 
{
//    if( !(self.docToken.documentWriteable) ) return;
    
    [super setEditing:editing 
             animated:animated];
//	
//    // If commencing editing, hold off on save operations until we end editing.
    
	// Don't show the Back button while editing.
	[self.navigationItem setHidesBackButton:editing 
                                   animated:YES];
    
	
//	[self.tableView beginUpdates];{
//        
//        NSUInteger count = self.fetchedObjects.count;
//        
//        NSArray *tagInsertIndexPath = [NSArray arrayWithObject:[NSIndexPath indexPathForRow:count inSection:0]];
//        
//        // Add or remove the Add row as appropriate.
//        UITableViewRowAnimation animationStyle = UITableViewRowAnimationNone;
//        if (editing) {
//            if (animated) {
//                animationStyle = UITableViewRowAnimationFade;
//            }
//            [self.tableView insertRowsAtIndexPaths:tagInsertIndexPath 
//                                  withRowAnimation:animationStyle];
//        }else{
//            [self.tableView deleteRowsAtIndexPaths:tagInsertIndexPath 
//                                  withRowAnimation:UITableViewRowAnimationFade];
//        }
//
//    }[self.tableView endUpdates];
	
//	// If editing is finished, save the document.
	if (!editing) {
        
        NSIndexPath *indexPath = [self.tableView indexPathForSelectedRow];
        
        UITableViewCell *cell =
        [self.tableView cellForRowAtIndexPath:indexPath];
        
        NSLog( @"cell %@", cell);
        //NSUInteger row = indexPath.row;
       //Tag *tag = [self.fetchedResultsController.fetchedObjects objectAtIndex:row];
        //tag.name = cell.text;
        
        //[self.docToken saveDocumentForOverwriting];
	}
    
    [self.tableView reloadData];
}
-  (void)tableView:(UITableView *)tableView
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
 forRowAtIndexPath:(NSIndexPath *)indexPath 
{
    
//	NSManagedObjectContext *context = self.event.managedObjectContext;
	
//    if (editingStyle == UITableViewCellEditingStyleDelete) {
//        
//		// Delete the tag.
//		
//		// Ensure the cell is not being edited, otherwise the callback in textFieldShouldEndEditing: may look for a non-existent row.
//		EditableTableViewCell *cell = (EditableTableViewCell *)[self.tableView cellForRowAtIndexPath:indexPath];
//		[cell.textField resignFirstResponder];
//        
//		
//		// Find the tag to delete.
//		Tag *tag = [self.fetchedResultsController.fetchedObjects objectAtIndex:indexPath.row];
//        
//		// Delete the tag from the context.  Because the relationship between Tag and Event is defined in both directions, and the delete rule is nullify, Core Data automatically removes the tag from any relationships in which it is present.
//		[context deleteObject:tag];
//        
//		// Remove the tag from the tags array and the corresponding row from the table view.
//		[self.fetchedResultsController.fetchedObjects  removeObject:tag];
//		
//        NSArray *indexPathArray = [NSArray arrayWithObject:indexPath];
//		[tableView deleteRowsAtIndexPaths: indexPathArray
//                         withRowAnimation:UITableViewRowAnimationFade];
//		
//    }
	
	if (editingStyle == UITableViewCellEditingStyleInsert) {
		[self insertTagAnimated:YES];
	}
}

- (void)insertTagAnimated:(BOOL)animated {
	
	// Create a new instance of Tag, insert it into the tags array, and add a corresponding new row to the table view.
	
    NSString *entityName = [[self class] entityName];
    NSManagedObjectContext *moc = self.document.managedObjectContext;
    
    NSEntityDescription *entity = [NSEntityDescription entityForName: entityName
                                              inManagedObjectContext: moc];
    
    NSAssert( entity, @"Drat. No model.");

	Tag *tag = [NSEntityDescription insertNewObjectForEntityForName: entityName
                                             inManagedObjectContext: moc];
	tag.name = @"";
    
	// Presumably the tag was added for the current event, so relate it to the event.
	[self.event addTagsObject:tag];
	
	// Add the new tag to the tags array and to the table view.	
//	[self.fetchedResultsController.fetchedObjects  addObject:tag];
//	
    NSUInteger newTagRow = [self.fetchedResultsController.fetchedObjects
                            indexOfObject: tag];
    
	NSIndexPath *indexPath = [NSIndexPath indexPathForRow:newTagRow
                                                inSection:0];	
	// Start editing the tag's name.
	EditableTableViewCell *cell = (EditableTableViewCell *)[self.tableView cellForRowAtIndexPath:indexPath];
    cell.textField.delegate = self;
    cell.textField.text = tag.name;
	[cell.textField becomeFirstResponder];
    
}


#pragma mark -
#pragma mark Row selection

- (NSIndexPath *)tableView:(UITableView *)tableView 
  willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
	// Disable selection of the Add row.
	if (indexPath.row == self.fetchedResultsController.fetchedObjects.count) {
		return nil;
	}
	return indexPath;
}


- (void)      tableView:(UITableView *)tableView 
didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	
	/*
	 Find the tag at the row index in the tags array, then
	 * If the event currently has this tag, remove the tag;
	 * If the event doesn't have this tag, add the tag.
     
	 (The Add row is not selectable.)
	 */
	Tag *tag = [self.fetchedResultsController.fetchedObjects  objectAtIndex:indexPath.row];
    NSAssert( tag, @"bogus tag at indexPath = %@", [indexPath description] );
    
	UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    NSAssert( cell, @"bogus cell at indexPath = %@", [indexPath description] );
	
	if ([self.event.tags containsObject:tag]) {
		cell.accessoryType = UITableViewCellAccessoryNone;
		[self.event removeTagsObject:tag];
	}else{
		cell.accessoryType = UITableViewCellAccessoryCheckmark;
		[self.event addTagsObject:tag];
	}
    
    //[self.docToken saveDocumentForOverwriting];
}


#pragma mark -
#pragma mark Editing text fields

-(void)updateEventFromTextField: (UITextField *)textField
{
    NSUInteger row = textField.tag;
    
    NSManagedObjectContext *moc = self.document.managedObjectContext;
    
    [moc performBlockAndWait: ^{
        
        Tag *tag =
        [self.fetchedResultsController.fetchedObjects objectAtIndex:row];
        tag.name = textField.text;
        
        [moc processPendingChanges];
        
    }]; //[moc performBlockAndWait:
    
}
- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
{
    
    [self performSelectorOnMainThread: @selector(updateEventFromTextField:)
                           withObject:textField
                        waitUntilDone: NO];
    return YES;
}


- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    return YES;
}
- (BOOL)textFieldShouldReturn:(UITextField *)textField {	
	[textField resignFirstResponder];
	return YES;	
}


@end


