//
//  TLDemoAppDelegate+StorageChoiceManaging.h
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/29/13.
//
//

#import "TLDemoAppDelegate.h"


/**
	See:  http://developer.apple.com/library/mac/#documentation/General/Conceptual/iCloudDesignGuide/Chapters/iCloudFundametals.html, especially Listing 1-4: Inviting the user to use iCloud.
 
 */
typedef enum TLDemoUserChoiceOfStorageCase
{
	TLDemoUserChoiceOfStorageCaseNeverAsked = 0, /**< The user has not yet been asked about iCloud storage; perhaps the app has not yet run with iCloud available. */
	TLDemoUserChoiceOfStorageCaseChoseLocalForever, /**< The user has chosen to use local storage permanently. */
	TLDemoUserChoiceOfStorageCaseChoseCloudForever /**< The user has chosen to use iCloud storage permanently. */
} TLDemoUserChoiceOfStorageCase;


/**
 This category of TLDemoAppDelegate supports the behavior described in:
 http://developer.apple.com/library/mac/#documentation/General/Conceptual/iCloudDesignGuide/Chapters/iCloudFundametals.html
 
 Prepare Your App to Use iCloud
 
 When a user launches your iCloud-enabled app for the first time, invite them to use iCloud.
 The choice should be all-or-none. In particular, it is best practice to:
 
 Use iCloud exclusively or use local storage exclusively; in other words, do not attempt to mirror documents between your ubiquity container and your sandbox container.
 Never prompt the user again about whether they want to use iCloud vs. local storage, unless they delete and reinstall your app.
 
 
 */
@interface TLDemoAppDelegate (StorageChoiceManaging)

// These properties support the behavior specified by iCloud Design Guide:

/**
 Accesses the user's storage choice in user defaults under the key TLDemo_UserChoiceOfStorageCaseKey. If the specified key does not exist, this method returns 0 (namely, TLDemoUserChoiceOfStorageCaseNeverAsked). Such is the expected case on the application's first launch.
 */
@property (readonly, assign) enum TLDemoUserChoiceOfStorageCase storageChoice;

/**
 If TLDemoUserChoiceOfStorageCaseNeverAsked == self.storageChoice, this method calls -askUserAboutUpgradingToCloudStorage, which posts an alert view. The callbacks set self.storageChoice to TLDemoUserChoiceOfStorageCaseChoseLocalForever or TLDemoUserChoiceOfStorageCaseChoseCloudForever, and then call -manageStorageChoice again. See also -manageStorageChoiceWithReminder, for early development.
 */
-(void)manageStorageChoice;


/**
 Like -manageStorageChoice, but reminds the developer using the simulator that the simulator can't access iCloud.
 */
-(void)manageStorageChoiceWithReminder;
@end


