//
//  TLDemoAppDelegate+StorageChoiceManaging.m
//  TaggedLocationsDemo
//
//  Created by DevGuy on 1/29/13.
//
//

#import "TLDemoAppDelegate+StorageChoiceManaging.h"

#import "PNAlertViewHelper.h"

#import "PNURLToken.h"
#import "NSURL+NPAssisting.h"

#import "PNDocSupportNetController.h"
#import "PNDocSupportNetController+StorageControlling.h"


#import "RootViewController+UuidTokenArchiving.h"
@interface TLDemoAppDelegate ()


@property (readwrite, assign) enum TLDemoUserChoiceOfStorageCase storageChoice;

@end




static NSString *TLDemo_UserChoiceOfStorageCaseKey =
@"TLDemo User Choice Of Storage Case";

@implementation TLDemoAppDelegate (StorageChoiceManaging)


@dynamic storageChoice;

-(enum TLDemoUserChoiceOfStorageCase)storageChoice
{
    enum TLDemoUserChoiceOfStorageCase result =
    [self.userDefaults integerForKey: TLDemo_UserChoiceOfStorageCaseKey];
    // "If the specified key does not exist,
    // this method returns 0 (namely, TLDemoUserChoiceOfStorageCaseNeverAsked).",
    return  result;
}

-(void)setStorageChoice:(enum TLDemoUserChoiceOfStorageCase)storageChoice
{
    switch( storageChoice ){
        case TLDemoUserChoiceOfStorageCaseChoseLocalForever:
        case TLDemoUserChoiceOfStorageCaseChoseCloudForever:
        {
            [self.userDefaults setInteger: storageChoice
                                   forKey: TLDemo_UserChoiceOfStorageCaseKey];
            break;
        }
        case TLDemoUserChoiceOfStorageCaseNeverAsked:
        default:
        {
            NSAssert( NO, @"Bogus value of storageChoice = %d",
                     storageChoice);
            break;
        }
    }
}


-(NSString*)defaultUbiquityContainerIdentifier
{
    NSBundle *mainBundle = [NSBundle mainBundle];
    NSArray *x =
    [mainBundle objectForInfoDictionaryKey:@"com.apple.developer.ubiquity-container-identifiers"];
    
    if( 0 == x.count ) return nil;
    return [x lastObject];
}

#pragma mark Methods to update user's storage choice as required.


-(void)didChooseLocalStorageForever
{
    self.storageChoice = TLDemoUserChoiceOfStorageCaseChoseLocalForever;
    [self manageStorageChoice]; //
}
-(void)didChooseCloudStorageForever
{
    self.storageChoice = TLDemoUserChoiceOfStorageCaseChoseCloudForever;
    [self manageStorageChoice];
}

-(void)askUserAboutUpgradingToCloudStorage

{
    NSString *title =
    NSLocalizedString( @"Choose Storage Option",
                      @"-[TLDemoAppDelegate askUserAboutUpgradingToCloud] Alert Title");
    
    NSString *message =
    NSLocalizedString( @"Should documents be stored in iCloud and available on all your devices?",
                      @"-[TLDemoAppDelegate askUserAboutUpgradingToCloud] Alert Message");
    
    NSString *cancelButtonTitle =
    NSLocalizedString( @"Local Only",
                      @"-[TLDemoAppDelegate askUserAboutUpgradingToCloud] Alert Cancel Button Title");
    
    NSString *otherButtonTitle =
    NSLocalizedString( @"Use iCloud",
                      @"-[TLDemoAppDelegate askUserAboutUpgradingToCloud] Alert Other Button Title");
    
    PNAlertViewHelper *helper = [[PNAlertViewHelper alloc] init];
    [helper setTarget: self
             selector: @selector(didChooseLocalStorageForever)
       forButtonTitle: cancelButtonTitle];
    
    [helper setTarget: self
             selector: @selector(didChooseCloudStorageForever)
       forButtonTitle: otherButtonTitle];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle: title
                          message: message
                          delegate: helper
                          cancelButtonTitle: cancelButtonTitle
                          otherButtonTitles: otherButtonTitle, nil];
    [alert show];
}

-(void)manageStorageChoice
{
    
    PNDocSupportNetController *dsnc = [PNDocSupportNetController sharedDocSupportNetController];
    
    enum TLDemoUserChoiceOfStorageCase choice = self.storageChoice;
    
    switch( choice ){
        case TLDemoUserChoiceOfStorageCaseNeverAsked:
        {
            id currentiCloudToken = [[NSFileManager defaultManager] ubiquityIdentityToken];
            if(nil ==currentiCloudToken){
                [dsnc useLocalStorageByDefault];
            }else{
                [self askUserAboutUpgradingToCloudStorage];
            }
            break;
        }
        case TLDemoUserChoiceOfStorageCaseChoseLocalForever:
        {
            [dsnc useLocalStorageForever]; // returns immediately
            
            break;
        }
        case TLDemoUserChoiceOfStorageCaseChoseCloudForever:
        {
            
            //[dsnc useCloudStorageWithCallback: nil];
            [dsnc useUbiquityContainerIdentifier: [self defaultUbiquityContainerIdentifier]];
            // starts a metadata query and
            // observes NSUbiquityIdentityDidChangeNotification
            
            break;
        }

        default:
        {
            NSAssert( NO, @"-manageStorageChoice found %d", choice);
            break;
        }
    }
}

/**
 Early on, I found it helpful to remind myself that the simulator can't get to the cloud.
 */
-(void)remindDevelopingOnSimulator
{
    
    NSString *title =
    NSLocalizedString( @"Running on Simulator",
                      @"-[TLDemoAppDelegate remindDevelopingOnSimulator] Alert Title");
    
    NSString *message =
    NSLocalizedString( @"You can't upgrade to Cloud storage on the simulator.",
                      @"-[TLDemoAppDelegate remindDevelopingOnSimulator] Alert Message");
    
    NSString *cancelButtonTitle =
    NSLocalizedString( @"Local Only",
                      @"-[TLDemoAppDelegate remindDevelopingOnSimulator] Alert Cancel Button Title");
    
    
    PNAlertViewHelper *helper = [[PNAlertViewHelper alloc] init];
    
    [helper setTarget: self
             selector: @selector(didChooseLocalStorageForever)
       forButtonTitle: cancelButtonTitle];
    
    UIAlertView *alert = [[UIAlertView alloc]
                          initWithTitle: title
                          message: message
                          delegate: helper
                          cancelButtonTitle: cancelButtonTitle
                          otherButtonTitles: nil];
    [alert show];
}

-(void)manageStorageChoiceWithReminder
{
    /**
     This method is for development only.
     It reminds the developer that the simulator does not support cloud storage.
     For deployment, call -manageStorageChoice instead.
     */
    
    NSURL *x = [PNURLToken containerUrl];
    enum PNUrlBaseCase baseCase = [x baseCase];
    
    // A passage for developers:
    switch( baseCase ){
        case PNUrlBaseCaseSimulator:
        {
            [self remindDevelopingOnSimulator];
            // also calls -manageStorageChoice
            break;
        }
        case PNUrlBaseCaseLocal:
        case PNUrlBaseCaseCloud:
        {
            [self manageStorageChoice];
            break;
        }
        case PNUrlBaseCaseInvalid:
        default:
        {
            NSAssert( NO, @"-manageStorageChoiceWithReminder: bogus baseCase %d", baseCase);
            break;
        }
    }
    
}

@end
